@@ -23,12 +23,10 @@
 
 #include <string.h>
 
-#include <pthread.h>
+#include <libgnomeui/libgnomeui.h>
 
-#include <gnome.h>
-
-#include <pan/base/article-thread.h>
 #include <pan/base/debug.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/status-item.h>
 
@@ -46,6 +44,7 @@
 #include <pan/filters/filter-ui.h>
 #include <pan/filters/filter-manager.h>
 
+#include <pan/article-list-toolbar-mediator.h> /* just for the util */
 #include <pan/article-toolbar.h>
 #include <pan/globals.h>
 #include <pan/gui.h>
@@ -61,6 +60,8 @@
 #include <xpm/new_read.xpm>
 #include <xpm/old_unread.xpm>
 #include <xpm/watched.xpm>
+#include <xpm/by_me.xpm>
+#include <xpm/by_others.xpm>
 
 static GtkWidget * show_matches_button = NULL;
 static GtkWidget * show_subthread_button = NULL;
@@ -82,6 +83,8 @@ static GtkWidget * show_queued_button = NULL;
 static GtkWidget * show_idle_button = NULL;
 static GtkWidget * show_cached_button = NULL;
 static GtkWidget * show_non_cached_button = NULL;
+static GtkWidget * show_by_me_button = NULL;
+static GtkWidget * show_by_others_button = NULL;
 static GtkWidget * _custom_filters_menu = NULL;
 static GtkWidget * _no_customs_button = NULL;
 
@@ -94,6 +97,7 @@ static Filter * _scratch_filter = NULL;
 static Filter * _filter = NULL;
 static gchar * _filter_phrase_str = NULL;
 static FilterPhraseState _filter_phrase_state = FILTER_PHRASE_SUBJECT;
+static gint _prev_filter_phrase_state = -1;
 static FilterShow _filter_show = FILTER_SHOW_MATCHES_AND_REFERENCES;
 
 static void fire_filter_changed (void);
@@ -125,174 +129,6 @@ get_on_the_fly_name (void)
 **** 
 ***/
 
-static Filter*
-create_filter_from_ulong (gulong               l,
-                          FilterPhraseState    phrase_state,
-                          const gchar        * phrase_str)
-{
-	Filter * retval;
-	FilterAggregate * top;
-	const gboolean show_new = (l & STATE_FILTER_NEW) != 0;
-	const gboolean show_old = (l & STATE_FILTER_OLD) != 0;
-	const gboolean show_unread = (l & STATE_FILTER_UNREAD) != 0;
-	const gboolean show_read = (l & STATE_FILTER_READ) != 0;
-	const gboolean show_bin = (l & STATE_FILTER_COMPLETE_BINARIES) != 0;
-	const gboolean show_inc_bin = (l & STATE_FILTER_INCOMPLETE_BINARIES) != 0;
-	const gboolean show_nonbinary = (l & STATE_FILTER_NONBINARIES) != 0;
-	const gboolean show_watched = (l & STATE_FILTER_WATCHED) != 0;
-	const gboolean show_ignored = (l & STATE_FILTER_IGNORED) != 0;
-	const gboolean show_normal = (l & STATE_FILTER_NORMAL_RANK) != 0;
-	const gboolean show_saved = (l & STATE_FILTER_SAVED) != 0;
-	const gboolean show_idle = (l & STATE_FILTER_IDLE) != 0;
-	const gboolean show_queued = (l & STATE_FILTER_QUEUED) != 0;
-	const gboolean show_cached = (l & STATE_FILTER_CACHED) != 0;
-	const gboolean show_non_cached = (l & STATE_FILTER_NOT_CACHED) != 0;
-	gint n;
-	debug_enter ("create_filter_from_ulong");
-
-	/* top */
-	top = FILTER_AGGREGATE(filter_aggregate_new ());
-	filter_aggregate_set_type (top, AGGREGATE_TYPE_AND);
-
-	/* new/old */
-	n = 0;
-	if (show_new) ++n;
-	if (show_old) ++n;
-	if (n == 1) {
-		Filter * f = filter_new_article_new ();
-		if (show_old)
-			filter_negate (f);
-		filter_aggregate_add (top, &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-	}
-
-	/* read/unread */
-	n = 0;
-	if (show_read) ++n;
-	if (show_unread) ++n;
-	if (n == 1) {
-		Filter * f = filter_read_new ();
-		if (show_unread)
-			filter_negate (f);
-		filter_aggregate_add (top, &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-	}
-
-	/* attachments */
-	n = 0;
-	if (show_bin) ++n;
-	if (show_inc_bin) ++n;
-	if (show_nonbinary) ++n;
-	if (n==1 || n==2) {
-		Filter * f = filter_binary_new ();
-		if (n==1) {
-			if (show_bin)
-				filter_binary_set_state (FILTER_BINARY(f), FILTER_BINARY_COMPLETE);
-			else if (show_inc_bin)
-				filter_binary_set_state (FILTER_BINARY(f), FILTER_BINARY_INCOMPLETE);
-			else if (show_nonbinary)
-				filter_binary_set_state (FILTER_BINARY(f), FILTER_BINARY_NONBINARY);
-		} else {
-			if (!show_bin)
-				filter_binary_set_state (FILTER_BINARY(f), FILTER_BINARY_COMPLETE);
-			else if (!show_inc_bin)
-				filter_binary_set_state (FILTER_BINARY(f), FILTER_BINARY_INCOMPLETE);
-			else if (!show_nonbinary)
-				filter_binary_set_state (FILTER_BINARY(f), FILTER_BINARY_NONBINARY);
-			filter_negate (f);
-		}
-		filter_aggregate_add (top, &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-	}
-
-
-	/* thread state */
-	n = 0;
-	if (show_watched) ++n;
-	if (show_ignored) ++n;
-	if (show_normal) ++n;
-	if (n==1 || n==2) {
-		Filter * f = filter_thread_new ();
-		if (n==1) {
-			if (show_watched)
-				filter_thread_set_state (FILTER_THREAD(f), FILTER_THREAD_WATCHED);
-			else if (show_ignored)
-				filter_thread_set_state (FILTER_THREAD(f), FILTER_THREAD_IGNORED);
-			else if (show_normal)
-				filter_thread_set_state (FILTER_THREAD(f), FILTER_THREAD_NOTHING);
-		} else {
-			if (!show_watched)
-				filter_thread_set_state (FILTER_THREAD(f), FILTER_THREAD_WATCHED);
-			else if (!show_ignored)
-				filter_thread_set_state (FILTER_THREAD(f), FILTER_THREAD_IGNORED);
-			else if (!show_normal)
-				filter_thread_set_state (FILTER_THREAD(f), FILTER_THREAD_NOTHING);
-			filter_negate (f);
-		}
-		filter_aggregate_add (top, &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-	}
-
-	/* saved/queued/idle */
-	n = 0;
-	if (show_saved) ++n;
-	if (show_queued) ++n;
-	if (show_idle) ++n;
-	if (n==1 || n==2) {
-		Filter * f = filter_saved_new ();
-		if (n==1) {
-			if (show_saved)
-				filter_saved_set_state (FILTER_SAVED(f), FILTER_SAVED_SAVED);
-			else if (show_queued)
-				filter_saved_set_state (FILTER_SAVED(f), FILTER_SAVED_QUEUED);
-			else if (show_idle)
-				filter_saved_set_state (FILTER_SAVED(f), FILTER_SAVED_NOT_SAVED);
-		} else {
-			if (!show_saved)
-				filter_saved_set_state (FILTER_SAVED(f), FILTER_SAVED_SAVED);
-			else if (!show_queued)
-				filter_saved_set_state (FILTER_SAVED(f), FILTER_SAVED_QUEUED);
-			else if (!show_idle)
-				filter_saved_set_state (FILTER_SAVED(f), FILTER_SAVED_NOT_SAVED);
-			filter_negate (f);
-		}
-		filter_aggregate_add (top, &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-	}
-
-	/* cached */
-	n = 0;
-	if (show_cached) ++n;
-	if (show_non_cached) ++n;
-	if (n == 1) {
-		Filter * f = filter_cached_new ();
-		if (show_non_cached)
-			filter_negate (f);
-		filter_aggregate_add (top, &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-	}
-
-	/* phrase */
-	if (is_nonempty_string(phrase_str)) {
-		Filter * f = filter_phrase_new ();
-		filter_phrase_set (FILTER_PHRASE(f), phrase_state, phrase_str, 0);
-		filter_aggregate_add (top, &f, 1);
-		pan_object_unref (PAN_OBJECT(f));
-	}
-
-	/* retval */
-	if (filter_aggregate_child_size(top) != 1)
-		retval = FILTER(top);
-	else {
-		retval = filter_aggregate_get_child_at (top, 0);
-		pan_object_ref (PAN_OBJECT(retval));
-		pan_object_unref (PAN_OBJECT(top));
-	}
-
-	debug_exit ("create_filter_from_ulong");
-	return retval;
-}
-
 static gboolean
 is_active (GtkWidget * w)
 {
@@ -340,6 +176,11 @@ get_filter_bits (void)
 	if (is_active (show_non_cached_button))
 	 	l |= STATE_FILTER_NOT_CACHED;
 
+	if (is_active (show_by_me_button))
+		l |= STATE_FILTER_MINE;
+	if (is_active (show_by_others_button))
+		l |= STATE_FILTER_NOT_MINE;
+
 	debug_exit ("get_filter_bits");
 	return l;
 }
@@ -347,9 +188,9 @@ get_filter_bits (void)
 static Filter*
 create_filter_from_settings (void)
 {
-	return create_filter_from_ulong (get_filter_bits(),
-	                                 _filter_phrase_state,
-	                                 _filter_phrase_str);
+	return filter_create_from_bits_with_phrase (get_filter_bits(),
+	                                            _filter_phrase_state,
+	                                            _filter_phrase_str);
 }
 
 /***
@@ -420,6 +261,10 @@ update_filter_bit_menu_buttons (gulong l)
 	                                l & STATE_FILTER_CACHED);
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(show_non_cached_button),
 	                                l & STATE_FILTER_NOT_CACHED);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(show_by_me_button),
+	                                l & STATE_FILTER_MINE);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(show_by_others_button),
+	                                l & STATE_FILTER_NOT_MINE);
 
 	ignore_toggle = FALSE;
 	pan_unlock ();
@@ -463,22 +308,25 @@ sensitize_builtin_filters (gboolean b)
 	gtk_widget_set_sensitive (show_cached_button, b);
 	gtk_widget_set_sensitive (show_non_cached_button, b);
 
+	gtk_widget_set_sensitive (show_by_me_button, b);
+	gtk_widget_set_sensitive (show_by_others_button, b);
+
 	debug_exit ("sensitize_builtin_filters");
 }
 
 static void
 use_bit_filter (void)
 {
-	Filter * old;
+	Filter * old_filter;
 	debug_enter ("use_bit_filter");
 
 	sensitize_builtin_filters (TRUE);
 
 	/* update the filter variable */
-	old = _filter;
+	old_filter = _filter;
 	_filter = create_filter_from_settings ();
-	if (old != NULL)
-		pan_object_unref (PAN_OBJECT(old));
+	if (old_filter != NULL)
+		pan_object_unref (PAN_OBJECT(old_filter));
 
 	/* notify clients of the change */
 	fire_filter_changed ();
@@ -495,7 +343,7 @@ use_bit_filter (void)
 static void
 activate_canned_filter_by_name (const gchar * name)
 {
-	Filter * old = _filter;
+	Filter * old_filter = _filter;
 	debug_enter ("activate_canned_filter_by_name");
 
 	/* sanity clause */
@@ -509,8 +357,8 @@ activate_canned_filter_by_name (const gchar * name)
 		_filter = filter_dup (_scratch_filter);
 	else
 		_filter = filter_manager_get_named_filter (name);
-	if (old != NULL)
-		pan_object_unref (PAN_OBJECT(old));
+	if (old_filter != NULL)
+		pan_object_unref (PAN_OBJECT(old_filter));
 
 	/* notify clients of the change */
 	fire_filter_changed ();
@@ -545,7 +393,7 @@ custom_filter_selected_cb (GtkCheckMenuItem * item, gpointer user_data)
 
 			/* Custom filter: erase filter phrase */
  			replace_gstr (&_filter_phrase_str, NULL);
-			gtk_editable_delete_text (GTK_EDITABLE(_filter_phrase_entry), 0, -1);
+			gtk_editable_delete_text (GTK_EDITABLE(gnome_entry_gtk_entry(GNOME_ENTRY(_filter_phrase_entry))), 0, -1);
 		}
 	}
 
@@ -563,15 +411,15 @@ edit_dialog_clicked_cb (GnomeDialog * dialog, gint index, gpointer data)
 
 		/* update our scratch filter */
 		if (1) {
-			Filter * old;
+			Filter * old_filter;
 			GtkWidget * w;
 
 			/* make a copy of the dialog's scratch filter */
 			w = GTK_WIDGET(dialog);
-			old = _scratch_filter;
+			old_filter = _scratch_filter;
 			_scratch_filter = filter_dup (FILTER(filter_edit_dialog_get_filter(w)));
-			if (old != NULL)
-				pan_object_unref (PAN_OBJECT(old));
+			if (old_filter != NULL)
+				pan_object_unref (PAN_OBJECT(old_filter));
 		}
 
 		/* if _filter is a copy of _scratch, then update _filter too */
@@ -581,10 +429,10 @@ edit_dialog_clicked_cb (GnomeDialog * dialog, gint index, gpointer data)
 			if (!pan_strcmp(ft->name, FILTER_TOP(_scratch_filter)->name))
 			{
 				/* copy _scratch to _filter */
-				Filter * old = _filter;
+				Filter * old_filter = _filter;
 				_filter = filter_dup (_scratch_filter);
-				if (old != NULL)
-					pan_object_unref (PAN_OBJECT(old));
+				if (old_filter != NULL)
+					pan_object_unref (PAN_OBJECT(old_filter));
 
 				/* fire an update */
 				fire_filter_changed ();
@@ -627,7 +475,7 @@ update_custom_menus (void)
 {
 	guint i;
 	gchar * pch;
-	GPtrArray * a;
+	GPtrArray * filters;
 	GtkMenu * menu;
 	GtkWidget * w;
 	debug_enter ("update_custom_menus");
@@ -637,13 +485,13 @@ update_custom_menus (void)
 	menu = GTK_MENU(gtk_menu_new());
 
 	/* get the filters */
-	a = g_ptr_array_new ();
-	filter_manager_get_filters (a);
-	g_ptr_array_add (a, filter_dup(_scratch_filter));
+	filters = g_ptr_array_new ();
+	filter_manager_get_filters (filters);
+	g_ptr_array_add (filters, filter_dup(_scratch_filter));
 
 	/* create a hashtable for finding the menu button */
 	if (_name_to_menu != NULL) {
-		// FIXME: g_free the key strings
+		/* FIXME: g_free the key strings */
 		g_hash_table_destroy (_name_to_menu);
 	}
 	_name_to_menu = g_hash_table_new (g_str_hash, g_str_equal);
@@ -661,9 +509,9 @@ update_custom_menus (void)
 	/* add separator */
 	gtk_menu_append (menu, create_menu_separator());
 
-	for (i=0; i<a->len; ++i)
+	for (i=0; i<filters->len; ++i)
 	{
-		FilterTop * ft = FILTER_TOP (g_ptr_array_index(a,i));
+		FilterTop * ft = FILTER_TOP (g_ptr_array_index(filters,i));
 		if (ft==NULL || !ft->is_visible)
 			continue;
 
@@ -699,8 +547,8 @@ update_custom_menus (void)
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM(_custom_filters_menu), GTK_WIDGET(menu));
 
 	/* cleanup */
-	pan_g_ptr_array_foreach (a, (GFunc)pan_object_unref, NULL);
-	g_ptr_array_free (a, TRUE);
+	pan_g_ptr_array_foreach (filters, (GFunc)pan_object_unref, NULL);
+	g_ptr_array_free (filters, TRUE);
 	debug_exit ("update_custom_menus");
 }
 
@@ -719,10 +567,10 @@ filter_bit_toggled_cb (GtkCheckMenuItem * tb, gpointer user_data)
 	if (!ignore_toggle)
 	{
 		/* update filter */
-		Filter * old = _filter;
+		Filter * old_filter = _filter;
 		_filter = create_filter_from_settings ();
-		if (old != NULL) {
-			pan_object_unref (PAN_OBJECT(old));
+		if (old_filter != NULL) {
+			pan_object_unref (PAN_OBJECT(old_filter));
 		}
 
 		/* update to listeners that filter has changed */
@@ -763,11 +611,10 @@ bit_menu_new (const gchar * label, gchar ** xpm, const gchar * stock, GtkWidget
 	return w;
 }
 
-static gint
+static void
 custom_filters_changed_cb (gpointer foo, gpointer bar, gpointer mum)
 {
 	update_custom_menus ();
-	return 0;
 }
 
 static void
@@ -874,16 +721,6 @@ build_popdown_filter_menu (void)
 	                                 new_read_xpm, NULL,
 	                                 &show_read_button));
 	gtk_menu_append (m, create_menu_separator());
-	gtk_menu_append (m, bit_menu_new(_("Show Watched Threads"),
-	                                 watched_xpm, NULL,
-	                                 &show_watched_button));
-	gtk_menu_append (m, bit_menu_new(_("Show Ignored Threads"),
-	                                 NULL, NULL,
-	                                 &show_killfile_button));
-	gtk_menu_append (m, bit_menu_new(_("Show Threads Neither Watched nor Ignored"),
-	                                 NULL, NULL,
-	                                 &show_normal_rank_button));
-	gtk_menu_append (m, create_menu_separator());
 	gtk_menu_append (m, bit_menu_new(_("Show Complete Binary Articles"),
 	                                 binary_complete_xpm, NULL,
 	                                 &show_complete_binary_button));
@@ -910,7 +747,24 @@ build_popdown_filter_menu (void)
 	gtk_menu_append (m, bit_menu_new(_("Show Articles which are not Cached Locally"),
 	                                 NULL, NULL,
 	                                 &show_non_cached_button));
-
+	gtk_menu_append (m, create_menu_separator());
+	gtk_menu_append (m, bit_menu_new(_("Show Articles posted by Me"),
+	                                 by_me_xpm, NULL,
+                                         &show_by_me_button));
+	gtk_menu_append (m, bit_menu_new(_("Show Articles posted by Others"),
+	                                 by_others_xpm, NULL,
+	                                 &show_by_others_button));
+	gtk_menu_append (m, create_menu_separator());
+	gtk_menu_append (m, bit_menu_new(_("Show Watched Threads"),
+	                                 watched_xpm, NULL,
+	                                 &show_watched_button));
+	gtk_menu_append (m, bit_menu_new(_("Show Ignored Threads"),
+	                                 NULL, GNOME_STOCK_MENU_TRASH_FULL,
+	                                 &show_killfile_button));
+	gtk_menu_append (m, bit_menu_new(_("Show Threads Neither Watched nor Ignored"),
+	                                 NULL, NULL,
+	                                 &show_normal_rank_button));
+	
 	/* add the submenu onto a dropdown button */
 	w = gtk_menu_item_new ();
 	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (filter_xpm));
@@ -939,26 +793,25 @@ build_popdown_filter_menu (void)
 ****/
 
 static void
-set_filter_mode_cb (GtkMenuItem * item, gpointer data)
-{
-	_filter_phrase_state = GPOINTER_TO_INT(data);
-}
-
-static void
-article_phrase_activate_cb (GtkWidget* w, gpointer data)
+article_phrase_activate_cb (void)
 {
+	GtkWidget * e;
 	gchar * s;
 	debug_enter ("article_phrase_activate_cb");
 
-
 	/* if the new string differs from the old one, then update. */
-	s = gtk_editable_get_chars (GTK_EDITABLE(w), 0, -1);
+	e = gnome_entry_gtk_entry (GNOME_ENTRY(_filter_phrase_entry));
+	s = gtk_editable_get_chars (GTK_EDITABLE(e), 0, -1);
 	s = g_strstrip (s);
-	if (!pan_strcmp (_filter_phrase_str, s))  {
+
+	if (!pan_strcmp (_filter_phrase_str, s) && _prev_filter_phrase_state==(gint)_filter_phrase_state) 
+	{
 		g_free (s);
 	}
-	else {
+	else
+	{
 		replace_gstr (&_filter_phrase_str, s);
+		_prev_filter_phrase_state = (int) _filter_phrase_state;
 
 		if (is_active(_no_customs_button))
 			use_bit_filter ();
@@ -969,6 +822,14 @@ article_phrase_activate_cb (GtkWidget* w, gpointer data)
 	debug_exit ("article_phrase_activate_cb");
 }
 
+static void
+set_filter_mode_cb (GtkMenuItem * item, gpointer data)
+{
+	_filter_phrase_state = GPOINTER_TO_INT(data);
+
+	article_phrase_activate_cb ();
+}
+
 /****
 *****
 *****  BUILDING THE TOOLBAR
@@ -1006,15 +867,17 @@ article_toolbar_new (void)
 		int i;
 		ArticleToolbarOptionMenuStruct foo[] = {
 			{NULL, FILTER_PHRASE_SUBJECT},
-			{NULL, FILTER_PHRASE_AUTHOR}
+			{NULL, FILTER_PHRASE_AUTHOR},
+			{NULL, FILTER_PHRASE_MESSAGE_ID}
 		};
 		const int row_qty = sizeof(foo) / sizeof(foo[0]);
 		foo[0].name = _("Subject");
 		foo[1].name = _("Author");
+		foo[2].name = _("Message-ID");
 		for (i=0; i<row_qty; ++i) {
 			GtkWidget * item = gtk_menu_item_new_with_label (foo[i].name);
 			gtk_signal_connect (GTK_OBJECT(item), "activate",
-				(GtkSignalFunc)set_filter_mode_cb,
+				GTK_SIGNAL_FUNC(set_filter_mode_cb),
 				GINT_TO_POINTER(foo[i].state));
 			gtk_menu_append (GTK_MENU (menu), item);
 			gtk_widget_show (item);
@@ -1025,9 +888,12 @@ article_toolbar_new (void)
 		w = option_menu;
 	}
 	gtk_box_pack_start (GTK_BOX(toolbar), w, FALSE, FALSE, 0);
-	_filter_phrase_entry = w = gtk_entry_new();
-	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC(article_phrase_activate_cb), w);
-	gtk_signal_connect (GTK_OBJECT (w), "focus-out-event", GTK_SIGNAL_FUNC(article_phrase_activate_cb), w);
+	_filter_phrase_entry = w = gnome_entry_new ("Article Toolbar Filter");
+	gnome_entry_set_max_saved (GNOME_ENTRY(w), 10);
+	gtk_signal_connect (GTK_OBJECT (gnome_entry_gtk_entry(GNOME_ENTRY(w))), "activate",
+	                    GTK_SIGNAL_FUNC(article_phrase_activate_cb), NULL);
+	gtk_signal_connect (GTK_OBJECT(GTK_COMBO(w)->popwin), "button_press_event",
+	                    GTK_SIGNAL_FUNC(article_phrase_activate_cb), NULL);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
 	        _("Type in a search string and press ENTER. "
 	          "Wildcards are allowed; see http://pan.rebelbase.com/wildmat.html "
@@ -1076,23 +942,26 @@ article_toolbar_set_filter (const gchar  * name,
 	if (is_nonempty_string (name))
 		w = g_hash_table_lookup (_name_to_menu, name);
 
+	pan_lock ();
 	if (w != NULL) /* we have that custom filter */
 	{
-		pan_lock ();
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(w), TRUE);
-		pan_unlock ();
 	}
 	else /* make a bit-filter */
 	{
-		Filter           * old = _filter;
+		GtkWidget * e;
+		Filter * old_filter = _filter;
 
 		_filter = create_filter_from_settings ();
-		if (old != NULL)
-			pan_object_unref (PAN_OBJECT(old));
+		if (old_filter != NULL)
+			pan_object_unref (PAN_OBJECT(old_filter));
 
+ 		replace_gstr (&_filter_phrase_str, NULL);
+		e = gnome_entry_gtk_entry (GNOME_ENTRY(_filter_phrase_entry));
+		gtk_editable_delete_text (GTK_EDITABLE(e), 0, -1);
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(_no_customs_button), TRUE);
-
 	}
+	pan_unlock ();
 
 	/* update to listeners that filter has changed */
 	fire_filter_changed ();
