@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include <libgnomeui/libgnomeui.h>
 
@@ -33,6 +34,7 @@
 #include <pan/base/article-thread.h>
 #include <pan/base/article.h>
 #include <pan/base/debug.h>
+#include <pan/base/file-headers.h>
 #include <pan/base/group.h>
 #include <pan/base/log.h>
 #include <pan/base/pan-glib-extensions.h>
@@ -103,7 +105,6 @@ static Group * my_group                          = NULL;
 static Filter * my_filter                        = NULL;
 static FilterShow my_filter_show                 = FILTER_SHOW_MATCHES_AND_REFERENCES;
 static GHashTable * messageid_to_node            = NULL;
-static gboolean is_threaded                      = TRUE;
 static int sort_type                             = ARTICLE_SORT_SUBJECT;
 static GStaticMutex article_ctree_lock           = G_STATIC_MUTEX_INIT;
 static GStaticMutex hash_mutex                   = G_STATIC_MUTEX_INIT;
@@ -141,7 +142,7 @@ GdkColor watched_color;
 GdkColor thread_normal_color;
 GdkColor thread_read_color;
 
-static GnomeUIInfo header_pane_popup_uiinfo [];
+GnomeUIInfo header_pane_popup_uiinfo [];
 GtkWidget * header_pane_popup_menu;
 GnomeUIInfo articlelist_menu [];
 
@@ -156,8 +157,8 @@ refresh_nolock_impl (RefreshActions actions);
 static void
 apply_filter_tests (Filter * filter, GPtrArray * headers);
 
-static void
-fire_group_changed (Group * g);
+static gint
+fire_group_changed (gpointer group);
 
 static void
 articlelist_menu_popup_nolock (GdkEventButton* bevent);
@@ -994,7 +995,7 @@ group_articles_added_cb (gpointer call_obj, gpointer call_arg, gpointer client_a
 	/* sanity checks */
 	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (added!=NULL);
-	g_return_if_fail (added->len>0);
+	g_return_if_fail (added->len>0u);
 
 	if (group == my_group) {
 		pan_lock ();
@@ -1359,6 +1360,38 @@ articlelist_selected_flag_for_dl_nolock (void)
 	debug_exit ("articlelist_selected_flag_for_dl_nolock");
 }
 
+static void
+articlelist_selected_threads_unflag_for_dl_nolock (void)
+{
+	GPtrArray * sel;
+	GPtrArray * all;
+	debug_enter ("articlelist_selected_threads_unflag_for_dl_nolock");
+
+       	sel = articlelist_get_selected_articles_nolock ();
+	all = article_get_unique_threads (sel, GET_WHOLE_THREAD);
+	articles_remove_flag ((Article**)all->pdata, all->len, STATE_DOWNLOAD_FLAGGED);
+	g_ptr_array_free (all, TRUE);
+	g_ptr_array_free (sel, TRUE);
+
+	debug_exit ("articlelist_selected_threads_unflag_for_dl_nolock");
+}
+
+static void
+articlelist_selected_threads_flag_for_dl_nolock (void)
+{
+	GPtrArray * sel;
+	GPtrArray * all;
+	debug_enter ("articlelist_selected_threads_flag_for_dl_nolock");
+
+       	sel = articlelist_get_selected_articles_nolock ();
+	all = article_get_unique_threads (sel, GET_WHOLE_THREAD);
+	articles_add_flag ((Article**)all->pdata, all->len, STATE_DOWNLOAD_FLAGGED);
+	g_ptr_array_free (all, TRUE);
+	g_ptr_array_free (sel, TRUE);
+
+	debug_exit ("articlelist_selected_threads_flag_for_dl_nolock");
+}
+
 
 static void
 articlelist_download_flagged_bodies (void)
@@ -1540,7 +1573,7 @@ articlelist_selected_thread_download_nolock (void)
 
 	/* add the task */
 	sel = articlelist_get_selected_articles_nolock ();
-	all = article_get_unique_threads (sel, GET_SUBTHREAD);
+	all = article_get_unique_threads (sel, GET_WHOLE_THREAD);
 	task = TASK(task_bodies_new(my_group, all));
 	if (task != NULL)
 		queue_add (task);
@@ -1572,7 +1605,7 @@ rekill_thread_nolock (Article * article)
 	g_return_if_fail (article != NULL);
 
 	/* get the articles and their state */
-	state = thread_get_state (article);
+	state = pan_thread_get_state (article);
        	articles = g_ptr_array_new ();
 	article_get_entire_thread (article, articles);
 
@@ -1608,21 +1641,46 @@ articlelist_spamize_nolock (void)
 	debug_exit ("articlelist_spamize_nolock");
 }
 
+static void
+articlelist_selected_thread_watch_nolock_ghfunc (gpointer key, gpointer val, gpointer data)
+{
+	Article * a = ARTICLE(val);
+	gint o = pan_thread_get_state (a);
+	gint n = o==THREAD_WATCHED ? THREAD_NOTHING : THREAD_WATCHED;
+
+	pan_thread_set_state (a, n);
+	rekill_thread_nolock (a);
+
+	g_free (key);
+}
 static void
 articlelist_selected_thread_watch_nolock (void)
 {
-	guint i;
 	GPtrArray * sel;
 	debug_enter ("articlelist_selected_thread_watch_nolock");
 
 	sel = articlelist_get_selected_articles_nolock ();
-	for (i=0; i<sel->len; ++i)
+	if (sel->len > 0u)
 	{
-		Article * a = ARTICLE(g_ptr_array_index(sel,i));
-		gint o = thread_get_state (a);
-		gint n = o==THREAD_WATCHED ? THREAD_NOTHING : THREAD_WATCHED;
-		thread_set_state (a, n);
-		rekill_thread_nolock (a);
+		guint i;
+		GHashTable * hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+		/* build a hash table of {root message_id,root Article*} pairs */
+		for (i=0; i<sel->len; ++i) {
+			Article * a = ARTICLE(g_ptr_array_index (sel, i));
+			gchar * thread_message_id = article_get_thread_message_id (a);
+			if (g_hash_table_lookup (hash, thread_message_id) == NULL)
+				g_hash_table_insert (hash, thread_message_id, a);
+			else
+				g_free (thread_message_id);
+		}
+
+		if (g_hash_table_size(hash) > 0u) {
+			gtk_clist_freeze (GTK_CLIST(Pan.article_ctree));
+			g_hash_table_foreach (hash, articlelist_selected_thread_watch_nolock_ghfunc, NULL);
+			g_hash_table_destroy (hash);
+			gtk_clist_thaw (GTK_CLIST(Pan.article_ctree));
+		}
 	}
 
 	g_ptr_array_free (sel, TRUE);
@@ -1640,9 +1698,9 @@ articlelist_selected_thread_ignore_nolock (void)
 	for (i=0; i<sel->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index(sel,i));
-		gint o = thread_get_state (a);
+		gint o = pan_thread_get_state (a);
 		gint n = o==THREAD_IGNORED ? THREAD_NOTHING : THREAD_IGNORED;
-		thread_set_state (a, n);
+		pan_thread_set_state (a, n);
 		rekill_thread_nolock (a);
 	}
 
@@ -1764,20 +1822,20 @@ articlelist_set_node_style_nolock (GtkCTreeNode   * node,
 		const gboolean is_new = article ? article_is_new (article) : FALSE;
 		const gboolean new_children = article ? article->new_children!=0 : FALSE;
 		const gboolean unread_children = article ? article->unread_children!=0 : FALSE;
-		const gint thread_state = article ? thread_get_state(article) : THREAD_NOTHING;
+		const gint thread_state = article ? pan_thread_get_state(article) : THREAD_NOTHING;
 
 		if (thread_state == THREAD_IGNORED)
 			style = killfile_style;
 		else if (thread_state == THREAD_WATCHED)
 			style = watched_style;
-		else if (!is_read || (is_threaded && unread_children))
+		else if (!is_read || (header_pane_is_threaded && unread_children))
 			style = normal_style;
 		else
 			style = read_style;
 
 		gtk_ctree_node_set_row_style (tree, node, style[0]);
 
-		if (is_new || (is_threaded && new_children))
+		if (is_new || (header_pane_is_threaded && new_children))
 			gtk_ctree_node_set_cell_style (tree, node, 2, 
 				style[1]);
 	}
@@ -1945,7 +2003,7 @@ build_article_ctree_recursive (GtkCTree      * tree,
 		GtkCTreeNode* node = NULL;
 
 		/* possibly don't show subtrees that don't measure up to filter */
-		if (is_threaded && !article->tree_passes_filter)
+		if (header_pane_is_threaded && !article->tree_passes_filter)
 			continue;
 
 		/* possibly don't show 2...n of multipart nodes */
@@ -2007,19 +2065,19 @@ articlelist_repopulate_nolock (const Group   * group,
 		/* make sure we pass the state filter */
 		if (!article)
 			continue;
-		if (is_threaded && !article->tree_passes_filter)
+		if (header_pane_is_threaded && !article->tree_passes_filter)
 			continue;
-		if (!is_threaded && !article->self_passes_filter)
+		if (!header_pane_is_threaded && !article->self_passes_filter)
 			continue;
 
 		has_parent = article->parent != NULL;
 
-		if ((!is_threaded && article->part<2) ||
-			(is_threaded && (!has_parent || !article->parent->tree_passes_filter)))
+		if ((!header_pane_is_threaded && article->part<2) ||
+			(header_pane_is_threaded && (!has_parent || !article->parent->tree_passes_filter)))
 			sibling = node = add_article_to_ctree_nolock (tree, root, sibling, article, FALSE);
 
 		/* if threads are turned on, and there are children of this thread... */
-		if (is_threaded && node!=NULL && article->threads!=NULL)
+		if (header_pane_is_threaded && node!=NULL && article->threads!=NULL)
 			build_article_ctree_recursive (tree, node, article->threads, TRUE);
 	}
 
@@ -2346,7 +2404,7 @@ articlelist_selection_changed_cb (gpointer call_object,
 	debug_enter ("articlelist_selection_changed_cb");
 
 	/* load the specified article */
-       	articles = pan_g_ptr_array_dup ((GPtrArray*)call_arg);
+       	articles = (GPtrArray*) call_arg;
 	if (articles!=NULL && articles->len==1)
 		single_article = ARTICLE(g_ptr_array_index(articles,0));
         if ((single_article!=NULL)
@@ -2357,7 +2415,7 @@ articlelist_selection_changed_cb (gpointer call_object,
 	{
 		pan_warn_if_fail (single_article->group != NULL);
 		pan_warn_if_fail (single_article->group->server != NULL);
-		gui_page_set (MESSAGE_PAGE, Pan.text);
+		gui_page_set (MESSAGE_PAGE);
 		queue_add (TASK(task_body_new(single_article)));
 	}
 
@@ -2375,10 +2433,17 @@ articlelist_key_press (GtkWidget      * widget,
 	switch (event->keyval)
 	{
 		case GDK_Up: case GDK_Down:
-		case GDK_Left: case GDK_Right:
 		case GDK_Page_Up: case GDK_Page_Down:
 			button_click_count = 1;
 			break;
+		case GDK_leftarrow: case GDK_Left:
+			button_click_count = 1;
+			articlelist_collapse_selected_threads ();
+			break;
+		case GDK_rightarrow: case GDK_Right:
+			button_click_count = 1;
+			articlelist_expand_selected_threads ();
+			break;
 		default:
 			button_click_count = -1;
 	}
@@ -2825,9 +2890,9 @@ articlelist_collapse_all_threads (void)
 void
 articlelist_set_threaded (gboolean threaded_on)
 {
-	if (threaded_on != is_threaded)
+	if (threaded_on != header_pane_is_threaded)
 	{
-		is_threaded = threaded_on;
+		header_pane_is_threaded = threaded_on;
 		refresh_nolock_impl (0);
 	}
 }
@@ -2858,6 +2923,7 @@ articlelist_menu_popup_nolock (GdkEventButton* bevent)
 	++i;
 	gtk_widget_set_sensitive (articlelist_menu[++i].widget, have_article); /* delete */
 	gtk_widget_set_sensitive (articlelist_menu[++i].widget, have_article); /* delete */
+	gtk_widget_set_sensitive (articlelist_menu[++i].widget, have_article); /* delete */
 
 	gtk_menu_popup (GTK_MENU (header_pane_popup_menu),
 			NULL, NULL, NULL, NULL,
@@ -2874,6 +2940,12 @@ articlelist_menu_popup_nolock (GdkEventButton* bevent)
 	{ GNOME_APP_UI_ITEM, label, tooltip, (gpointer)callback, NULL, NULL, \
 		GNOME_APP_PIXMAP_DATA, xpm_data, accel, (GdkModifierType)modifier,  NULL }
 
+static void
+group_empty_cb (void)
+{
+	group_empty (my_group, FALSE);
+}
+
 GnomeUIInfo
 articlelist_menu [] =
 {
@@ -2903,7 +2975,7 @@ articlelist_menu [] =
 	                              flagged_xpm, 'J', 0),
 	GNOMEUIINFO_ITEM_XPM_ACCEL   (N_("Flag T_hreads"),
 	                              N_("Flag Selected Threads"),
-	                              articlelist_selected_flag_for_dl_nolock,
+	                              articlelist_selected_threads_flag_for_dl_nolock,
 	                              flagged_xpm, 'J', GDK_CONTROL_MASK),
 	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Unflag"),
 	                              N_("Unf_lag Selected Articles"),
@@ -2911,7 +2983,7 @@ articlelist_menu [] =
 	                              GNOME_STOCK_MENU_BLANK, 'K', 0),
 	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Unflag Thr_eads"),
 	                              N_("Unflag Selected Threads"),
-	                              articlelist_selected_unflag_for_dl_nolock,
+	                              articlelist_selected_threads_unflag_for_dl_nolock,
 	                              GNOME_STOCK_MENU_BLANK, 'K', GDK_CONTROL_MASK),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_ITEM_NONE  (N_("Download"),
@@ -2967,10 +3039,14 @@ articlelist_menu [] =
 	                              N_("Delete Threads"),
 	                              article_action_delete_selected_threads,
 	                              GNOME_STOCK_MENU_TRASH, 'D', GDK_SHIFT_MASK),
+	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Delete All"),
+	                              N_("Delete All Articles"),
+				      group_empty_cb,
+	                              GNOME_STOCK_MENU_TRASH, '\0', 0),
 	GNOMEUIINFO_END
 };
 
-static GnomeUIInfo
+GnomeUIInfo
 header_pane_popup_uiinfo [] =
 {
 	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Save _Articles"),
@@ -2997,7 +3073,7 @@ header_pane_popup_uiinfo [] =
 	                              flagged_xpm, 'J', 0),
 	GNOMEUIINFO_ITEM_XPM_ACCEL   (N_("Flag T_hreads"),
 	                              N_("Flag Selected Threads"),
-	                              articlelist_selected_flag_for_dl_nolock,
+	                              articlelist_selected_threads_flag_for_dl_nolock,
 	                              flagged_xpm, 'J', GDK_CONTROL_MASK),
 	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Unflag"),
 	                              N_("Unf_lag Selected Articles"),
@@ -3005,7 +3081,7 @@ header_pane_popup_uiinfo [] =
 	                              GNOME_STOCK_MENU_BLANK, 'K', 0),
 	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Unflag Thr_eads"),
 	                              N_("Unflag Selected Threads"),
-	                              articlelist_selected_unflag_for_dl_nolock,
+	                              articlelist_selected_threads_unflag_for_dl_nolock,
 	                              GNOME_STOCK_MENU_BLANK, 'K', GDK_CONTROL_MASK),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("_Delete"),
@@ -3016,6 +3092,11 @@ header_pane_popup_uiinfo [] =
 	                              N_("Delete Threads"),
 	                              article_action_delete_selected_threads,
 	                              GNOME_STOCK_MENU_TRASH, 'D', GDK_SHIFT_MASK),
+	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Delete All"),
+	                              N_("Delete All Articles"),
+				      group_empty_cb,
+	                              GNOME_STOCK_MENU_TRASH, '\0', 0),
+	GNOMEUIINFO_END
 };
 
 
@@ -3031,12 +3112,15 @@ articlelist_get_group_changed_callback (void)
 	return cb;
 }
 
-static void
-fire_group_changed (Group * g)
+static gint
+fire_group_changed (gpointer group_gp)
 {
+	Group * group = GROUP(group_gp);
+
 	pan_callback_call (articlelist_get_group_changed_callback(),
 	                   Pan.article_ctree,
-	                   g);
+	                   group);
+	return 0;
 }
 
 /***
@@ -3086,7 +3170,7 @@ refresh_mainthread_end (gpointer p)
 	return 0;
 }
 
-static void
+static void*
 refresh_worker (void * p)
 {
 	ArgSet * argset;
@@ -3117,13 +3201,14 @@ refresh_worker (void * p)
 	}
 	if (actions & REFRESH_THREAD) {
 		status_item_emit_status (status, _("Rethreading Articles"));
-		group_thread_if_needed (group, NULL);
+		group_thread_if_needed (group);
 	}
 	if (actions & REFRESH_FILTER) {
 		status_item_emit_status (status, _("Refiltering Articles"));
 		apply_filter_tests (my_filter, articles);
 	}
 	status_item_emit_status (status, _("Rebuilding Header Pane..."));
+	usleep (50000);
 
 	/* hard work done; back to the main thread */
 	pan_lock ();
@@ -3131,6 +3216,7 @@ refresh_worker (void * p)
 	pan_unlock ();
 
 	debug_exit ("refresh_worker");
+	return NULL;
 }
 
 static gint
@@ -3204,23 +3290,12 @@ clear_group_nolock (void)
 	/* clear the model */
 	if (my_group != NULL)
 	{
+		file_headers_save (my_group, NULL);
+		server_save_grouplist_if_dirty (my_group->server, NULL);
 		group_unref_articles (my_group, NULL);
 		g_ptr_array_set_size (my_articles, 0);
 		my_group = NULL;
-		run_in_main_thread_nolock ((GSourceFunc)fire_group_changed, my_group);
-	}
-}
-
-static void
-headers_fetched (gpointer callobj, gpointer callarg, gpointer client)
-{
-	Group * group = GROUP(client);
-
-	if (group->article_qty > 0)
-	{
-		pan_lock ();
-		articlelist_set_group_nolock (group);
-		pan_unlock ();
+		run_in_main_thread_nolock (fire_group_changed, my_group);
 	}
 }
 
@@ -3258,8 +3333,11 @@ set_group_mainthread_end (gpointer p)
 	tmp = pan_g_ptr_array_dup (my_articles);
 	my_group = group;
 	pan_g_ptr_array_assign (my_articles, articles->pdata, articles->len);
-	if (old_group != NULL)
+	if (old_group != NULL) {
+		file_headers_save (old_group, NULL);
+		server_save_grouplist_if_dirty (old_group->server, NULL);
 		group_unref_articles (old_group, NULL);
+	}
 	fire_group_changed (my_group);
 
 	/* no longer using the status */
@@ -3277,7 +3355,7 @@ set_group_mainthread_end (gpointer p)
 	return 0;
 }
 
-static void
+static void*
 set_group_worker (void * p)
 {
 	ArgSet * argset;
@@ -3308,13 +3386,14 @@ set_group_worker (void * p)
 	status_item_emit_status (status, _("Sorting Articles"));
 	sort_articles ((Article**)articles->pdata, articles->len, abs(sort), sort>0);
 	status_item_emit_status (status, _("Threading Articles"));
-	group_thread_if_needed (group, NULL);
+	group_thread_if_needed (group);
 	status_item_emit_status (status, _("Filtering Articles"));
 	filter = filter_create_from_bits (group->filter_name, group->filter_bits);
 	articlelist_set_filter_bits_nolock (filter, group->filter_show);
 	apply_filter_tests (filter, articles);
 	pan_object_unref (PAN_OBJECT(filter));
 	status_item_emit_status (status, _("Updating Header Pane..."));
+	usleep (50000);
 
 	/* hard work over, pass back to main thread */
 	pan_lock ();
@@ -3322,6 +3401,7 @@ set_group_worker (void * p)
 	pan_unlock ();
 
 	debug_exit ("set_group_worker");
+	return NULL;
 }
 
 static gint
@@ -3350,6 +3430,19 @@ set_group_mainthread_begin (gpointer p)
 	return 0;
 }
 
+static void
+headers_fetched (gpointer callobj, gpointer callarg, gpointer client)
+{
+	Group * group = GROUP(client);
+
+	if (group->article_qty > 0)
+	{
+		pan_lock ();
+		run_in_main_thread_nolock (set_group_mainthread_begin, argset_new1(group));
+		pan_unlock ();
+	}
+}
+
 void
 articlelist_set_group_nolock (Group * group)
 {
