@@ -74,13 +74,13 @@ normalize_subject (gchar * buf, const Article * a)
 	gchar * out = buf;
 
 	/* skip the leading noise */
-	while (*in && !isalnum((int)*in) && !isdigit((int)*in))
+	while (*in && !isalnum((guchar)*in) && !isdigit((guchar)*in))
 		++in;
 
 	while (*in)
 	{
 		/* strip multipart information */
-		if (multipart && (*in=='('||*in=='[') && isdigit((int)in[1])) {
+		if (multipart && (*in=='('||*in=='[') && isdigit((guchar)in[1])) {
 			const char ch = *in=='(' ? ')' : ']';
 			while (*in && *in!=ch)
 				++in;
@@ -88,7 +88,7 @@ normalize_subject (gchar * buf, const Article * a)
 		}
 
 		/* strip out junk that breaks sorting  */
-		if (isalnum((int)*in) || isdigit((int)*in) || isspace((int)*in))
+		if (isalnum((guchar)*in) || isdigit((guchar)*in) || isspace((guchar)*in))
 			*out++ = ('A'<=*in && *in<='Z') ? tolower(*in) : *in;
 
 		++in;
@@ -118,6 +118,7 @@ normalize_articles (Article    ** articles,
 	/* sanity clause */
 	g_return_if_fail (articles!=NULL);
 	g_return_if_fail (qty>0);
+	g_return_if_fail (articles_are_valid ((const Article **)articles, qty));
 	g_return_if_fail (alloc_and_setme_norm!=NULL);
 	g_return_if_fail (alloc_and_setme_str!=NULL);
 
@@ -141,6 +142,41 @@ normalize_articles (Article    ** articles,
 	}
 }
 
+static int
+compare_pA_to_pA_by_subject (const void * va, const void * vb)
+{
+	int value;
+	const Article * a = (const Article *)va;
+	const Article * b = (const Article *)vb;
+	gboolean a_is_reply;
+	gboolean b_is_reply;
+
+	/* subject is the primary key, of course... */
+	if ((value = *a->subject - *b->subject))
+		return value;
+	if ((value = strcmp (a->subject, b->subject)))
+		return value;
+
+	/* if one but not both is a reply, the reply goes second */
+	a_is_reply = skip_reply_leader (a->subject) != a->subject ? 1 : 0;
+	b_is_reply = skip_reply_leader (b->subject) != b->subject ? 1 : 0;
+	if (a_is_reply != b_is_reply)
+		return a_is_reply ? 1 : -1;
+
+	/* check multipart */
+	if ((value = a->part - b->part))
+		return value;
+
+	/* oldest goes first... */
+	if (a->date < b->date)
+		value = -1;
+	else if (a->date > b->date)
+		value = 1;
+	else
+		value = 0;
+	return value;
+}
+
 static int
 compare_pN_to_pN_by_subject (const void * va, const void * vb)
 {
@@ -163,39 +199,23 @@ compare_pN_to_pN_by_subject (const void * va, const void * vb)
 		return value;
 
 	/* oldest goes first... */
-	return (int) difftime (a->a->date, b->a->date);
+	if (a->a->date < b->a->date)
+		value = -1;
+	else if (a->a->date > b->a->date)
+		value = 1;
+	else
+		value = 0;
+	return value;
 }
 
 
 static int
 compare_ppA_to_ppA_by_linecount (const void* va, const void* vb)
 {
-        const Article * a;
-        const Article * b;
-	long a_loc;
-	long b_loc;
-
-	/* get a's linecount... */
-	a = *(const Article**)va;
-	a_loc = a->linecount;
-	if (a->parts!=0 && a->threads!=NULL) {
-		GSList * l;
-		for (l=a->threads; l; l=l->next)
-			a_loc += ARTICLE(l->data)->linecount;
-	}
-
-	/* get b's linecount... */
-	b = *(const Article**)vb;
-	b_loc = b->linecount;
-	if (b->parts!=0 && b->threads!=NULL) {
-		GSList * l;
-		for (l=b->threads; l; l=l->next)
-			b_loc += ARTICLE(l->data)->linecount;
-	}
-
-	/* return the difference... */
-	return a_loc - b_loc;
-} 
+        const Article * a = *(const Article**)va;
+        const Article * b = *(const Article**)vb;
+	return article_get_combined_linecount(a) - article_get_combined_linecount(b);
+}
 
 static int
 compare_ppA_to_ppA_by_action (const void * va, const void * vb)
@@ -270,15 +290,24 @@ compare_ppA_to_ppA_by_read (const void * va, const void * vb)
 static int
 compare_ppA_to_ppA_by_date (const void* va, const void* vb)
 {
-	return (int) difftime ((**(const Article**)va).date,
-	                       (**(const Article**)vb).date);
+	gint value;
+	time_t date_a = (*(const Article**)va)->date;
+	time_t date_b = (*(const Article**)vb)->date;
+
+	if (date_a < date_b)
+		value = -1;
+	else if (date_a > date_b)
+		value = 1;
+	else
+		value = 0;
+	return value;
 }
 
 static int
 compare_ppA_to_ppA_by_message_id (const void* a, const void* b)
 {
-	const gchar * msg_id_a = article_get_message_id (*(const Article**)a);
-	const gchar * msg_id_b = article_get_message_id (*(const Article**)b);
+	const gchar * msg_id_a = (*(const Article**)a)->message_id;
+	const gchar * msg_id_b = (*(const Article**)b)->message_id;
 	return strcmp (msg_id_a, msg_id_b);
 }
 
@@ -303,6 +332,8 @@ sort_articles (Article      ** buf,
                int             sort_type,
                gboolean        ascending)
 {
+	g_return_if_fail (articles_are_valid ((const Article **)buf, article_qty));
+
 	switch (sort_type)
 	{
 		case ARTICLE_SORT_AUTHOR:
@@ -398,12 +429,50 @@ is_child_of (const Article * child,
 	}
 }
 
+static guint
+article_get_part_state (Article * a)
+{
+	guint retval;
+
+	if (a->parts<1)
+		retval = 0;
+	else if (a->part>1)
+		retval = STATE_MULTIPART_SOME;
+	else {
+		GSList * l;
+		gint part = a->part + 1;
+		for (l=a->threads; l!=NULL; l=l->next)
+			if (ARTICLE(l->data)->parts==a->parts && ARTICLE(l->data)->part==part)
+				++part;
+		retval = part==a->parts+1 ? STATE_MULTIPART_ALL : STATE_MULTIPART_SOME;
+	}
+
+	return retval;
+}
+
+static void
+set_children_part_state (Article * a, guint state)
+{
+	gint part;
+	GSList * l;
+
+	a->state |= state;
+
+	part = a->part + 1;
+	for (l=a->threads; l!=NULL; l=l->next) {
+		Article * child = ARTICLE(l->data);
+		if (child->parts==a->parts && child->part==part) {
+			child->state |= state;
+			++part;
+		}
+	}
+}
+
 /**
  * Thread the articles specified in list
  */
 void
-thread_articles (GPtrArray    * articles,
-                 StatusItem   * status)
+thread_articles (GPtrArray    * articles)
 {
 	guint i;
 	guint qty = articles->len;
@@ -415,25 +484,20 @@ thread_articles (GPtrArray    * articles,
 	Norm * sorted_norm;
 	GArray * buf = NULL;
 
+	/* sanity clause */
 	g_return_if_fail (articles!=NULL);
+	g_return_if_fail (articles_are_valid ((const Article**)articles->pdata, articles->len));
+
 	if (qty<1 || !articles)
 		return;
 
 	if (break_thread_when_subject_changes)
 		buf = g_array_new (FALSE, FALSE, 1);
 
-	if (status != NULL)
-		status_item_emit_status_va (status, _("Threading %u articles"), qty);
-	/* make a plausiably-legal article FIXME: indent this right after the string freeze is over*/ search_a.number = 1; search_a.subject = "dummy subject"; search_a.references = NULL;
-	/* unthread the articles, just in case they were threaded before */
-	for (i=0; i!=qty; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		a->parent = NULL;
-		a->unread_children = 0;
-		a->new_children = 0;
-		g_slist_free (a->threads);
-		a->threads = NULL;
-	}
+	/* make a plausiably-legal article */
+	search_a.number = 1;
+	search_a.subject = "dummy subject";
+	search_a.references = NULL;
 
 	/* make a message-id-sorted array of the articles */
 	refs = g_memdup (articles->pdata, sizeof(gpointer)*qty);
@@ -448,6 +512,17 @@ thread_articles (GPtrArray    * articles,
 	sorted_norm = g_memdup (norm, sizeof(Norm)*qty);
 	qsort (sorted_norm, qty, sizeof(Norm), compare_pN_to_pN_by_subject);
 
+	/* unthread the articles, just in case they were threaded before */
+	for (i=0; i!=qty; ++i) {
+		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+		a->parent = NULL;
+		a->unread_children = 0;
+		a->new_children = 0;
+		g_slist_free (a->threads);
+		a->threads = NULL;
+	}
+
+
 	/* thread the articles */
 	for (i=0; i!=qty; ++i)
 	{
@@ -456,14 +531,6 @@ thread_articles (GPtrArray    * articles,
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
 		gint index = -1;
 
-		/* let the user know what we're doing */
-		if (status != NULL) {
-			status_item_emit_next_step (status);
-			if (i==qty-1 || !(i % 256))
-				status_item_emit_status_va (status,
-					_("Threaded %u of %u articles"), i, qty);
-		}
-
 		/* thread by reference
 		   (except for parts 2...n of multiparts, which need to be threaded by multipart) */
 		references = article_get_header (a, HEADER_REFERENCES);
@@ -566,7 +633,7 @@ thread_articles (GPtrArray    * articles,
 					/* 1 original, 1 reply */
 					parent = match->a;
 				}
-				else if (!strcmp(match->subject, a->subject) && difftime(match->a->date,a->date)<0)
+				else if (!strcmp(match->subject, a->subject) && (match->a->date<a->date))
 				{
 					/* 2 replies, no top --  oldest on top */
 					parent = match->a;
@@ -580,17 +647,14 @@ thread_articles (GPtrArray    * articles,
 
 			/* link the two articles */
 			a->parent = parent;
-			parent->threads = g_slist_prepend (parent->threads, norm+i);
+			parent->threads = g_slist_prepend (parent->threads, norm[i].a);
 		}
 	}
 
 	/* right now all the children are normalized; point to articles */
 	for (i=0; i!=qty; ++i) {
-		GSList * l;
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		a->threads = g_slist_sort (a->threads, compare_pN_to_pN_by_subject);
-		for (l=a->threads; l!=NULL; l=l->next)
-			l->data = ((Norm*)l->data)->a;
+		a->threads = g_slist_sort (a->threads, compare_pA_to_pA_by_subject);
 	}
 
 	/* calculate new/unread child counts */
@@ -609,6 +673,14 @@ thread_articles (GPtrArray    * articles,
 				++a->new_children;
 	}
 
+	/* check multipart count */
+	for (i=0; i!=articles->len; ++i) {
+		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+		a->state &= ~(STATE_MULTIPART_ALL&STATE_MULTIPART_SOME);
+		if (a->parent == NULL)
+			set_children_part_state (a, article_get_part_state(a));
+	}
+
 	/* cleanup */
 	g_free (refs);
 	g_free (norm);
@@ -617,57 +689,3 @@ thread_articles (GPtrArray    * articles,
 	if (buf != NULL)
 		g_array_free (buf, TRUE);
 }
-
-
-void
-check_multipart_articles (GPtrArray * articles)
-{
-	guint i;
-	gint j;
-
-	/* entry assertions */
-	g_return_if_fail (articles!=NULL);
-
-	/* set the multipart state (all/partial) */
-	for (i=0; i!=articles->len; ++i)
-	{
-		GSList * p;
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-
-		/* clear old state */
-		a->state &= ~(STATE_MULTIPART_ALL&STATE_MULTIPART_SOME);
-
-		/* not a multipart because it has no parts */
-		if (!a->parts)
-			continue;
-
-		/* not a multipart because it's not the first of the set */
-		if (a->part!=1)
-			continue;
-
-		/* handle the single-part attachment message */
-		if (a->parts==1) {
-			a->state |= STATE_MULTIPART_ALL;
-			continue;
-		}
-
-		/* make sure we have each multipart. */
-		for (j=a->part+1, p=a->threads;
-		     j<=a->parts && p!=NULL;
-		     p=p->next)
-		{
-			Article* b = ARTICLE(p->data);
-			if (b->part > j)
-				break; /* some */
-			else if (b->part == j)
-				++j; /* okay so far */
-			else
-				; /* a repost of a multipart section? */
-		}
-		if (j==a->parts+1) {
-			a->state |= STATE_MULTIPART_ALL;
-		} else {
-			a->state |= STATE_MULTIPART_SOME;
-		}
-	}
-}
