@@ -48,13 +48,11 @@ static void fire_articles_changed (Group*, Article**, gint article_qty, ArticleC
 ***/
 
 void
-article_destructor (PanObject* o)
+article_destructor (Article * a)
 {
-	Article * a = ARTICLE(o);
         g_return_if_fail (a != NULL);
 
 	/* clean up article parts */
-	debug1 (DEBUG_PAN_OBJECT, "article_destructor: %p", a);
 	if (a->headers != NULL) {
 		g_hash_table_destroy (a->headers);
 		a->headers = NULL;
@@ -73,22 +71,15 @@ article_destructor (PanObject* o)
 	a->author_addr = NULL;
 	a->author_real = NULL;
 	a->group = NULL;
-
-	/* clean up parent */
-	pan_object_destructor (o);
 }
 
-void
+static void
 article_constructor (Article * article, Group * group)
 {
 	g_return_if_fail (article!=NULL);
 	g_return_if_fail (group!=NULL);
 
-	/* init the superclass */
-	pan_object_constructor (PAN_OBJECT(article), article_destructor);
-
 	/* init this class */
-        debug1 (DEBUG_PAN_OBJECT, "article_constructor: %p", article);
 	article->group = group;
 	article->self_passes_filter = FALSE;
 	article->tree_passes_filter = FALSE;
@@ -116,7 +107,11 @@ article_constructor (Article * article, Group * group)
 Article*
 article_new (Group * group)
 {
-	Article* article = g_new (Article, 1);
+	Article * article;
+
+	g_return_val_if_fail (group!=NULL, NULL);
+
+	article = group_alloc_new_article (group);
 	article_constructor (article, group);
 	return article;
 }
@@ -179,8 +174,7 @@ articles_set_dirty (Article ** articles, gint qty)
 
 	/* sanity check */
 	g_return_if_fail (qty >= 1);
-	g_return_if_fail (articles != NULL);
-	g_return_if_fail (articles[0] != NULL);
+	g_return_if_fail (articles_are_valid ((const Article **)articles, qty));
 
 	/* mark the articles' group as read */
 	group = articles[0]->group;
@@ -189,40 +183,11 @@ articles_set_dirty (Article ** articles, gint qty)
 }
 
 
-
-/**
- * Remove an article from its parent/child threadings.  The children and
- * parent are linked together instead.  This is useful for not breaking
- * the threading of articles surrounding an article which is about to be
- * destroyed.
- */
-void
-article_isolate (Article* article)
-{
-	GSList *l = NULL;
-	Article *parent = article->parent;
-
-	if (parent)
-		parent->threads = g_slist_remove (parent->threads, article);
-
-	for (l=article->threads; l!=NULL; l=l->next) {
-		Article *child = ARTICLE(l->data);
-		child->parent = parent;
-		if (parent)
-			parent->threads = g_slist_append (parent->threads, child);
-	}
-
-	g_slist_free (article->threads);
-	article->threads = NULL;
-	article->parent = NULL;
-}
-
-
 gboolean
 article_is_new (const Article * a)
 {
-	g_return_val_if_fail (a!=NULL, FALSE);
-	g_return_val_if_fail (a->group!=NULL, FALSE);
+	g_return_val_if_fail (article_is_valid(a), FALSE);
+
 	return a->number > a->group->article_high_old;
 }
 
@@ -292,6 +257,10 @@ articles_set_read_articles_ghfunc (gpointer key, gpointer value, gpointer user_d
 	g_array_free (articles, TRUE);
 }
 
+/**
+ * The hashtable maps from a Group* to a GArray*, which is returned.
+ * If no matching GArray* exists, it's created and added to the hash before returning.
+ */
 static GArray*
 get_group_array_from_hash (GHashTable * hash, Group * group, gint size)
 {
@@ -301,14 +270,23 @@ get_group_array_from_hash (GHashTable * hash, Group * group, gint size)
 	return arr;
 }
 
+/**
+ * Builds a GHashTable of Group* -> GArray*, where the GArray is filled with message numbers.
+ */
 static void
 mark_read_xreffunc (Group * g, gulong number, gpointer user_data)
 {
 	GHashTable ** group_to_numbers = (GHashTable**) user_data;
-	if (group_is_subscribed(g)) {
+
+	if (group_is_subscribed(g))
+	{
+		GArray * group_numbers;
+
 		if (*group_to_numbers == NULL)
 			*group_to_numbers = g_hash_table_new (g_direct_hash, g_direct_equal);
-		g_array_append_val (get_group_array_from_hash(*group_to_numbers,g,sizeof(gulong)), number);
+
+		group_numbers = get_group_array_from_hash (*group_to_numbers, g, sizeof(gulong));
+		g_array_append_val (group_numbers, number);
 	}
 }
 
@@ -318,13 +296,13 @@ articles_set_read (Article ** articles, int article_qty, gboolean read)
 	int i;
 	GHashTable * group_to_articles;
 	GHashTable * group_to_numbers = NULL;
-
 	debug_enter ("articles_set_read");
 
 	/* sanity check */
-	g_return_if_fail (articles != NULL);
 	g_return_if_fail (article_qty > 0);
+	g_return_if_fail (articles_are_valid ((const Article **)articles, article_qty));
 
+	/* get a big list, grouped by Group*,  of all the articles that we're marking read */
 	group_to_articles = g_hash_table_new (g_direct_hash, g_direct_equal);
 	for (i=0; i<article_qty; ++i)
 	{
@@ -335,15 +313,23 @@ articles_set_read (Article ** articles, int article_qty, gboolean read)
 		/* add this article to the group's array */
 	       	arr = get_group_array_from_hash (group_to_articles, a->group, sizeof(Article*));
 		g_array_append_val (arr, a);
-		article_xref_foreach (a, mark_read_xreffunc, &group_to_numbers, TRUE);
+		/* add the xrefs to their group's arrays */
+		article_xref_foreach (a, mark_read_xreffunc, &group_to_numbers,
+		                      SERVER_GROUPS_SUBSCRIBED, TRUE);
 
-		/* if it's a multipart, add parts 2...n as well */
-		if (a->part>0 && a->parts>1 && a->threads!=NULL) {
+		/* if it's a multipart, process the child parts too */
+		if (a->part>0 && a->parts>1 && a->threads!=NULL)
+		{
 			for (l=a->threads; l!=NULL; l=l->next) {
 				Article * child = ARTICLE(l->data);
 				if (child->part > 0) {
+					/* add this article to the group's array */
 					g_array_append_val (arr, child);
-					article_xref_foreach (child, mark_read_xreffunc, &group_to_numbers, TRUE);
+					/* add the xrefs to their group's arrays */
+					article_xref_foreach (child,
+						mark_read_xreffunc,
+						&group_to_numbers,
+						SERVER_GROUPS_SUBSCRIBED, TRUE);
 				}
 			}
 		}
@@ -358,6 +344,8 @@ articles_set_read (Article ** articles, int article_qty, gboolean read)
 		g_hash_table_foreach (group_to_numbers, articles_set_read_numbers_ghfunc, GINT_TO_POINTER(read));
 		g_hash_table_destroy (group_to_numbers);
 	}
+
+	debug_exit ("articles_set_read");
 }
 
 
@@ -372,9 +360,11 @@ articles_add_flag (Article  ** articles,
 {
 	gint i;
 
-	g_return_if_fail (articles != NULL);
+	/* sanity clause */
 	g_return_if_fail (qty >= 1);
+	g_return_if_fail (articles_are_valid ((const Article **)articles, qty));
 
+	/* mark 'em */
 	for (i=0; i<qty; ++i)
 		articles[i]->state |= flag;
 
@@ -389,9 +379,11 @@ articles_remove_flag (Article         ** articles,
 {
 	gint i;
 
-	g_return_if_fail (articles != NULL);
+	/* sanity clause */
 	g_return_if_fail (qty >= 1);
+	g_return_if_fail (articles_are_valid ((const Article **)articles, qty));
 
+	/* mark 'em */
 	for (i=0; i<qty; ++i)
 		articles[i]->state &= ~flag;
 
@@ -410,18 +402,25 @@ gchar*
 article_get_thread_message_id (const Article* article)
 {
 	char * pch;
-	const gchar * refs = article_get_header (article, HEADER_REFERENCES);
-	const gchar * msg_id = article_get_message_id (article);
+	const gchar * refs;
 
-	g_return_val_if_fail (article!=NULL, NULL);
-	g_return_val_if_fail (is_nonempty_string(msg_id), NULL);
+	/* sanity clause */
+	g_return_val_if_fail (article_is_valid(article), NULL);
 
+	/* go up as high as we can from our own threading, because sometimes
+	 * the References: header is broken by other newreaders. */
+	while (article->parent != NULL)
+		article = article->parent;
+
+	refs = article_get_header (article, HEADER_REFERENCES);
 	if (is_nonempty_string(refs))
 	{
 		pch = get_next_token_str (refs, ' ', NULL);
 	}
 	else /* top of the thread */
 	{
+		const gchar * msg_id = article_get_message_id (article);
+		g_return_val_if_fail (is_nonempty_string(msg_id), NULL);
 		pch = g_strdup (msg_id);
 	}
 
@@ -480,6 +479,54 @@ article_get_references (Article* article, GPtrArray* setme)
 	}
 }
 
+GPtrArray*
+article_get_unique_threads (const GPtrArray    * articles,
+                            ThreadGet            thread_get)
+{
+	int i;
+	GPtrArray * thread = g_ptr_array_new ();
+	GPtrArray * retval = g_ptr_array_new ();
+	GHashTable * all = g_hash_table_new (g_str_hash, g_str_equal);
+	debug_enter ("articlelist_get_unique_threads");
+
+	/* get the thread for each article passed in */
+	for (i=0; i<articles->len; ++i)
+	{
+		int j;
+		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+
+		/* if we already have the article, then we have its thread, so skip */
+		if (g_hash_table_lookup (all, article_get_message_id(a)) != NULL)
+			continue;
+
+		/* get the thread associated with sel... */
+		g_ptr_array_set_size (thread, 0);
+		if (thread_get == GET_SUBTHREAD)
+			article_get_subthread (a, thread);
+		else
+			article_get_entire_thread (a, thread);
+
+		/* add the articles into "all" hash */
+		for (j=0; j<thread->len; ++j)
+		{
+			Article * b = ARTICLE(g_ptr_array_index(thread,j));
+			const gchar * message_id = article_get_message_id (b);
+
+			if (g_hash_table_lookup (all, message_id) == NULL)
+			{
+				g_hash_table_insert (all, (gpointer)article_get_message_id(b), (gpointer)b);
+				g_ptr_array_add (retval, b);
+			}
+		}
+	}
+
+	/* cleanup */
+	g_hash_table_destroy (all);
+	g_ptr_array_free (thread, TRUE);
+	debug_exit ("article_get_unique_threads");
+	return retval;
+}
+
 
 /***
 ****
@@ -529,10 +576,12 @@ article_get_header (const Article * a, const gchar * key)
 	if (key == HEADER_MESSAGE_ID) retval = a->message_id;
 	else if (key == HEADER_SUBJECT) retval = a->subject;
 	else if (key == HEADER_REFERENCES) retval = a->references;
+	else if (key == HEADER_XREF) retval = a->xref;
 	else if (key == HEADER_FROM) pan_warn_if_reached ();
 	else if (!strcmp(key, HEADER_MESSAGE_ID)) retval = a->message_id;
 	else if (!strcmp (key, HEADER_SUBJECT)) retval = a->subject;
 	else if (!strcmp (key, HEADER_REFERENCES)) retval = a->references;
+	else if (!strcmp (key, HEADER_XREF)) retval = a->xref;
 	else if (!strcmp (key, HEADER_FROM)) pan_warn_if_reached ();
 	else if (a->headers!=NULL)
 		retval = (const gchar*) g_hash_table_lookup (a->headers, key);
@@ -560,6 +609,7 @@ article_header_is_extra (const gchar * key)
 	if (!strcmp(key,HEADER_NEWSGROUPS)) return FALSE;
 	if (!strcmp(key,HEADER_ORGANIZATION)) return FALSE;
 	if (!strcmp(key,HEADER_REPLY_TO)) return FALSE;
+	if (!strcmp(key,HEADER_DATE)) return FALSE;
 
 	return TRUE;
 }
@@ -575,6 +625,7 @@ void
 article_xref_foreach (const Article       * a,
                       ArticleXRefFunc       func,
                       gpointer              user_data,
+		      ServerGroupsType      set,
                       gboolean              skip_group_a)
 {
 	gchar * str;
@@ -604,9 +655,11 @@ article_xref_foreach (const Article       * a,
 		if (delimit != NULL)
 		{
 			const gulong number = strtoul (delimit+1, NULL, 10);
-			Group * g;
+			Group * g = NULL;
 			*delimit = '\0';
-			g = server_get_named_group (server, str);
+
+			if (server_has_group_in_type (server, str, set))
+				g = server_get_named_group (server, str);
 
 			if (g!=NULL && (a->group!=g || !skip_group_a))
 				(*func)(g, number, user_data);
@@ -631,7 +684,7 @@ article_set_header_impl (Article         * a,
 		val = group_chunk_string (a->group, val, action==DO_CHUNK_SHARE);
 
 	/* if we're possibly erasing a header, remove the old value */
-	if (!is_nonempty_string (val))
+	if (!init && !is_nonempty_string (val))
 		article_remove_header (a, key);
 
 	/* add the new header */
@@ -775,30 +828,32 @@ article_get_author_str (const Article * a)
 	return retval;
 }
 
-gchar*
-article_get_short_author_str (const Article * a)
+void
+article_get_short_author_str (const Article * a, gchar * buf, gint len)
 {
-	gchar * author;
+	g_return_if_fail (article_is_valid(a));
+	g_return_if_fail (buf!=NULL);
+	g_return_if_fail (len>0);
 
 	if (is_nonempty_string (a->author_real) && strcmp(a->author_real,default_incoming_name_real))
 	{
-		author = g_strdup (a->author_real);
+		strncpy (buf, a->author_real, len);
+		buf[len-1] = '\0';
 	}
 	else if (is_nonempty_string (a->author_addr) && strcmp(a->author_addr,default_incoming_name_addr))
 	{
-		author = g_strdup (a->author_addr);
-		if (author != NULL) {
-			gchar * pch = strchr (author, '@');
-			if (pch != NULL)
-				*pch = '\0';
-		}
+		const gchar * author = a->author_addr;
+		const gchar * pch = strchr (author, '@');
+		if (pch == NULL)
+			pch = author + strlen(author);
+		strncpy (buf, author, MIN(len,pch-author));
+		buf[len-1] = '\0';
 	}
 	else
 	{
-		author = g_strdup (default_incoming_name_real);
+		strncpy (buf, default_incoming_name_real, len);
+		buf[len-1] = '\0';
 	}
-
-	return author;
 }
 
 
@@ -864,30 +919,30 @@ article_get_headers (const Article * a)
 		/* subject */
 		c_tmp = a->subject;
 		if (is_nonempty_string (c_tmp))
-			g_string_sprintfa (s, "Subject: %s\n", c_tmp);
+			g_string_sprintfa (s, "%s: %s\n", HEADER_SUBJECT, c_tmp);
 
 		/* author */
 	       	tmp = article_get_author_str (a);
 		if (is_nonempty_string (tmp))
-			g_string_sprintfa (s, "From: %s\n", tmp);
+			g_string_sprintfa (s, "%s: %s\n", HEADER_FROM, tmp);
 		g_free (tmp);
 
 		/* date */
 		if (a->date != 0) {
 			tmp = rfc822_date_generate (a->date);
-			g_string_sprintfa (s, "Date: %s\n", tmp);
+			g_string_sprintfa (s, "%s: %s\n", HEADER_DATE, tmp);
 			g_free (tmp);
 		}
 
 		/* message-id */
 		c_tmp = a->message_id;
 		if (is_nonempty_string (c_tmp))
-			g_string_sprintfa (s, "Message-Id: %s\n", c_tmp);
+			g_string_sprintfa (s, "%s: %s\n", HEADER_MESSAGE_ID, c_tmp);
 
 		/* references */
 		c_tmp = a->references;
 		if (is_nonempty_string (c_tmp))
-			g_string_sprintfa (s, "References: %s\n", c_tmp);
+			g_string_sprintfa (s, "%s: %s\n", HEADER_REFERENCES, c_tmp);
 
 		/* extra headers */
 		if (1) {
@@ -896,7 +951,7 @@ article_get_headers (const Article * a)
 			for (i=0; i<z->len; i+=2) {
 				const char* key = (const char*) g_ptr_array_index (z, i);
 				const char* val = (const char*) g_ptr_array_index (z, i+1);
-				if (!article_header_is_internal (key))
+				if (strncmp (key, PAN_BODY, strlen(PAN_BODY)) != 0)
 					g_string_sprintfa (s, "%s: %s\n",  key, val);
 			}
 			g_ptr_array_free (z, TRUE);
@@ -954,8 +1009,10 @@ article_set_author_from_header_impl (Article       * a,
 	/* note that strict is FALSE here because we can't control
 	 * how other people have posted to usenet */
 	gnksa_do_check_from (header_from, &addr, &real, FALSE);
-	replace_gstr (&addr, g_mime_utils_8bit_header_decode(addr));
-	replace_gstr (&real, g_mime_utils_8bit_header_decode(real));
+	if (pan_header_is_8bit_encoded (addr))
+		replace_gstr (&addr, g_mime_utils_8bit_header_decode((const guchar *)addr));
+	if (pan_header_is_8bit_encoded (real))
+		replace_gstr (&real, g_mime_utils_8bit_header_decode((const guchar *)real));
 
 	/* use the real mail address, or fill in a default */
 	pch = addr;
@@ -1000,17 +1057,30 @@ article_set_author_from_header (Article         * a,
 }
 
 
+static void
+set_from_raw_init_header_gmhfunc (const gchar * name,
+                                  const gchar * value,
+                                  gpointer data)
+{
+	Article * a = ARTICLE(data);
+
+	g_return_if_fail (a!=NULL);
+	g_return_if_fail (is_nonempty_string(name));
+	g_return_if_fail (is_nonempty_string(value));
+
+	article_init_header (a, name, value, DO_CHUNK);
+}
+
 void
 article_set_from_raw_message   (Article         * a,
                                 const gchar     * text)
 {
-	guint i;
 	guint16 lines;
 	GMimeMessage * msg;
-	GPtrArray * arr;
 	gchar * body;
 	const gchar * pch;
 	gboolean foo;
+	debug_enter ("article_set_from_raw_message");
 
 	/* get the message */
 	g_return_if_fail (a != NULL);
@@ -1019,63 +1089,36 @@ article_set_from_raw_message   (Article         * a,
 	g_return_if_fail (msg != NULL);
 
 	/* body */
-	a->date = msg->header->date;
 	body = g_mime_message_get_body (msg, TRUE, &foo);
 	article_init_header (a, PAN_BODY, body, DO_CHUNK);
-	for (lines=0, pch=body; pch!=NULL && *pch; ++pch)
+	for (lines=0, pch=body; is_nonempty_string(pch); ++pch)
 		if (*pch=='\n')
 			++lines;
 	a->linecount = lines;
 	g_free (body);
 
 	/* headers */
-	article_init_header (a, HEADER_FROM, msg->header->from, DO_CHUNK_SHARE);
-	article_init_header (a, HEADER_REPLY_TO, msg->header->reply_to, DO_CHUNK_SHARE);
-	article_init_header (a, HEADER_SUBJECT, msg->header->subject, DO_CHUNK_SHARE);
-	article_init_header (a, HEADER_MESSAGE_ID, msg->header->message_id, DO_CHUNK);
-	arr = msg->header->arbitrary_headers;
-	for (i=0; i!=arr->len; ++i)
-	{
-		GMimeHeader * h = (GMimeHeader*) g_ptr_array_index (arr, i);
-		const gchar * name = h->name;
-		if (!pan_strcmp (name, "Author")) /* back compatability with 0.9.2 */
-			name = HEADER_FROM;
-		article_init_header (a, h->name, h->value, DO_CHUNK_SHARE);
+	if (msg->header != NULL) {
+		a->date = msg->header->date;
+		if (msg->header->from != NULL)
+			article_init_header (a, HEADER_FROM, msg->header->from, DO_CHUNK_SHARE);
+		if (msg->header->reply_to != NULL)
+			article_init_header (a, HEADER_REPLY_TO, msg->header->reply_to, DO_CHUNK_SHARE);
+		if (msg->header->subject != NULL)
+			article_init_header (a, HEADER_SUBJECT, msg->header->subject, DO_CHUNK_SHARE);
+		if (msg->header->message_id != NULL)
+			article_init_header (a, HEADER_MESSAGE_ID, msg->header->message_id, DO_CHUNK);
+		g_mime_header_foreach (msg->header->headers, set_from_raw_init_header_gmhfunc, a);
 	}
 
-	/* check to handle buggy <= 0.9.6 mboxes.  The problem was that Pan
-	   translated the header names, so instead of "From:" we get "Von:"
-	   and so on. */
-	if (!is_nonempty_string(a->message_id))
-	{
-		g_warning (_("Couldn't parse header info mbox message; trying harder."));
-
-		/* <= 0.9.6 wrote headers in a particular order: subject, author, date,
-		   message-id, references. Since improper i18n is the most likely glitch
-		   we're going to encounter here, do some simple checks to detect it. */
-		if (arr->len>= 5
-			&& (strchr(((GMimeHeader*)g_ptr_array_index(arr,1))->value,'@')!=NULL)
-			&& (((GMimeHeader*)g_ptr_array_index(arr,3))->value[0]=='<'))
-		{
-			const GMimeHeader * h;
-g_message ("looks like a <= 0.9.6 mbox");
-			h = (GMimeHeader*) g_ptr_array_index (arr, 0);
-			article_init_header (a, HEADER_SUBJECT, h->value, DO_CHUNK_SHARE);
-			h = (GMimeHeader*) g_ptr_array_index (arr, 1);
-			article_init_header (a, HEADER_FROM, h->value, DO_CHUNK_SHARE);
-			h = (GMimeHeader*) g_ptr_array_index (arr, 2);
-			a->date = g_mime_utils_header_decode_date (h->value, NULL);
-			h = (GMimeHeader*) g_ptr_array_index (arr, 3);
-			article_init_header (a, HEADER_MESSAGE_ID, h->value, DO_CHUNK);
-		}
-		else
-		{
-			gchar * id = gnksa_generate_message_id_from_email_addr (a->author_addr);
-g_message ("dunno what this is");
-			article_init_header (a, HEADER_MESSAGE_ID, id, DO_CHUNK);
-			g_free (id);
-			article_init_header (a, HEADER_SUBJECT, _("Unparseable Subject"), DO_CHUNK_SHARE);
-		}
+	/* fallback if no message-id */
+	if (!is_nonempty_string(a->message_id)) {
+		gchar * id;
+		g_warning (_("Couldn't parse a Message-ID from the raw message!"));
+		id = gnksa_generate_message_id_from_email_addr (a->author_addr);
+		article_init_header (a, HEADER_MESSAGE_ID, id, DO_CHUNK);
+		g_free (id);
+		article_init_header (a, HEADER_SUBJECT, _("Unparseable Subject"), DO_CHUNK_SHARE);
 	}
 
 	/* state */
@@ -1093,27 +1136,47 @@ g_message ("dunno what this is");
 
 	/* cleanup */
 	g_mime_message_destroy (msg);
+	debug_exit ("article_set_from_raw_message");
 }
 
 
+static gboolean
+breakpoint (const Article * a)
+{
+	if (a->group!=NULL)
+		g_message ("bad article's group: [%s]", a->group->name);
+	return FALSE;
+}
+
 gboolean
 article_is_valid (const Article * a)
 {
-#if 0
-	g_assert (a!=NULL);
-	g_assert (a->pan_object.ref_count>0);
-	g_assert (a->message_id!=NULL && *a->message_id=='<');
-	g_assert (is_nonempty_string(a->subject));
-	g_assert (a->number!=0);
-	g_assert (!a->references || *a->references=='<');
-#else
-	g_return_val_if_fail (a!=NULL, FALSE);
-	g_return_val_if_fail (a->pan_object.ref_count>0, FALSE);
-	g_return_val_if_fail (a->message_id!=NULL && *a->message_id=='<', FALSE);
-	g_return_val_if_fail (is_nonempty_string(a->subject), FALSE);
-	g_return_val_if_fail (a->number!=0, FALSE);
-	g_return_val_if_fail (!a->references || *a->references=='<', FALSE);
-#endif
+	g_return_val_if_fail (a->group!=NULL, breakpoint(a));
+	g_return_val_if_fail (group_is_valid(a->group), breakpoint(a));
+	g_return_val_if_fail (a->group->_articles_refcount>=0, breakpoint(a));
+	g_return_val_if_fail (group_is_folder(a->group) || a->group->_articles_refcount>0, breakpoint(a));
+	g_return_val_if_fail (a->message_id!=NULL, breakpoint(a));
+	g_return_val_if_fail (*a->message_id=='<', breakpoint(a));
+	g_return_val_if_fail (is_nonempty_string(a->subject), breakpoint(a));
+	g_return_val_if_fail (a->number!=0, breakpoint(a));
+	g_return_val_if_fail (!a->references || *a->references=='<', breakpoint(a));
+
+	return TRUE;
+}
+
+gboolean
+articles_are_valid (const Article ** a, int qty)
+{
+	gint i;
+
+	/* sanity clause */
+	g_return_val_if_fail (qty>=0, FALSE);
+	g_return_val_if_fail (qty==0 || a!=NULL, FALSE);
+
+	/* check each article */
+	for (i=0; i<qty; ++i)
+		g_return_val_if_fail (article_is_valid (a[i]), FALSE);
+
 	return TRUE;
 }
 
@@ -1132,6 +1195,62 @@ article_get_subject (const Article * a)
 }
 
 
+gulong
+article_get_combined_linecount (const Article * a)
+{
+	gulong retval = 0;
+
+	g_return_val_if_fail (article_is_valid(a), FALSE);
+
+	retval = a->linecount;
+	if (a->part!=0 && a->parts!=0 && a->threads!=NULL) {
+		GSList * l;
+		gint part = a->part + 1;
+		for (l=a->threads; l; l=l->next) {
+			Article * child = ARTICLE(l->data);
+			if (child->part == part) {
+				retval += child->linecount;
+				++part;
+			}
+		}
+	}
+
+	return retval;
+}
+
+/* Worker function for easily retrieving the newsreader */
+gchar*       
+article_get_newsreader (const Article * article)
+{
+	static gchar * tags[] = { HEADER_USER_AGENT": ",
+		                  HEADER_X_NEWSREADER": ",
+				  HEADER_X_MAILER": " };
+	static gint   tag_count = sizeof tags / sizeof tags[0];
+
+	gchar       * newsreader = NULL;
+	gchar       * headers = article_get_headers (article);
+	gchar      ** lines = g_strsplit (headers, "\n", -1);
+	gint          i;
+
+	for (i=0; newsreader == NULL && lines!=NULL && lines[i]!=NULL; ++i) {
+		gint tag;
+
+		for (tag = 0; tag < tag_count; tag++) {
+			gint tag_size = strlen(tags[tag]);
+
+			if (strncmp(lines[i], tags[tag], tag_size) == 0) {
+				newsreader = g_strdup (lines[i] + tag_size);
+				g_strstrip (newsreader);
+			}
+		}
+	}
+
+	g_strfreev (lines);	
+	g_free (headers);
+
+	return newsreader;
+}
+
 
 /***
 ****  EVENTS
