@@ -39,7 +39,7 @@ enum {
 	STATE_DECODED		= 1<<4,
 	STATE_ERROR		= 1<<5,
 	STATE_CACHED		= 1<<7,
-	STATE_DECODE_FAILED	= 1<<8,
+	STATE_DECODE_FAILED	= 1<<8
 };
 
 typedef struct _Article
@@ -84,13 +84,17 @@ Article;
 #define HEADER_ORGANIZATION       "Organization"
 #define HEADER_REFERENCES         "References"
 #define HEADER_REPLY_TO           "Reply-To"
-#define HEADER_SUPERSEDES         "Supersedes"
+#define HEADER_SUPERCEDES         "Supercedes"
 #define HEADER_XREF               "Xref"
 /* Headers stored in their own fields but here for completeness */
 #define HEADER_SUBJECT            "Subject"
 #define HEADER_FROM               "From"
 #define HEADER_MESSAGE_ID         "Message-Id"
 #define HEADER_DATE               "Date"
+/* Headers not stored (must be retrieved from the body). */
+#define HEADER_USER_AGENT         "User-Agent"
+#define HEADER_X_NEWSREADER       "X-Newsreader"
+#define HEADER_X_MAILER           "X-Mailer"
 /* Headers beginning with Pan- are for internal use. */
 #define PAN_ATTRIBUTION           "Pan-Attribution"
 #define PAN_ATTACH_FILE           "Pan-Attach-File"
@@ -256,6 +260,9 @@ void         article_xref_foreach             (const Article   * a,
  */
 gboolean     article_is_valid                 (const Article  * article);
 
+gboolean     articles_are_valid               (const Article ** articles,
+                                               gint             qty);
+
 gboolean     article_has_attachment           (const Article  * article);
 
 gboolean     article_has_body                 (const Article  * article);
@@ -279,6 +286,11 @@ void         article_get_short_author_str     (const Article  * article,
 void         article_set_from_raw_message     (Article        * article,
                                                const gchar    * text);
 
+gulong       article_get_combined_linecount   (const Article  * article);
+
+/* Worker function for easily retrieving the newsreader */
+gchar*       article_get_newsreader           (const Article * article);
+
 /**
 ***  READ
 **/
