@@ -55,17 +55,21 @@ file_headers_load_group (Group * group, StatusItem * status)
 	debug_enter ("file_headers_load_group");
 
 	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group->server!=NULL, FALSE);
+	g_return_val_if_fail (is_nonempty_string (group->name), FALSE);
+	g_return_val_if_fail (is_nonempty_string (group->server->name), FALSE);
+	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
 
 	if (status!=NULL)
 		status_item_emit_status_va (status, _("Loading %s"), group_get_readable_name(group));
 
 	/* open the index file */
-	path = g_strdup_printf ("/%s/%s/%s.idx", data_dir, group->server->name, group->name);
+	path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
 	idx = read_file (path);
 	g_free (path);
 
 	/* open the data file */
-	path = g_strdup_printf ("/%s/%s/%s.dat", data_dir, group->server->name, group->name);
+	path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
 	dat = read_file (path);
 	g_free (path);
 
@@ -130,8 +134,15 @@ file_headers_load_group (Group * group, StatusItem * status)
 				/* references */
 				l = get_next_token_long (march, '\n', &march);
 				if (0<=l && l<dat->len)
-					article_init_header (a, HEADER_REFERENCES, dat->data+l, 0);
-				/* xrefs added in version 4 */ if (version>=4) { l = get_next_token_long (march, '\n', &march); if (0<=l && l<dat->len) a->xref = dat->data + l; }
+					a->references = dat->data + l;
+
+				/* xrefs added in version 4 */
+				if (version>=4) {
+					l = get_next_token_long (march, '\n', &march);
+					if (0<=l && l<dat->len)
+						a->xref = dat->data + l;
+				}
+
 				/* numeric fields */
 				a->part           = (gint16) get_next_token_int (march, '\n', &march);
 				a->parts          = (gint16) get_next_token_int (march, '\n', &march);
@@ -166,7 +177,6 @@ file_headers_load_group (Group * group, StatusItem * status)
 					g_ptr_array_add (addme, a);
 				else {
 					++purged_article_count;
-					pan_object_unref (PAN_OBJECT(a));
 				}
 			}
 
@@ -205,12 +215,16 @@ get_mbox_filename (const Group * folder)
 	debug_enter ("get_mbox_filename");
 
 	g_return_val_if_fail (folder!=NULL, NULL);
+	g_return_val_if_fail (folder->server!=NULL, NULL);
 	g_return_val_if_fail (group_is_folder(folder), NULL);
+	g_return_val_if_fail (is_nonempty_string(data_dir), NULL);
+	g_return_val_if_fail (is_nonempty_string(folder->server->name), NULL);
 
-	retval = g_strdup_printf ("/%s/%s/%s.mbox",
-		data_dir,
-		folder->server->name,
-		folder->name);
+	retval = g_strdup_printf ("%s%c%s%c%s.mbox", data_dir,
+	                                             G_DIR_SEPARATOR,
+	                                             folder->server->name,
+	                                             G_DIR_SEPARATOR,
+	                                             folder->name);
 
 	debug_exit ("get_mbox_filename");
 	return retval;
@@ -223,6 +237,7 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 	gchar * text;
 	const gchar * march;
 	GPtrArray * articles;
+	GString * msg;
 	debug_enter ("file_headers_load_folder");
 
 	g_return_val_if_fail (folder!=NULL, FALSE);
@@ -248,29 +263,27 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 	i = 0;
 	march = text;
 	articles = g_ptr_array_new ();
+	msg = g_string_sized_new (2048);
 	while (march != NULL)
 	{
 		gchar * from = NULL;
-		gchar * msg = NULL;
 
 		/* parse the next mbox message */
-		march = mbox_get_next_message (march, &from, &msg);
-		if (msg != NULL)
+		march = mbox_get_next_message (march, &from, msg);
+		if (msg->len != 0)
 		{
 			Article * a = article_new (folder);
 			a->number = ++i;
-			article_set_from_raw_message (a, msg);
-			g_free (msg);
+			article_set_from_raw_message (a, msg->str);
 
 			/* keep the good ones */
 			if (article_is_valid (a))
 				g_ptr_array_add (articles, a);
-			else
-				pan_object_unref (PAN_OBJECT(a));
 		}
 
 		g_free (from);
 	}
+	g_string_free (msg, TRUE);
 
 	/* if we've got articles then add them; otherwise, clean up */
 	if (articles->len != 0)
@@ -378,29 +391,47 @@ get_string_offset (GHashTable   * hash,
 }
 
 
-static void
+static gboolean
 file_headers_save_folder (Group * folder, StatusItem * status)
 {
-	FILE * fp;
-	gchar * fname;
-	gchar * fname_tmp;
+	FILE * fp = NULL;
+	gchar * fname = NULL;
+	gchar * fname_tmp = NULL;
+	gboolean ok = TRUE;
 	debug_enter ("file_headers_save_folder");
 
-	g_return_if_fail (group_is_folder(folder));
+	/* sanity clause */
+	g_return_val_if_fail (folder!=NULL, FALSE);
+	g_return_val_if_fail (group_is_folder(folder), FALSE);
+
+	/* make sure the directory exists */
+	if (ok) {
+		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, folder->server->name);
+		if (!directory_check (pch)) {
+			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
+			ok = FALSE;
+		}
+		g_free (pch);
+	}
 
 	/* get the filenames & start writing to the outfile */
-	fname = get_mbox_filename (folder);
-	fname_tmp = g_strdup_printf ("%s.tmp", fname);
-	fp = fopen (fname_tmp, "w+");
+	if (ok) {
+		fname = get_mbox_filename (folder);
+		fname_tmp = g_strdup_printf ("%s.tmp", fname);
+		fp = fopen (fname_tmp, "w+");
+		if (fp == NULL) {
+			log_add_va (LOG_ERROR, _("The group will not be saved -- can't create file \"%s\""), fname_tmp);
+			ok = FALSE;
+		}
+	}
 
-	if (fp != NULL)
-	{
+	/* save the articles to the mbox */
+	if (ok) {
 		guint i = 0;
 		GPtrArray * array = group_get_article_array (folder);
 		GString * s = g_string_new (NULL);
 
-		for (i=0; i!=array->len; ++i)
-		{
+		for (i=0; i!=array->len; ++i) {
 			Article * a = ARTICLE(g_ptr_array_index(array,i));
 			gchar buf[32], *pch, *mbox;
 
@@ -422,33 +453,45 @@ file_headers_save_folder (Group * folder, StatusItem * status)
   
 			/* add body */ 
 			pch = article_get_body (a);
-			g_string_append (s, pch);
+			if (is_nonempty_string(pch))
+				g_string_append (s, pch);
 			g_free (pch);
 
 			/* write the message in mbox format */
 			mbox = mbox_format_message (s->str, a->author_addr, a->date);
-			if (mbox != NULL)
-			{
-				fputs (mbox, fp);
+			if (mbox != NULL) {
+				if (fputs (mbox, fp) == EOF) {
+					ok = FALSE;
+					break;
+				}
 				g_free (mbox);
 			}
+
+			pan_warn_if_fail (article_is_valid(a)); /* FIXME: just testing, remove later */
 		}
 		fclose (fp);
 
-		/* wrote to tmp file okay... now make it permanent */
-		pan_rename (fname_tmp, fname);
-
 		/* cleanup */
 		g_ptr_array_free (array, TRUE);
 		g_string_free (s, TRUE);
 	}
 
+	/* wrote to tmp file okay... now make it permanent */
+	if (ok) {
+		if (!pan_rename (fname_tmp, fname)) {
+			log_add_va (LOG_ERROR, _("The group will not be saved -- can't move \"%s\" to \"%s\""),
+			fname_tmp, fname);
+			ok = FALSE;
+		}
+	}
+
 	g_free (fname);
 	g_free (fname_tmp);
 	debug_exit ("file_headers_save_folder");
+	return ok;
 }
 
-static void
+static gboolean
 file_headers_save_group (Group * group, StatusItem * status)
 {
 	guint i;
@@ -464,41 +507,59 @@ file_headers_save_group (Group * group, StatusItem * status)
 	GHashTable * hash_misc = NULL;
 	GMemChunk * long_chunk = NULL;
 	GPtrArray * articles;
+	gboolean ok = TRUE;
 	debug_enter ("file_headers_save_group");
 
-	g_return_if_fail (group!=NULL);
-	g_return_if_fail (group->server!=NULL);
-	g_return_if_fail (group->_articles!=NULL);
+	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group->server!=NULL, FALSE);
+	g_return_val_if_fail (group->_articles!=NULL, FALSE);
+	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
 
 	/* if nothing to save */
 	articles = group_get_article_array (group);
 	if (articles->len == 0) {
 		g_ptr_array_free (articles, TRUE);
 		file_headers_destroy (group);
-		return;
+		return FALSE;
 	}
 
 	/**
 	***  Save the Headers
 	**/
 
+	if (1) {
+		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, group->server->name);
+		if (!directory_check (pch))
+			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
+		g_free (pch);
+	}
+
+
 	/* open index file */
-	idx_path = g_strdup_printf ("/%s/%s/%s.idx.tmp", data_dir, group->server->name, group->name);
+	idx_path = g_strdup_printf ("%s%c%s%c%s.idx.tmp",
+		data_dir, G_DIR_SEPARATOR,
+		group->server->name, G_DIR_SEPARATOR,
+		group->name);
 	idx_fp = fopen (idx_path, "w+");
 	if (idx_fp == NULL) {
+		log_add_va (LOG_ERROR, _("The group will not be saved -- can't create file \"%s\""), idx_path);
 		g_free (idx_path);
-		return;
+		return FALSE;
 	}
 
 	/* open data file */
-	dat_path = g_strdup_printf ("/%s/%s/%s.dat.tmp", data_dir, group->server->name, group->name);
+	dat_path = g_strdup_printf ("%s%c%s%c%s.dat.tmp",
+		data_dir, G_DIR_SEPARATOR,
+		group->server->name, G_DIR_SEPARATOR,
+		group->name);
 	dat_fp = fopen (dat_path, "w+");
 	if (dat_fp == NULL) {
+		log_add_va (LOG_ERROR, _("The group will not be saved -- can't create file \"%s\""), dat_path);
 		fclose (idx_fp);
 		remove (idx_path);
 		g_free (idx_path);
 		g_free (dat_path);
-		return;
+		return FALSE;
 	}
 
 	/* Write DATBASE_VERSION */
@@ -573,6 +634,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 			fprintf (idx_fp, "%ld\n%ld\n", key_index, val_index);
 		}
 
+		pan_warn_if_fail (article_is_valid(a)); /* FIXME: just testing, remove later */
 		g_ptr_array_free (extra_headers, TRUE);
 	}
 	g_hash_table_destroy (hash_name); hash_name = NULL;
@@ -581,66 +643,108 @@ file_headers_save_group (Group * group, StatusItem * status)
 	g_hash_table_destroy (hash_misc); hash_misc = NULL;
 	g_mem_chunk_destroy (long_chunk);
 
-	/* the write went okay; move the idx file over */
+	/* close the temp files */
 	fclose (idx_fp);
-	pch = g_strdup_printf ("/%s/%s/%s.idx", data_dir, group->server->name, group->name);
-	pan_rename (idx_path, pch);
-	g_free (idx_path);
-	g_free (pch);
-
-	/* the write went okay; move the dat file over */
 	fclose (dat_fp);
-	pch = g_strdup_printf ("/%s/%s/%s.dat", data_dir, group->server->name, group->name);
-	pan_rename (dat_path, pch);
-	g_free (dat_path);
-	g_free (pch);
+
+	ok = TRUE;
+
+	if (ok)
+	{
+		pch = g_strdup_printf ("%s%c%s%c%s.idx",
+			data_dir, G_DIR_SEPARATOR,
+			group->server->name, G_DIR_SEPARATOR,
+			group->name);
+		ok = pan_rename (idx_path, pch);
+		g_free (idx_path);
+		g_free (pch);
+	}
+
+	if (ok)
+	{
+		pch = g_strdup_printf ("%s%c%s%c%s.dat",
+			data_dir, G_DIR_SEPARATOR,
+			group->server->name, G_DIR_SEPARATOR,
+			group->name);
+		ok = pan_rename (dat_path, pch);
+		g_free (dat_path);
+		g_free (pch);
+	}
 
 	/* cleanup */
 	g_ptr_array_free (articles, TRUE);
 	debug_exit ("file_headers_save_group");
+	return ok;
+}
+
+void
+file_headers_save_noref (Group * group, StatusItem * status)
+{
+	static GStaticMutex _mutex = G_STATIC_MUTEX_INIT;
+
+	debug_enter ("file_headers_save_noref");
+	g_static_mutex_lock (&_mutex);
+
+	if (group->articles_dirty)
+	{
+		GPtrArray * articles;
+
+		/* since we're saving, mark dirty to FALSE */
+		group->articles_dirty = FALSE;
+
+		articles = group_get_article_array (group);
+		if (articles->len == 0) /* destroy the group */
+		{
+			file_headers_destroy (group);
+		}
+		else /* save the group */
+		{
+			struct timeval start;
+			struct timeval finish;
+			double diff;
+			gboolean ok = TRUE;
+
+			/* start the stopwatch */
+			gettimeofday (&start, NULL);
+
+			/* save the group */
+			ok = group_is_folder (group)
+				? file_headers_save_folder (group, status)
+				: file_headers_save_group (group, status);
+
+			if (!ok)
+				group->articles_dirty = TRUE;
+			else {
+				/* end the stopwatch */
+				gettimeofday (&finish, NULL);
+				diff = finish.tv_sec - start.tv_sec;
+				diff += (finish.tv_usec - start.tv_usec)/1000000.0;
+				log_add_va (LOG_INFO, _("Saved %d articles in \"%s\" in %.1f seconds (%.0f art/sec)"),
+					articles->len,
+					group_get_readable_name (group),
+					diff,
+					articles->len/(fabs(diff)<0.001?0.001:diff));
+			}
+		}
+
+		/* cleanup */
+		g_ptr_array_free (articles, TRUE);
+	}
+
+	g_static_mutex_unlock (&_mutex);
+
+	debug_exit ("file_headers_save_noref");
 }
 void
 file_headers_save (Group * group, StatusItem * status)
 {
-	GPtrArray * articles;
-	struct timeval start;
-	struct timeval finish;
-	double diff;
 	debug_enter ("file_headers_save");
+	g_return_if_fail (group!=NULL);
 
-	gettimeofday (&start, NULL);
+	group_ref_articles (group, status);
+	file_headers_save_noref (group, status);
+	group_unref_articles (group, status);
 
-	/* trivial case #1: no change, so no need to save */
-	if (!group->articles_dirty)
-		return;
-
-	/* since we're saving, mark dirty to FALSE */
-	group->articles_dirty = FALSE;
-
-	articles = group_get_article_array (group);
-	if (articles->len == 0) /* destroy the group */
-	{
-		file_headers_destroy (group);
-		return;
-	}
-	else /* save the group */
-	{
-		if (group_is_folder (group))
-			file_headers_save_folder (group, status);
-		else
-			file_headers_save_group (group, status);
-		/* timing stats */
-		gettimeofday (&finish, NULL);
-		diff = finish.tv_sec - start.tv_sec;
-		diff += (finish.tv_usec - start.tv_usec)/1000000.0;
-		log_add_va (LOG_INFO, _("Saved %d articles in \"%s\" in %.1f seconds (%.0f art/sec)"),
-			articles->len,
-			group_get_readable_name (group),
-			diff,
-			articles->len/(fabs(diff)<0.001?0.001:diff));
-	}
-
-	g_ptr_array_free (articles, TRUE);
 	debug_exit ("file_headers_save");
 }
 
@@ -663,11 +767,11 @@ file_headers_destroy (const Group * group)
 	{
 		gchar * path;
 
-		path = g_strdup_printf ("/%s/%s/%s.idx", data_dir, group->server->name, group->name);
+		path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
 		remove (path);
 		g_free (path);
 
-		path = g_strdup_printf ("/%s/%s/%s.dat", data_dir, group->server->name, group->name);
+		path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
 		remove (path);
 		g_free (path);
 	}
