@@ -309,7 +309,7 @@ gnksa_check_domain (const char * domain)
 	/* count the labels */
 	label_qty = 0;
 	pch = domain;
-	while (get_next_token_range (pch, '.', &pch, &label, &label_len))
+	while (get_next_token_run (pch, '.', &pch, &label, &label_len))
 		++label_qty;
 
 	/* make sure we have more than one label in the domain */
@@ -320,7 +320,7 @@ gnksa_check_domain (const char * domain)
 	pch = domain;
 	for (i=0; i<label_qty-1; ++i)
 	{
-		get_next_token_range (pch, '.', &pch, &label, &label_len);
+		get_next_token_run (pch, '.', &pch, &label, &label_len);
 		if (label_len > 63)
 			return GNKSA_ILLEGAL_LABEL_LENGTH;
 		if (label[0]=='-' || label[label_len-1]=='-')
@@ -328,12 +328,12 @@ gnksa_check_domain (const char * domain)
 	}
 
 	/* last label -- toplevel domain */
-	top = get_next_token_range (pch, '.', &pch, &label, &label_len);
+	top = get_next_token_run (pch, '.', &pch, &label, &label_len);
 	debug1 (DEBUG_GNKSA, "top level domain: [%s]", top);
 	switch ((int)strlen(top))
 	{
 		case 1:
-			if (isdigit((int)top[0]))
+			if (isdigit((guchar)top[0]))
 				return gnksa_check_domain_literal (domain);
 
 			/* single-letter TLDs dont exist */
@@ -341,11 +341,11 @@ gnksa_check_domain (const char * domain)
 			break;
 
 		case 2:
-			if (isdigit((int)top[0]) || isdigit((int)top[1]))
+			if (isdigit((guchar)top[0]) || isdigit((guchar)top[1]))
 				return gnksa_check_domain_literal (domain);
 			break;
 		case 3:
-			if (isdigit((int)top[0]) || isdigit((int)top[2]) || isdigit((int)top[3]))
+			if (isdigit((guchar)top[0]) || isdigit((guchar)top[2]) || isdigit((guchar)top[3]))
 				return gnksa_check_domain_literal (domain);
 			break;
 		default:
@@ -372,7 +372,7 @@ gnksa_check_localpart (const char * localpart)
 
 	/* break localpart up into its unquoted words */
 	pch = localpart;
-	while (get_next_token_range (pch, '.', &pch, &word, &word_len))
+	while (get_next_token_run (pch, '.', &pch, &word, &word_len))
 	{
 		gint i;
 
@@ -928,8 +928,10 @@ gnksa_generate_message_id_from_email_addr (const gchar * addr)
 		else
 			++pch;
 	}
-	if (!is_nonempty_string(pch))
+	if (!is_nonempty_string(pch)) {
+		g_warning (_("No email address provided; generating message-id with domain \"%s\""), default_domain);
 		pch = default_domain;
+	}
 
 	/* make a copy of the rhs of the email address and trim out the '>' */
 	domain = g_strdup (pch);
@@ -1089,3 +1091,45 @@ rfc822_date_generate (time_t secs)
 	strftime (buf, sizeof(buf), fmt, &local);
 	return g_strdup (buf);
 }
+
+/***
+****
+***/
+
+#ifndef HAVE_LOCALTIME_R
+struct tm*
+localtime_r(const time_t *const timep, struct tm *p_tm)
+{
+	struct tm * retval;
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+
+	g_static_mutex_lock (&mutex);
+	retval = localtime (timep);
+	if (retval != NULL) {
+		memcpy (p_tm, retval, sizeof(struct tm));
+		retval = p_tm;
+	}
+	g_static_mutex_unlock (&mutex);
+
+	return retval;
+}
+#endif
+
+#ifndef HAVE_GMTIME_R
+struct tm*
+gmtime_r (const time_t * const timep, struct tm * p_tm)
+{
+	struct tm * retval;
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+
+	g_static_mutex_lock (&mutex);
+	retval = gmtime (timep);
+	if (retval != NULL) {
+		memcpy (p_tm, retval, sizeof(struct tm));
+		retval = p_tm;
+	}
+	g_static_mutex_unlock (&mutex);
+
+	return retval;
+}
+#endif
