@@ -38,7 +38,7 @@
 #include <pan/base/pan-callback.h>
 #include <pan/base/server.h>
 
-static void fire_group_changed (Group *);
+static void fire_groups_changed (Group **, gint qty);
 static void fire_group_cleared (Group *);
 static void fire_group_marked_read (Group *, gboolean read);
 
@@ -47,26 +47,27 @@ static void fire_group_marked_read (Group *, gboolean read);
 **/
 
 Group*
-group_new (const char * name)
+group_new (Server * server, const char * name)
 {
-	Group * g = g_new (Group, 1);
-	debug1 (DEBUG_PAN_OBJECT, "group_new: %p", g);
+	Group * group;
+	
+	g_return_val_if_fail (server!=NULL, NULL);
 	pan_warn_if_fail (is_nonempty_string(name));
-	group_constructor (g, group_destructor, name);
-	return g;
+
+	group = server_alloc_new_group (server);
+	group_constructor (group, name);
+	return group;
 }
 
 void
 group_constructor (Group                * g,
-                   PanObjectDestructor    dtor,
 		   const char           * name)
 {
 	pan_warn_if_fail (g!=NULL);
 	pan_warn_if_fail (is_nonempty_string(name));
 
-	pan_object_constructor (PAN_OBJECT(g), dtor);
-
 	debug1 (DEBUG_PAN_OBJECT, "group_constructor: %p", g);
+	g->article_mutex = g_mutex_new ();
 	g->articles_dirty = FALSE;
 	g->articles_threading_dirty = FALSE;
 	g->flags = (gint8)0;
@@ -81,21 +82,28 @@ group_constructor (Group                * g,
 	g->article_high = (gulong)0;
 	g->name = g_strdup (name);
 	g->filter_name = NULL;
+	g->identity_name = NULL;
 	g->description = NULL;
 	g->readable_name = NULL;
 	g->download_dir = NULL;
 	g->chunk = NULL;
-	g->server = NULL;
 
 	g->_articles_refcount = (gint)0;
+	g->_article_chunk = NULL;
 	g->_newsrc = NULL;
 	g->_purged = NULL;
 	g->_articles = NULL;
 	g->_one_big_chunk = NULL;
 }
 
+static void
+destroy_articles_ghfunc (gpointer key, gpointer val, gpointer user_data)
+{
+	article_destructor (ARTICLE(val));
+}
+
 void
-group_destructor (PanObject * o)
+group_destructor (Group * o)
 {
 	Group * g = GROUP(o);
 
@@ -109,6 +117,7 @@ group_destructor (PanObject * o)
 	replace_gstr (&g->readable_name, NULL);
 	replace_gstr (&g->download_dir, NULL);
 	replace_gstr (&g->filter_name, NULL);
+	replace_gstr (&g->identity_name, NULL);
 
 	/* newsrc */
 	if (g->_newsrc != NULL)
@@ -119,16 +128,20 @@ group_destructor (PanObject * o)
 	/* articles */
 	pan_warn_if_fail (g->_articles_refcount >= 0);
 	replace_gstr (&g->_one_big_chunk, NULL);
+	if (g->_articles != NULL) {
+		g_hash_table_foreach (g->_articles, destroy_articles_ghfunc, NULL);
+		g_hash_table_destroy (g->_articles);
+		g->_articles = NULL;
+	}
 	if (g->chunk != NULL) {
 		g_string_chunk_free (g->chunk);
 		g->chunk = NULL;
 	}
-	if (g->_articles != NULL) {
-		g_hash_table_destroy (g->_articles);
-		g->_articles = NULL;
+	if (g->_article_chunk != NULL) {
+		g_mem_chunk_destroy (g->_article_chunk);
+		g->_article_chunk = NULL;
 	}
-
-	pan_object_destructor (o);
+	g_mutex_free (g->article_mutex);
 }
 
 /***
@@ -151,31 +164,50 @@ group_set_new (Group * group, gboolean is_new)
 gboolean 
 group_is_new (const Group * group)
 {
-	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(group), FALSE);
+
 	return (group->flags & GROUP_NEW) ? 1 : 0;
 }
 
 void
-group_set_subscribed (Group * group, gboolean subscribed)
+groups_set_subscribed (Group ** groups, gint qty, gboolean subscribed)
 {
+	gint i;
 	ServerGroupsType type;
+	Server * server = NULL;
 
-	g_return_if_fail (group != NULL);
+	/* sanity clause */
+	g_return_if_fail (groups != NULL);
+	g_return_if_fail (qty > 0);
 
 	type = SERVER_GROUPS_SUBSCRIBED|SERVER_GROUPS_UNSUBSCRIBED;
-	if (group->server != NULL)
-		server_set_group_type_dirty (group->server, type);
 
-	if (subscribed)
-		group_set_flags (group, group->flags|GROUP_SUBSCRIBED);
-	else
-		group_set_flags (group, group->flags&~GROUP_SUBSCRIBED);
+	for (i=0; i<qty; ++i)
+	{
+		Group * group = groups[i];
+
+		if (group->server != NULL)
+		{
+			server = group->server;
+			server_set_group_type_dirty (server, type);
+		}
+
+		if (subscribed)
+			group_set_flags (group, group->flags|GROUP_SUBSCRIBED);
+		else
+			group_set_flags (group, group->flags&~GROUP_SUBSCRIBED);
+	}
+
+	if (server != NULL)
+		server_save_grouplist_if_dirty (server, NULL);
+
+	fire_groups_changed (groups, qty);
 }
 
 gboolean 
 group_is_subscribed (const Group * group)
 {
-	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(group), FALSE);
 
 	return (group->flags & GROUP_SUBSCRIBED) ? 1 : 0;
 }
@@ -183,7 +215,7 @@ group_is_subscribed (const Group * group)
 void
 group_set_dirty (Group * group)
 {
-	g_return_if_fail (group != NULL);
+	g_return_if_fail (group_is_valid(group));
 
 	if (group->server != NULL)
 		server_set_group_type_dirty (group->server, server_get_group_type (group));
@@ -192,7 +224,7 @@ group_set_dirty (Group * group)
 void
 group_set_download_dir (Group * group, const gchar* download_dir)
 {
-	g_return_if_fail (group!=NULL);
+	g_return_if_fail (group_is_valid(group));
 
 	replace_gstr (&group->download_dir, g_strdup(download_dir));
 }
@@ -200,7 +232,7 @@ group_set_download_dir (Group * group, const gchar* download_dir)
 void
 group_set_flags (Group * group, guint flags)
 {
-	g_return_if_fail (group!=NULL);
+	g_return_if_fail (group_is_valid(group));
 
 	group->flags = flags;
 	group_set_dirty (group);
@@ -212,7 +244,7 @@ group_set_filter (Group         * group,
                   gulong          filter_show,
                   const gchar   * filter_name)
 {
-	g_return_if_fail (group!=NULL);
+	g_return_if_fail (group_is_valid(group));
 
 	group->filter_bits = filter_bits;
 	group->filter_show = filter_show;
@@ -221,6 +253,17 @@ group_set_filter (Group         * group,
 	group_set_dirty (group);
 }
 
+void
+group_set_identity (Group       * group,
+                    const gchar * identity_name)
+{
+	g_return_if_fail (group!=NULL);
+
+	replace_gstr (&group->identity_name, g_strdup (identity_name));
+
+	group_set_dirty (group);
+}
+
 void
 group_set_sort_style (Group * group, int sort_style)
 {
@@ -235,7 +278,7 @@ group_set_article_range (Group    * g,
                          gulong     article_low,
                          gulong     article_high)
 {
-	g_return_if_fail (g!=NULL);
+	g_return_if_fail (group_is_valid(g));
 
 	g->article_low = article_low;
 	g->article_high = article_high;
@@ -250,7 +293,7 @@ group_get_article_range (const Group   * g,
                          gulong        * article_low,
                          gulong        * article_high)
 {
-	g_return_if_fail (g!=NULL);
+	g_return_if_fail (group_is_valid(g));
 	g_return_if_fail (article_low!=NULL);
 	g_return_if_fail (article_high!=NULL);
 
@@ -263,35 +306,39 @@ void
 group_mark_new_article_number (Group         * group,
                                gulong          everything_above_this_is_new)
 {
+	g_return_if_fail (group_is_valid(group));
+
 	group->article_high_old = everything_above_this_is_new;
 }
 
 void
 group_set_article_qty (Group * group, gint article_qty)
 {
-	g_return_if_fail (group != NULL);
+	g_return_if_fail (group_is_valid(group));
 
 	if (article_qty != group->article_qty)
 	{
 		group->article_qty = article_qty;
 		group_set_dirty (group);
-		fire_group_changed (group);
+		fire_groups_changed (&group, 1);
 	}
 }
 
 void
 group_set_article_read_qty (Group * group, gint article_read_qty)
 {
-	g_return_if_fail (group != NULL);
+	g_return_if_fail (group_is_valid(group));
 
 	article_read_qty = MAX (article_read_qty, 0);
 	group->article_read_qty = article_read_qty;
 	group_set_dirty (group);
-	fire_group_changed (group);
+	fire_groups_changed (&group, 1);
 }
 void
 group_inc_article_read_qty (Group * group, gint inc)
 {
+	g_return_if_fail (group_is_valid(group));
+
 	if (inc != 0)
 		group_set_article_read_qty (group, group->article_read_qty + inc);
 }
@@ -301,7 +348,7 @@ group_mark_all_read (Group * g, gboolean read)
 {
 	Newsrc * newsrc;
 
-	g_return_if_fail (g != NULL);
+	g_return_if_fail (group_is_valid(g));
 
 	newsrc = group_get_newsrc (g);
 	newsrc_set_group_range (newsrc, g->article_low, g->article_high);
@@ -313,27 +360,51 @@ group_mark_all_read (Group * g, gboolean read)
 gboolean
 group_is_read_only (const Group * group)
 {
-	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(group), FALSE);
+
 	return group->permission == 'n';
 }
 
 gboolean
 group_is_moderated (const Group * group)
 {
-	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(group), FALSE);
+
 	return group->permission == 'm';
 }
 
+gboolean
+group_is_valid (const Group * group)
+{
+	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group->server!=NULL, FALSE);
+	g_return_val_if_fail (group->article_mutex!=NULL, FALSE);
+	g_return_val_if_fail (is_nonempty_string(group->name), FALSE);
+
+	return TRUE;
+}
+
+gboolean
+group_is_group (const Group * group)
+{
+	g_return_val_if_fail (group_is_valid(group), FALSE);
+
+	return group->flags & GROUP_FOLDERS ? 0 : 1;
+}
+
 gboolean
 group_is_folder (const Group * group)
 {
-	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(group), FALSE);
+
 	return group->flags & GROUP_FOLDERS ? 1 : 0;
 }
 
 void
 group_set_is_folder (Group * group, gboolean folder)
 {
+	g_return_if_fail (group_is_valid(group));
+
 	if (!folder != !group_is_folder(group))
 	{
 		if (folder)
@@ -350,27 +421,23 @@ group_set_is_folder (Group * group, gboolean folder)
 ****
 ***/
 
-static gboolean
-real_group_empty_ghrfunc (gpointer key, gpointer value, gpointer data)
-{
-	pan_object_unref (PAN_OBJECT(value));
-	return TRUE;
-}
-
 void
 group_empty (Group * group, gboolean clear_counts)
 {
-	g_return_if_fail (group != NULL);
+	g_return_if_fail (group_is_valid(group));
 
-	/* unload group (if it's loaded) */
 	fire_group_cleared (group);
-
-	/* clear out the article headers */
 	file_headers_destroy (group);
 
-	/* clear out the articles... */
-	if (group->_articles != NULL)
-		g_hash_table_foreach_remove (group->_articles, real_group_empty_ghrfunc, NULL);
+	if (group->_articles != NULL) {
+		g_hash_table_foreach (group->_articles, destroy_articles_ghfunc, NULL);
+		g_hash_table_destroy (group->_articles);
+		group->_articles = NULL;
+	}
+	if (group->_article_chunk != NULL) {
+		g_mem_chunk_destroy (group->_article_chunk);
+		group->_article_chunk = NULL;
+	}
 	group_set_article_qty (group, 0);
 	group_set_article_read_qty (group, 0);
 
@@ -392,16 +459,9 @@ group_empty (Group * group, gboolean clear_counts)
 static void
 group_add_articles_ghfunc (gpointer key, gpointer val, gpointer data)
 {
-	guint i;
 	Group * group = GROUP(key);
 	GPtrArray * articles = (GPtrArray*)val;
 
-	/* this is so that unreffing from the old group doesn't kill
-	   the article -- this ref, plus group_remove_article's unref,
-	   has a net change of zero */
-	for (i=0; i!=articles->len; ++i)
-		pan_object_ref (PAN_OBJECT(g_ptr_array_index(articles,i)));
-
 	/* remove the articles from the other group */
 	group_remove_articles (group, articles);
 
@@ -433,7 +493,7 @@ group_add_articles_remove_unused (Group            * group,
 	GPtrArray * used;
 
 	/* entry assertions */
-	g_return_if_fail (group != NULL);
+	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (articles != NULL);
 	g_return_if_fail (articles->len != 0);
 
@@ -444,7 +504,6 @@ group_add_articles_remove_unused (Group            * group,
 
 	/* destroy unused articles & remove from the incoming array */
 	pan_g_ptr_array_assign (articles, used->pdata, used->len);
-	pan_g_ptr_array_foreach (ignored, (GFunc)pan_object_unref, NULL);
 
 	/* cleanup */
 	g_ptr_array_free (ignored, TRUE);
@@ -471,11 +530,12 @@ group_add_articles (Group             * group,
 	debug_enter ("group_add_articles");
 
 	/* sanity checks */
-	g_return_if_fail (group!=NULL);
+	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (group->_articles_refcount>=0);
 	g_return_if_fail (articles!=NULL);
 	if (!articles->len) return;
 	g_return_if_fail (articles->len>0);
+	g_return_if_fail (articles_are_valid ((const Article**)articles->pdata, articles->len));
 
 	/* warn the caller if they're doing something inefficient by adding
 	   articles to a group whose refcount is zero.  The old articles in
@@ -533,8 +593,6 @@ group_add_articles (Group             * group,
 
 	low = group->article_low;
 	high = group->article_high;
-	ours = group_get_articles (group);
-	old_qty = (int) g_hash_table_size (ours);
 
 	/**
 	***  Now try to add the articles that the client passed in
@@ -545,6 +603,10 @@ group_add_articles (Group             * group,
 	newsrc = group_get_newsrc (group);
 	purged = group_get_purged (group);
 	pan_g_ptr_array_reserve (tmp_used, articles->len);
+
+	g_mutex_lock (group->article_mutex);
+	ours = group_get_articles (group);
+	old_qty = (int) g_hash_table_size (ours);
 	for (i=0; i!=articles->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
@@ -569,6 +631,7 @@ group_add_articles (Group             * group,
 			g_ptr_array_add (ignored, a);
 		}
 	}
+	g_mutex_unlock (group->article_mutex);
 
 	/**
 	***  Now update our stats if articles were added
@@ -587,7 +650,7 @@ group_add_articles (Group             * group,
 			new_read_qty += group->article_read_qty;
 		group_set_article_read_qty (group, new_read_qty);
 		group_set_articles_dirty (group);
-		fire_group_changed (group);
+		fire_groups_changed (&group, 1);
 
 		pan_callback_call (group_get_articles_added_callback(), group, tmp_used);
 	}
@@ -611,24 +674,25 @@ group_set_articles_dirty (Group * group)
 }
 
 void
-group_thread_if_needed (Group * group, StatusItem * status)
+group_thread_if_needed (Group * group)
 {
-	debug_enter ("group_thread_if_needed");
+	g_return_if_fail (group_is_valid(group));
 
-	g_return_if_fail (group != NULL);
+	debug_enter ("group_thread_if_needed");
+	g_mutex_lock (group->article_mutex);
 
 	if (group->articles_threading_dirty && group->_articles!=NULL && g_hash_table_size(group->_articles)!=0)
 	{
 		GPtrArray * a;
 
 		a = group_get_article_array (group);
-		thread_articles (a, status);
-		check_multipart_articles (a);
+		thread_articles (a);
 		group->articles_threading_dirty = FALSE;
 
 		g_ptr_array_free (a, TRUE);
 	}
 
+	g_mutex_unlock (group->article_mutex);
 	debug_exit ("group_thread_if_needed");
 }
 
@@ -636,20 +700,58 @@ group_thread_if_needed (Group * group, StatusItem * status)
 gpointer
 group_get_article_by_message_id (Group * group, const gchar * message_id)
 {
-	g_return_val_if_fail (group!=NULL, NULL);
+	g_return_val_if_fail (group_is_valid(group), NULL);
 	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+	g_return_val_if_fail (*message_id=='<', NULL);
+
 	return g_hash_table_lookup (group_get_articles(group), message_id);
 }
 
 void
 group_remove_article (Group * group, Article * article)
 {
-	GPtrArray * a = g_ptr_array_new ();
+	GPtrArray * a;
+
+	g_return_if_fail (group_is_valid(group));
+	g_return_if_fail (article_is_valid(article));
+
+       	a = g_ptr_array_new ();
 	g_ptr_array_add (a, article);
 	group_remove_articles (group, a);
 	g_ptr_array_free (a, TRUE);
 }
 
+static void
+group_isolate_article (Group * g, Article * a)
+{
+	GSList *l = NULL;
+	Article * parent;
+
+	g_return_if_fail (group_is_valid (g));
+	g_return_if_fail (article_is_valid (a));
+	g_return_if_fail (a->group == g);
+	
+	parent = a->parent;
+
+	g_mutex_lock (g->article_mutex);
+
+	if (parent)
+		parent->threads = g_slist_remove (parent->threads, a);
+
+	for (l=a->threads; l!=NULL; l=l->next) {
+		Article *child = ARTICLE(l->data);
+		child->parent = parent;
+		if (parent)
+			parent->threads = g_slist_append (parent->threads, child);
+	}
+
+	g_slist_free (a->threads);
+	a->threads = NULL;
+	a->parent = NULL;
+
+	g_mutex_unlock (g->article_mutex);
+}
+
 void
 group_remove_articles (Group * group, const GPtrArray * articles)
 {
@@ -660,6 +762,8 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 	GHashTable * ours;
 	debug_enter ("group_remove_articles");
 
+	g_return_if_fail (group_is_valid(group));
+
 	/* sanity clause */
 	g_return_if_fail (group != NULL);
 	g_return_if_fail (articles != NULL);
@@ -676,13 +780,14 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 	for (i=0; i!=tmp_remove->len; ++i) {
 		Article * article = ARTICLE(g_ptr_array_index(tmp_remove,i));
 		if (PAN_OBJECT(article)->ref_count == 1) /* about to die */
-			article_isolate (article);
+			group_isolate_article (article->group, article);
 	}
 
 	/**
 	***  Try to remove the articles
 	**/
 
+	g_mutex_lock (group->article_mutex);
 	ours = group_get_articles (group);
 	removed_total = 0;
 	removed_read = 0;
@@ -699,11 +804,9 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 			++removed_total;
 			if (article_is_read (our_a))
 				++removed_read;
-
-			our_a->group = NULL;
-			pan_object_unref (PAN_OBJECT(our_a));
 		}
 	}
+	g_mutex_unlock (group->article_mutex);
 
 	/**
 	***  Update the stats
@@ -716,7 +819,7 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 		group->article_qty -= removed_total;
 		group->article_read_qty -= removed_read;
 		group_set_articles_dirty (group);
-		fire_group_changed (group);
+		fire_groups_changed (&group, 1);
 	}
 
 	/* cleanup */
@@ -732,7 +835,8 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 	GPtrArray * removeme;
 	debug_enter ("group_expire_articles_not_in_range");
 
-	g_return_if_fail (g != NULL);
+	/* sanity clause */
+	g_return_if_fail (group_is_valid(g));
 
 	/* find old articles to remove */
 	tmp = group_get_article_array (g);
@@ -771,41 +875,44 @@ group_ref_articles (Group * group, StatusItem * status)
 	debug_enter ("group_ref_articles");
 
 	/* sanity clause */
-	g_return_if_fail (group != NULL);
+	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (group->_articles_refcount >= 0);
 
-	/* if we're up from zero, load the headers */
-	if (++group->_articles_refcount == 1)
+	/* if we're up from zero, load the headers
+	   odebug2 ("group %s refcount about to be incremented to %d", group->name, group->_articles_refcount+1); */
+	if (++group->_articles_refcount == 1) {
 		file_headers_load (group, status);
+		group_set_article_qty (group, g_hash_table_size(group->_articles));
+	}
+	/* odebug2 ("group %s refcount incremented to %d", group->name, group->_articles_refcount); */
 
-	/* g_message ("group %s refcount inc to %d", group->name, group->_articles_refcount); */
-	group_set_article_qty (group, g_hash_table_size(group->_articles));
 	debug_exit ("group_ref_articles");
 }
 
-static void
-group_unref_articles_ghfunc (gpointer key, gpointer val, gpointer data)
-{
-	pan_object_unref (PAN_OBJECT(val));
-}
 void
 group_unref_articles (Group * group, StatusItem * status)
 {
 	debug_enter ("group_unref_articles");
 
-	/* sanity clause */
-	g_return_if_fail (group != NULL);
+	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (group->_articles_refcount > 0);
 
-	/* if down to zero, save & flush the articles */
-	if (!--group->_articles_refcount)
-	{
-		file_headers_save (group, status);
+	g_mutex_lock (group->article_mutex);
 
-		/* free the articles */
-		g_hash_table_foreach (group->_articles, group_unref_articles_ghfunc, NULL);
-		g_hash_table_destroy (group->_articles);
-		group->_articles = NULL;
+	if (group->_articles_refcount == 1)
+	{
+		file_headers_save_noref (group, NULL);
+		server_save_grouplist_if_dirty (group->server, NULL);
+
+		if (group->_articles!=NULL) {
+		       	g_hash_table_foreach (group->_articles, destroy_articles_ghfunc, NULL);
+			g_hash_table_destroy (group->_articles);
+			group->_articles = NULL;
+		}
+		if (group->_article_chunk != NULL) {
+			g_mem_chunk_destroy (group->_article_chunk);
+			group->_article_chunk = NULL;
+		}
 
 		/* free the memory used by those articles */
 		replace_gstr (&group->_one_big_chunk, NULL);
@@ -815,7 +922,10 @@ group_unref_articles (Group * group, StatusItem * status)
 		}
 	}
 
-	/* g_message ("group %s refcount dec to %d", group->name, group->_articles_refcount); */
+	--group->_articles_refcount;
+	/* odebug2 ("group %s article refcount decremented to %d", group->name, (int)group->_articles_refcount); */
+	g_mutex_unlock (group->article_mutex);
+
 	debug_exit ("group_unref_articles");
 }
 
@@ -824,24 +934,27 @@ group_unref_articles (Group * group, StatusItem * status)
 **/
 
 
+void
+group_set_readable_name (Group         * group,
+	                 const gchar   * readable_name)
+{
+	g_return_if_fail (group_is_valid(group));
+
+	replace_gstr (&group->readable_name, g_strdup(readable_name));
+
+	group_set_dirty (group);
+	fire_groups_changed (&group, 1);
+}
+
+
 const gchar*
 group_get_readable_name (const Group * group)
 {
-	const gchar * retval;
+	g_return_val_if_fail (group_is_valid(group), _("(No Group)"));
 
-	if (group == NULL)
-	{
-		pan_warn_if_reached ();
-		retval = _("(No Group)");
-	}
-	else
-	{
-		retval = is_nonempty_string (group->readable_name)
-			? group->readable_name
-			: group->name;
-	}
-
-	return retval;
+	return is_nonempty_string (group->readable_name)
+		? group->readable_name
+		: group->name;
 }
 
 /**
@@ -855,7 +968,7 @@ group_chunk_string (Group         * group,
 {
 	gchar * retval = NULL;
 
-	g_return_val_if_fail (group!=NULL, NULL);
+	g_return_val_if_fail (group_is_valid(group), NULL);
 
 	if (chunkme != NULL)
 	{
@@ -881,7 +994,7 @@ group_chunk_string (Group         * group,
 Newsrc*
 group_get_newsrc (Group * group)
 {
-	g_return_val_if_fail (group!=NULL, NULL);
+	g_return_val_if_fail (group_is_valid(group), NULL);
 
 	if (group->_newsrc == NULL)
 		group->_newsrc = newsrc_new (NULL, group->article_low, group->article_high);
@@ -892,7 +1005,8 @@ group_get_newsrc (Group * group)
 gboolean
 group_has_newsrc (const Group * group)
 {
-	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(group), FALSE);
+
 	return group->_newsrc != NULL;
 }
 
@@ -914,7 +1028,7 @@ group_has_newsrc (const Group * group)
 Newsrc*
 group_get_purged (Group * group)
 {
-	g_return_val_if_fail (group!=NULL, NULL);
+	g_return_val_if_fail (group_is_valid(group), NULL);
 
 	if (group->_purged == NULL)
 		group->_purged = newsrc_new (NULL, group->article_low, group->article_high);
@@ -937,8 +1051,7 @@ group_remove_crossposts (Group     * g,
 		         gulong    * numbers,
                          gint        number_qty)
 {
-
-	g_return_if_fail (g!=NULL);
+	g_return_if_fail (group_is_valid(g));
 	g_return_if_fail (numbers!=NULL);
 	g_return_if_fail (number_qty>0);
 
@@ -971,7 +1084,6 @@ group_remove_crossposts (Group     * g,
 
 			if (exact_match) {
 				Article * a = ARTICLE(g_ptr_array_index(articles,index));
-				odebug2 ("group [%s] removing crosspost [%s]", g->name, a->subject);
 				g_ptr_array_add (remove, a);
 			}
 		}
@@ -991,18 +1103,30 @@ group_remove_crossposts (Group     * g,
 		
 		n = group_get_purged (g);
 		for (i=0; i<number_qty; ++i) {
-			odebug2 ("group [%s] is removing xref [%lu]", g->name, numbers[i]);
 			newsrc_mark_article (n, numbers[i], TRUE);
 		}
 	}
 }
 
+struct _Article*
+group_alloc_new_article (Group * group)
+{
+	Article * a;
+	g_return_val_if_fail (group_is_valid(group), NULL);
+
+	if (group->_article_chunk == NULL)
+		group->_article_chunk = g_mem_chunk_create (Article, 200, G_ALLOC_ONLY);
+
+	a =  g_chunk_new0 (Article, group->_article_chunk);
+	a->group = group;
+	return a;
+}
 
 GHashTable*
 group_get_articles (Group * group)
 {
-	g_return_val_if_fail (group!=NULL, NULL);
-
+	g_return_val_if_fail (group_is_valid(group), NULL);
+ 
 	if (group->_articles == NULL)
 		group->_articles = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -1017,11 +1141,12 @@ group_get_article_array (Group * group)
 
 	/* sanity clause */
 	retval = g_ptr_array_new ();
-	g_return_val_if_fail (group!=NULL, retval);
+	g_return_val_if_fail (group_is_valid(group), retval);
 
 	/* populate retval */
 	hash = group_get_articles (group);
 	pan_hash_to_ptr_array (hash, retval);
+	pan_warn_if_fail (articles_are_valid ((const Article**)retval->pdata, retval->len));
 
 	return retval;
 }
@@ -1037,7 +1162,7 @@ group_get_article_array (Group * group)
 **/
  
 PanCallback*
-group_get_group_changed_callback (void)
+group_get_groups_changed_callback (void)
 {
 	static PanCallback * cb = NULL;
 	if (cb==NULL) cb = pan_callback_new ();
@@ -1045,11 +1170,11 @@ group_get_group_changed_callback (void)
 }
 
 static void
-fire_group_changed (Group * g)
+fire_groups_changed (Group ** g, gint qty)
 {
 	g_return_if_fail (g!=NULL);
 
-	pan_callback_call (group_get_group_changed_callback(), g, NULL);
+	pan_callback_call (group_get_groups_changed_callback(), g, GINT_TO_POINTER(qty));
 }
 
 PanCallback*
