@@ -42,6 +42,10 @@ static void fire_online_status_changed (Server*, gboolean online);
 static void fire_groups_added          (Server*, Group**, gint group_qty);
 static void fire_groups_removed        (Server*, Group**, gint group_qty);
 
+/* this option for turning off saving exists for the regression tests,
+ * which need to be able to turn it off */
+gint server_saving_enabled = TRUE;
+
 /**
 ***  Server Object Life Cycle
 **/
@@ -84,7 +88,7 @@ server_destructor (PanObject *obj)
 	Server * server = SERVER(obj);
 
 	/* clear out the server bits: groups */
-	server_save_dirty_groups (server, NULL);
+	server_save_if_dirty (server, NULL);
 	for (i=0; i!=server->_groups->len; ++i)
 		group_destructor (GROUP(g_ptr_array_index(server->_groups,i)));
 	g_ptr_array_free (server->_groups, TRUE);
@@ -238,26 +242,40 @@ server_set_group_type_dirty (Server * server, ServerGroupsType type)
 }
 
 void
-server_save_dirty_groups (Server * server, StatusItem * status)
+server_save_grouplist_if_dirty (Server * server, StatusItem * status)
 {
-	guint i;
-	debug_enter ("server_save_dirty_groups");
-
 	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_saving_enabled);
 
 	if (server->_groups_dirty)
 	{
 		file_grouplist_save (server, server->_groups_dirty, status);
 		server->_groups_dirty = FALSE;
 	}
+}
+
+void
+server_save_if_dirty (Server * server, StatusItem * status)
+{
+	guint i;
+	debug_enter ("server_save_if_dirty");
+
+	g_return_if_fail (server!=NULL);
+
+	server_save_grouplist_if_dirty (server, status);
 
 	for (i=0; i<server->_groups->len; ++i)
 	{
 		Group * group = GROUP(g_ptr_array_index(server->_groups,i));
 		if (group->articles_dirty)
+		{
+			group_ref_articles (group, NULL);
 			file_headers_save (group, status);
-	}                                                                                                               
-	debug_exit ("server_save_dirty_groups");
+			group_unref_articles (group, NULL);
+		}
+	}
+
+	debug_exit ("server_save_if_dirty");
 }
 
 /**
@@ -397,6 +415,9 @@ server_add_groups (Server       * server,
 			pan_g_ptr_array_assign (fillme_used, added->pdata, added->len);
 	}
 
+	/* save them for safe-keeping */
+	server_save_grouplist_if_dirty (server, NULL);
+
 	/* cleanup */
 	g_hash_table_destroy (old);
 	g_ptr_array_free (added, TRUE);
