@@ -61,8 +61,6 @@ dialog_about (GtkWidget * window)
 	                      _("Programmer"));
 	pan_about_add_author (a, "Adam Huffman", "bloch@verdurin.com",
 	                      _("Programmer & Documentation"));
-	pan_about_add_author (a, "Tov are Jacobsen", "tovj@stud.ntnu.no",
-	                      _("Translation Coordinator"));
 	pan_about_add_author (a, "Matt Eagleson", "e.Messiah@superpimp.org",
 	                      _("Project Originator, Programmer"));
 	pan_about_add_author (a, "Jason Leach", "jleach@ximian.com",
@@ -110,17 +108,17 @@ dialog_about (GtkWidget * window)
 static gchar *
 get_credits_translators (void)
 {
-	return g_strdup ("\n\tAntonio de la Torre \n\tJarmo Kielosto \n\tAndrs Tmr \n\tMarcel Telka \n\tChristian Rose");
+	return g_strdup ("\n\tChinese - zipeco \n\tEstonian - Tivo Leedjrv \n\tFinnish - Jarmo Kielosto \n\tGalician - Jess Bravo lvarez \n\tHungarian - Andras Timar \n\tJapanese - Takeshi Aihana \n\tKorean - Nam SungHyun \n\tPolish - GNOME PL Team \n\tPortugese - Duarte Loreto \n\tRussian - Valek Filippov \n\tSlovak - Marcel Telka \n\tSwedish - Christian Rose");
 }
 
 static gchar *
 get_credits_code (void)
 {
-	return g_strdup ("\n\tVolodymyr M . Lisivka\n\tFredrik Pettersson\n\tMike Robinson\n\tOwen Taylor");
+	return g_strdup ("\n\tJacob Ils Christensen \n\tRichard Deken \n\tThomas Gamble \n\tMax Horn \n\tChristian Kuhtz \n\tTivo Leedjrv \n\tDerek W. Poon \n\tRick \n\tMike Robinson \n\tJoseph Turian");
 }
 
 static gchar *
 get_credits_users (void)
 {
-	return g_strdup ("\n\tAnubis \n\tJed S. Baer \n\tGavin Baker \n\tSteven Behnke \n\tAdrian Von Bidder \n\tCN \n\tDuncan Coutts \n\tRyan Daniels \n\tTrey Egan \n\tPaolo Fiore \n\tJim Henderson \n\tDave Hill \n\tPhilip Van Hoof \n\tSimon Jester \n\tFrancesco Lamonica \n\tLenny \n\tShawn Lindsay \n\tToralf Lund \n\tMatt \n\tmdew \n\tJohn Mellor \n\tDavid Oberbeck \n\tHuub Reuver \n\tNoah Romer \n\tB J Rosen \n\tAlan Schmidt \n\tFlorian Schmidt \n\tT'aZ \n\tClark Tompsett \n\tWilliam Shotts \n\tDavid Stuart \n\tSam Simpson \n\tC Sullivan \n\tJohn Summerfield \n\tTester \n\tMystic Zen Biker Twigboy \n\tWanderer \n\tAndy Zbikowski \n\tTaylor James Zingsheim");
+	return g_strdup ("\n\tJed S. Baer \n\tVladimir Berezniker \n\tDouglas Bollinger \n\tPeter Borgmann \n\tGabi Davar \n\tCory Dodt \n\tAlex V Flinsch \n\tErik W Hanson \n\tJim Henderson \n\tJim22 \n\tNam SungHyun \n\tKevin \n\tKosmo \n\tJoris Lambrecht \n\tFrancesco Lamonica \n\tSebastien Leclair \n\tMuralito \n\tAndreas Ollinger \n\tChris Petersen \n\tTony \n\tUli Wachowitz \n\tDavid Watson \n\tMorten Welinder \n\tMartin Wulffeld \n\tAlan Young \n\tJake Zhukov ");
 }
