@@ -39,6 +39,7 @@
 #include <pan/filters/filter-binary.h>
 #include <pan/filters/filter-cached.h>
 #include <pan/filters/filter-crosspost.h>
+#include <pan/filters/filter-mine.h>
 #include <pan/filters/filter-date.h>
 #include <pan/filters/filter-linecount.h>
 #include <pan/filters/filter-new.h>
@@ -74,6 +75,7 @@ typedef struct
 	GtkWidget * phrase_menu;
 	GtkWidget * phrase_entry;
 	GtkWidget * phrase_regexp_checkbutton;
+	GtkWidget * mine_rb;
 	GtkWidget * lines_rb;
 	GtkWidget * lines_sb;
 	GtkWidget * xpost_rb;
@@ -343,6 +345,10 @@ get_new_filter (FilterEditDialog * d)
 		}
 		g_free (phrase);
 	}
+	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->mine_rb)))
+	{
+		f = filter_mine_new ();
+	}
 	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->lines_rb)))
 	{
 		f = filter_line_count_new ();
@@ -760,59 +766,66 @@ filter_edit_dialog_new (const FilterTop * top)
 	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->phrase_rb));
 	gtk_container_add (GTK_CONTAINER(w2), w);
 
+	/* mine */
+	w = d->mine_rb = gtk_radio_button_new_with_label (rb_group, "");
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
+	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article was posted by _me"));
+	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
+	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
+
 	/* lines */
 	w = d->lines_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article has at _least"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	adj = gtk_adjustment_new (1, 0, 300000, 1, 10, 10);
 	w = d->lines_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
 	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->lines_rb));
-	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 3, 4, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	w = gtk_label_new (_("lines"));
-	gtk_table_attach (GTK_TABLE(table), w, 2, 3, 2, 3, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 2, 3, 3, 4, GTK_FILL, 0, 0, 0);
 	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC(w), 0, 0.5);
 
 	/* crosspost */
 	w = d->xpost_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 4, 5, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article was _posted to at least"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	adj = gtk_adjustment_new (1, 0, 300000, 1, 10, 10);
 	w = d->xpost_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
 	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->xpost_rb));
-	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 3, 4, GTK_EXPAND|GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 4, 5, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	w = gtk_label_new (_("newsgroups"));
-	gtk_table_attach (GTK_TABLE(table), w, 2, 3, 3, 4, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 2, 3, 4, 5, GTK_FILL, 0, 0, 0);
 	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC(w), 0, 0.5);
 
 	/* age */
 	w = d->age_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 4, 5, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 5, 6, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article _was at least"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	adj = gtk_adjustment_new (1, 0, 300000, 1, 10, 10);
 	w = d->age_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
 	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->age_rb));
-	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 4, 5, GTK_EXPAND|GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 5, 6, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	w = gtk_label_new (_("days old"));
-	gtk_table_attach (GTK_TABLE(table), w, 2, 3, 4, 5, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 2, 3, 5, 6, GTK_FILL, 0, 0, 0);
 	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC(w), 0, 0.5);
 
 	/* thread */
 	w = d->thread_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 5, 6, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 6, 7, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article _thread is"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	w = gtk_option_menu_new ();
-	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 5, 6, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 6, 7, GTK_FILL, 0, 0, 0);
 	w2 = d->thread_menu = gtk_menu_new ();
 	tmp = gtk_menu_item_new_with_label (_("Watched"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_THREAD_WATCHED));
@@ -833,19 +846,19 @@ filter_edit_dialog_new (const FilterTop * top)
 
 	/* read */
 	w = d->read_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 6, 7, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 7, 8, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article is _read"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 
 	/* attachment */
 	w = d->attach_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 7, 8, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 8, 9, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article h_as"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	w = gtk_option_menu_new ();
-	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 7, 8, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 8, 9, GTK_FILL, 0, 0, 0);
 	w2 = d->attach_menu = gtk_menu_new ();
 	tmp = gtk_menu_item_new_with_label (_("Attachments"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_BINARY_COMPLETE));
@@ -866,37 +879,37 @@ filter_edit_dialog_new (const FilterTop * top)
 
 	/* cached */
 	w = d->cached_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 8, 9, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 9, 10, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article body is _cached locally"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 
 	/* new */
 	w = d->new_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 9, 10, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 10, 11, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article is _new"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 
 	/* score */
 	w = d->score_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 10, 11, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 11, 12, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article _score is at least"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	adj = gtk_adjustment_new (0, -32768, 32767, 1, 1, 1);
 	w = d->score_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
 	gtk_signal_connect_object (GTK_OBJECT(w), "focus_in_event", tb_activate, GTK_OBJECT(d->score_rb));
-	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 10, 11, GTK_EXPAND|GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 11, 12, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
 	/* saved */
 	w = d->saved_rb = gtk_radio_button_new_with_label (rb_group, "");
-	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 11, 12, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 12, 13, GTK_FILL, 0, 0, 0);
 	uint = gtk_label_parse_uline (GTK_LABEL(GTK_BIN(w)->child), _("Article has been"));
 	gtk_widget_add_accelerator (w, "clicked", accel_group, uint, GDK_MOD1_MASK, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	w = gtk_option_menu_new ();
-	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 11, 12, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 12, 13, GTK_FILL, 0, 0, 0);
 	w2 = d->saved_menu = gtk_menu_new ();
 	tmp = gtk_menu_item_new_with_label (_("Saved or Decoded"));
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(FILTER_SAVED_SAVED));
