@@ -23,6 +23,7 @@
 
 #include <gnome.h>
 
+#include <pan/base/argset.h>
 #include <pan/base/debug.h>
 #include <pan/base/file-grouplist.h>
 #include <pan/base/fnmatch.h>
@@ -37,18 +38,17 @@
 #include <pan/gui.h>
 #include <pan/prefs.h>
 #include <pan/queue.h>
-#include <pan/task-group-count.h>
-#include <pan/task-bodies.h>
-#include <pan/task-headers.h>
 #include <pan/task-grouplist.h>
 #include <pan/util.h>
 
 #include "xpm/folder.xpm"
 
-static void grouplist_update_ui_thread (void* data);
+static gint grouplist_update_ui_mainthread (gpointer);
 static void group_menu_popup (GdkEventButton * event);
-static void grouplist_filter_changed_cb (void);
-static void grouplist_refresh_filter (void);
+static void grouplist_filter_changed_cb_nolock (void);
+static void grouplist_refresh_filter_nolock (void);
+static void grouplist_update_groups (const Group ** groups, int group_qty);
+static void grouplist_set_server_nolock (Server *);
 
 extern GtkWidget   * ttips;
 static GnomePixmap * folder_pixmap           = NULL;
@@ -56,7 +56,6 @@ static GnomePixmap * subscribed_pixmap       = NULL;
 
 static int           group_mode              = GROUP_ALL;
 static GtkWidget   * group_clist_menu        = NULL;
-static GtkWidget   * folder_clist_menu       = NULL;
 static Server      * my_server               = NULL;
 static GtkWidget   * grouplist_mode_menu     = NULL;
 static GtkWidget   * groupname_filter_entry  = NULL;
@@ -71,20 +70,50 @@ static GStaticMutex  grouplist_ui_mutex      = G_STATIC_MUTEX_INIT;
 PanCallback * grouplist_server_set_callback = NULL;
 PanCallback * grouplist_group_selection_changed = NULL;
 
-static gchar*
-grouplist_get_group_readable_name (const Group * group)
+static void
+grouplist_get_group_readable_name (const Group * group, gchar * buf, gint buflen)
 {
-	gchar * retval;
-	GString * str = g_string_new (group_get_readable_name(group));
+	const gchar * readable = group_get_readable_name (group);
+	gint bufleft;
 
-	if (group_is_moderated (group))
-		g_string_sprintfa (str, _(" (Moderated)"));
-	if (group_is_read_only (group))
-		g_string_sprintfa (str, _(" (Read-Only)"));
+	if (!collapse_group_names)
+		strncpy (buf, readable, buflen);
+	else
+	{
+		gint pos = 0;
+		const gchar * pch = NULL;
+		const gchar * march = readable;
+		const gchar * last = "";
+		gint len = 0;
 
-	retval = str->str;
-	g_string_free (str, FALSE);
-	return retval;
+		while (pos+2<buflen && get_next_token_run (march, '.', &march, &pch, &len)) {
+			last = pch;
+			if (len > 0) {
+				buf[pos++] = *pch;
+				buf[pos++] = '.';
+			}
+		}
+		if (pos != 0)
+			pos -= 2;
+		buf[pos] = '\0';
+
+		if (is_nonempty_string(last))
+			strncpy (buf+pos, last, buflen-pos);
+	}
+	buf[buflen-1] = '\0';
+	bufleft = buflen - strlen(buf);
+
+	if (group_is_moderated (group)) {
+		strncat (buf, _(" (Moderated)"), bufleft);
+		buf[buflen-1] = '\0';
+		bufleft = buflen - strlen(buf);
+	}
+
+	if (group_is_read_only (group)) {
+		strncat (buf, _(" (Read-Only)"), bufleft);
+		buf[buflen-1] = '\0';
+		bufleft = buflen - strlen(buf);
+	}
 }
 
 static Server *
@@ -95,12 +124,6 @@ grouplist_get_visible_server (void)
 		: my_server;
 }
 
-Server *
-grouplist_get_server (void)
-{
-	return my_server;
-}
-
 static void
 group_dialog_clicked_cb (GnomeDialog * dialog, int num, Server * server)
 {
@@ -108,56 +131,6 @@ group_dialog_clicked_cb (GnomeDialog * dialog, int num, Server * server)
 		queue_add (TASK(task_grouplist_new (server, GROUPLIST_ALL)));
 }
 
-static void
-grouplist_set_server (Server * server)
-{
-	Server * old_server = my_server;
-	debug_enter ("grouplist_set_server");
-
-
-	g_return_if_fail (server != NULL);
-	if (my_server == server)
-		return;
-
-	/* set the active list */
-	my_server = server;
-	pan_callback_call (grouplist_server_set_callback, my_server, NULL);
-
-	if (grouplist_get_visible_server() == old_server) {
-		pan_lock ();
-		gtk_clist_clear (GTK_CLIST(Pan.group_clist));
-		pan_unlock ();
-	}
-
-	if (file_grouplist_exists (my_server))
-	{
-		grouplist_update_ui (my_server);
-	}
-	else
-	{
-		GtkWidget * w;
-		gchar * s = g_strdup_printf (
-			_("We don't have a group list for server \"%s\".\n"
-			  "Shall we get one?"), server->name);
-
-		pan_lock ();
-		w = gnome_message_box_new (s,
-			GNOME_MESSAGE_BOX_QUESTION,
-			GNOME_STOCK_BUTTON_YES,
-			GNOME_STOCK_BUTTON_NO, NULL);
-		gtk_signal_connect (GTK_OBJECT(w), "clicked",
-		                    GTK_SIGNAL_FUNC(group_dialog_clicked_cb),
-				    server);
-		gnome_dialog_set_parent (GNOME_DIALOG(w),
-		                         GTK_WINDOW(Pan.window));
-		gtk_widget_show_all (w);
-		pan_unlock ();
-
-		g_free (s);
-	}
-
-	debug_exit ("grouplist_set_server");
-}
 
 /*****
 ******
@@ -187,7 +160,7 @@ grouplist_set_server (Server * server)
 static gint button_click_count = -1;
 static gint mb = -1;
 
-static gint
+static void
 grouplist_selection_changed_cb (gpointer call_object,
                                 gpointer call_arg,
                                 gpointer user_data)
@@ -206,8 +179,10 @@ grouplist_selection_changed_cb (gpointer call_object,
 					&& (  (mb==2 && grouplist_mb2_action==CLICK_ACTION_LOAD)
 					    ||(mb==1 && grouplist_mb1_action==CLICK_ACTION_LOAD)))))
 	{
-		gui_page_set (ARTICLELIST_PAGE, Pan.article_ctree);
-		articlelist_set_group_thread (single_group);
+		gui_page_set (ARTICLELIST_PAGE);
+		pan_lock ();
+		articlelist_set_group_nolock (single_group);
+		pan_unlock ();
 	}
 
 	/* reset the click count for next time... */
@@ -215,7 +190,6 @@ grouplist_selection_changed_cb (gpointer call_object,
 	mb = -1;
 
 	debug_exit ("grouplist_selection_changed_cb");
-	return 0;
 }
 
 static void
@@ -233,13 +207,6 @@ grouplist_button_press (GtkWidget *widget, GdkEventButton * event)
 				button_click_count = 1;
 			break;
 		case 3:
-#if 0
-			if (grouplist_get_selected_group() == NULL) {
-				gint row=0, col=0;
-				if (gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y, &row, &col))
-					gtk_clist_select_row (GTK_CLIST(widget), row, 0);
-			}
-#endif
 			group_menu_popup (event);
 			break;
 	}
@@ -268,10 +235,10 @@ grouplist_key_press (GtkWidget     * widget,
 static gboolean select_callback_pending = FALSE;
 
 static gint
-select_row_cb_idle (gpointer data)
+select_row_cb_mainthread (gpointer data)
 {
 	GPtrArray * groups;
-	debug_enter ("select_row_cb_idle");
+	debug_enter ("select_row_cb_mainthread");
 
        	groups = grouplist_get_selected_groups ();
 	pan_callback_call (grouplist_group_selection_changed,
@@ -280,7 +247,7 @@ select_row_cb_idle (gpointer data)
 	select_callback_pending = FALSE;
 	g_ptr_array_free (groups, TRUE);
 
-	debug_exit ("select_row_cb_idle");
+	debug_exit ("select_row_cb_mainthread");
 	return FALSE;
 }
 static void
@@ -294,7 +261,7 @@ select_row_cb (GtkWidget  * widget,
 	if (!select_callback_pending)
 	{
 		select_callback_pending = TRUE;
-		gtk_idle_add (select_row_cb_idle, NULL);
+		run_in_main_thread_nolock (select_row_cb_mainthread, NULL);
 	}
 
 	debug_exit ("select_row_cb");
@@ -304,19 +271,17 @@ select_row_cb (GtkWidget  * widget,
 ******
 *****/
 
-void
-grouplist_set_selected_groups (const GPtrArray * a)
+static void
+grouplist_set_selected_groups_nolock (const GPtrArray * a)
 {
 	GtkCList * clist;
-	debug_enter ("grouplist_set_selected_groups");
+	debug_enter ("grouplist_set_selected_groups_nolock");
 
 	if (a != NULL)
 	{
 		guint i;
 
 		clist = GTK_CLIST(Pan.group_clist);
-		g_static_mutex_lock (&grouplist_ui_mutex);
-		pan_lock ();
 		gtk_clist_freeze (clist);
 		gtk_clist_unselect_all (clist);
 		for (i=0; i!=a->len; ++i)
@@ -332,11 +297,9 @@ grouplist_set_selected_groups (const GPtrArray * a)
 			}
 		}
 		gtk_clist_thaw (clist);
-		pan_unlock ();
-		g_static_mutex_unlock (&grouplist_ui_mutex);
 	}
 
-	debug_exit ("grouplist_set_selected_groups");
+	debug_exit ("grouplist_set_selected_groups_nolock");
 }
 
 Group*
@@ -388,27 +351,6 @@ grouplist_get_selected_groups (void)
 *****/
 
 static void
-group_download_all_articles (Group * group)
-{
-	Task * task = NULL;
-	GPtrArray * a = NULL;
-
-	g_return_if_fail (group != NULL);
-	g_return_if_fail (group->_articles != NULL);
-
-	group_ref_articles (group, NULL);
-	a = group_get_article_array (group);
-	if (a->len)
-		task = TASK(task_bodies_new (group, a));
-	if (task != NULL)
-		queue_add (task);
-	group_unref_articles (group, NULL);
-
-	/* cleanup */
-	g_ptr_array_free (a, TRUE);
-}
-
-void
 grouplist_remove_row (const Group *group)
 {
 	debug_enter ("grouplist_remove_row");
@@ -429,345 +371,6 @@ grouplist_remove_row (const Group *group)
 
 	debug_exit ("grouplist_remove_row");
 }
-static void
-grouplist_update_selected_rows (void)
-{
-	GPtrArray * groups;
-	debug_enter ("grouplist_update_selected_rows");
-
-       	groups = grouplist_get_selected_groups ();
-	if (groups->len)
-		grouplist_update_groups ((const Group**)groups->pdata, groups->len);
-	g_ptr_array_free (groups, TRUE);
-
-	debug_exit ("grouplist_update_selected_rows");
-}
-static void
-grouplist_process_selected (int type)
-{
-	gint i;
-	GPtrArray * groups;
-	debug_enter ("grouplist_process_selected");
-
-	/* find out which group(s) we're to operate on */
-	groups = grouplist_get_selected_groups ();
-	if (groups->len==0) {
-		Group * group = articlelist_get_group ();
-		if (group != NULL)
-			g_ptr_array_add (groups, group);
-	}
-
-	/* iterate through the groups */
-	for (i=0; i!=groups->len; ++i)
-	{
-		Group * group = GROUP(g_ptr_array_index(groups,i));
-
-		switch (type)
-		{
-			case 1: /* subscribe */
-				group_set_subscribed (group, TRUE);
-				break;
-
-			case 2: /* unsubscribe */
-				group_set_subscribed (group, FALSE);
-				if (group_mode==GROUP_SUBSCRIBED)
-					grouplist_remove_row (group);
-				if (group->article_qty != 0)
-					group_empty_dialog (group);
-				break;
-
-			case 3: /* download all */
-				queue_add (TASK(task_headers_new(group, HEADERS_ALL)));
-				break;
-
-			case 4: /* download new */
-				queue_add (TASK(task_headers_new(group, HEADERS_NEW)));
-				break;
-
-			case 5: /* clean/empty */
-				group_empty_dialog (group);
-				break;
-
-			case 6: /* group properties */
-				dialog_group_props (group);
-				break;
-
-			case 10: /* download all bodies */
-				group_download_all_articles (group);
-				break;
-
-			default:
-				break;
-		}
-	}
-
-	/* refresh the ui */
-	grouplist_update_selected_rows ();
-
-	/* cleanup */
-	g_ptr_array_free (groups, TRUE);
-	debug_exit ("grouplist_process_selected");
-}
-
-void
-grouplist_selected_subscribe (void)
-{
-	grouplist_process_selected (1);
-}
-void
-grouplist_selected_unsubscribe (void)
-{
-	grouplist_process_selected (2);
-}
-void
-grouplist_selected_empty (void)
-{
-	grouplist_process_selected (5);
-}
-void
-grouplist_selected_properties (void)
-{
-	grouplist_process_selected (6);
-}
-
-void
-grouplist_selected_update_count_info (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_update_count_info");
-
-       	a = grouplist_get_selected_groups ();
-	if (a->len)
-		queue_add (TASK(task_group_count_new (
-			(Group**)a->pdata, a->len)));
-	else
-		g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_selected_update_count_info");
-}
-
-void
-grouplist_selected_mark_read (void)
-{
-	GPtrArray * a = grouplist_get_selected_groups ();
-	guint i;
-	debug_enter ("grouplist_selected_mark_read");
-
-        for (i=0; i<a->len; ++i)
-	{
-		Group * g = GROUP(g_ptr_array_index(a,i));
-		group_mark_all_read (g, TRUE);
-	}
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_selected_mark_read");
-}
-
-/*****
-******
-*****/
-
-static void
-delete_groups_dialog_cb (gint reply, gpointer data)
-{
-	GPtrArray * a;
-	debug_enter ("delete_groups_dialog_cb");
-
-	a = (GPtrArray*) data;
-	if (reply == 0 /*yes*/)
-	{
-		guint i;
-		GtkCList * clist = GTK_CLIST(Pan.group_clist);
-
-		pan_lock ();
-		gtk_clist_freeze (clist);
-		pan_unlock ();
-
-		for (i=0; i!=a->len; ++i)
-		{
-			Group * g = GROUP(g_ptr_array_index(a,i));
-			server_destroy_group (g->server, g);
-		}
-
-		pan_lock ();
-		gtk_clist_thaw (clist);
-		pan_unlock ();
-	}
-
-	g_ptr_array_free (a, TRUE);
-	debug_exit ("delete_groups_dialog_cb");
-}
-
-void
-grouplist_selected_destroy (void)
-{
-	GPtrArray * a = grouplist_get_selected_groups ();
-	if (a->len != 0)
-	{
-		gchar * txt = g_strdup_printf (_("Are you sure you want to delete these %d groups/folders and their articles?"), (gint)a->len);
-		pan_lock ();
-		gnome_question_dialog_parented (txt,
-		                                delete_groups_dialog_cb,
-		                                a,
-		                                GTK_WINDOW(Pan.window));
-		pan_unlock ();
-		g_free (txt);
-	}
-	else g_ptr_array_free (a, TRUE);
-}
-
-/*****
-******
-*****/
-
-/**
-***  Process all the commands to download
-***  [new | all | sample] [articles | headers]
-**/
-
-static void
-grouplist_start_task_headers (GPtrArray * a,
-                              HeaderDownloadType type,
-                              gboolean bodies)
-{
-	guint i;
-	debug_enter ("grouplist_start_task_headers");
-
-	/* sample all the selected groups */
-	for (i=0; i!=a->len; ++i)
-	{
-		Group * group = GROUP(g_ptr_array_index(a,i));
-		PanObject * o = bodies
-			? task_headers_new_with_bodies (group, type)
-			: task_headers_new (group, type);
-		queue_add (TASK(o));
-	}
-
-	/* refresh the ui */
-	grouplist_update_selected_rows ();
-
-	debug_exit ("grouplist_start_task_headers");
-}
-
-void
-grouplist_selected_download_dialog (void)
-{
-	guint i;
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_download_dialog");
-
-	/* sample all the selected groups */
-       	a = grouplist_get_selected_groups ();
-	for (i=0; i!=a->len; ++i) {
-		Group * group = GROUP(g_ptr_array_index(a,i));
-		if (!group_is_folder(group))
-			dialog_download_headers (group);
-	}
-
-	/* update ui */
-	grouplist_update_selected_rows ();
-
-	/* cleanup */
-	g_ptr_array_free (a, TRUE);
-	debug_exit ("grouplist_selected_download_dialog");
-}
-
-void
-grouplist_selected_download_sample (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_download_sample");
-
-       	a = grouplist_get_selected_groups ();
-	grouplist_start_task_headers (a, HEADERS_SAMPLE, FALSE);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_selected_download_sample");
-}
-void
-grouplist_selected_download_sample_bodies (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_download_sample_bodies");
-
-       	a = grouplist_get_selected_groups ();
-	grouplist_start_task_headers (a, HEADERS_SAMPLE, TRUE);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_selected_download_sample_bodies");
-}
-void
-grouplist_selected_download_all (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_download_all");
-
-       	a = grouplist_get_selected_groups ();
-	grouplist_start_task_headers (a, HEADERS_ALL, FALSE);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_selected_download_all");
-}
-void
-grouplist_selected_download_all_bodies (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_download_all_bodies");
-
-       	a = grouplist_get_selected_groups ();
-	grouplist_start_task_headers (a, HEADERS_ALL, TRUE);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_selected_download_all_bodies");
-}
-void
-grouplist_selected_download_new (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_download_new");
-
-       	a = grouplist_get_selected_groups ();
-	grouplist_start_task_headers (a, HEADERS_NEW, FALSE);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_selected_download_new");
-}
-void
-grouplist_selected_download_new_bodies (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_download_new_bodies");
-
-       	a = grouplist_get_selected_groups ();
-	grouplist_start_task_headers (a, HEADERS_NEW, TRUE);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_selected_download_new_bodies");
-}
-void
-grouplist_subscribed_download_new (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_subscribed_download_new");
-
-	a = server_get_groups (my_server, SERVER_GROUPS_SUBSCRIBED);
-	grouplist_start_task_headers (a, HEADERS_NEW, FALSE);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_subscribed_download_new");
-}
-void
-grouplist_subscribed_download_new_bodies (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_subscribed_download_new_bodies");
-
-	a = server_get_groups (my_server, SERVER_GROUPS_SUBSCRIBED);
-	grouplist_start_task_headers (a, HEADERS_NEW, TRUE);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("grouplist_subscribed_download_new_bodies");
-}
 
 /*****
 ******
@@ -806,24 +409,28 @@ grouplist_select_all (void)
 	pan_unlock();
 }
 
+void
+grouplist_deselect_all (void)
+{
+	pan_lock ();
+	gtk_clist_unselect_all (GTK_CLIST(Pan.group_clist));
+	pan_unlock ();
+}
+
 /*****
 ******
 *****/
 
 static void
-grouplist_set_view_mode (GtkWidget *widget, gpointer data)
+grouplist_set_view_mode_nolock_impl (gint view_mode)
 {
 	GtkWidget *menu = NULL;
-	debug_enter ("grouplist_set_view_mode");
+	debug_enter ("grouplist_set_view_mode_nolock_impl");
 
-	group_mode = GPOINTER_TO_INT (data);
-	grouplist_update_ui (grouplist_get_visible_server());
-
-        pan_lock ();
         menu = gtk_option_menu_get_menu (GTK_OPTION_MENU(grouplist_mode_menu));
         gtk_object_ref (GTK_OBJECT(menu));
         gtk_option_menu_remove_menu (GTK_OPTION_MENU (grouplist_mode_menu));
-	switch (group_mode) {
+	switch (group_mode = view_mode) {
 		case GROUP_ALL:
 			gtk_menu_set_active (GTK_MENU (menu), 0);
 			break;
@@ -840,9 +447,15 @@ grouplist_set_view_mode (GtkWidget *widget, gpointer data)
 	}
         gtk_option_menu_set_menu (GTK_OPTION_MENU(grouplist_mode_menu), menu);
         gtk_object_unref (GTK_OBJECT(menu));
-        pan_unlock ();
 
-	debug_exit ("grouplist_set_view_mode");
+	debug_exit ("grouplist_set_view_mode_nolock_impl");
+}
+
+static void
+grouplist_set_view_mode_nolock (GtkWidget *widget, gpointer data)
+{
+	grouplist_set_view_mode_nolock_impl (GPOINTER_TO_INT(data));
+	grouplist_refresh_nolock ();
 }
 
 typedef struct
@@ -876,7 +489,7 @@ grouplist_mode_menu_create (void)
 		gtk_widget_show (item);
 		gtk_menu_append (GTK_MENU (menu), item);
 		gtk_signal_connect (GTK_OBJECT(item), "activate",
-				    (GtkSignalFunc)grouplist_set_view_mode,
+				    (GtkSignalFunc)grouplist_set_view_mode_nolock,
 				    GINT_TO_POINTER(foo[i].mode));
 		if (group_mode == foo[i].mode)
 			index = i;
@@ -893,28 +506,23 @@ grouplist_mode_menu_create (void)
 ******  CALLBACKS
 *****/
 
-static gint
+static void
 grouplist_server_groups_added_cb (gpointer server, gpointer groups, gpointer foo)
 {
 	Server * visible;
-
 	debug_enter ("grouplist_server_groups_added_cb");
 
 	visible = grouplist_get_visible_server ();
 	if (visible!=NULL && visible==SERVER(server))
-		grouplist_update_ui (visible);
+		grouplist_refresh_nolock ();
 
 	debug_exit ("grouplist_server_groups_added_cb");
-	return 0;
 }
 
-static gint
-grouplist_server_groups_removed_cb (gpointer server,
-                                    gpointer groups,
-                                    gpointer foo)
+static void
+grouplist_server_groups_removed_cb (gpointer server, gpointer groups, gpointer foo)
 {
 	Server * visible;
-
 	debug_enter ("grouplist_server_groups_removed_cb");
 
 	visible = grouplist_get_visible_server ();
@@ -924,23 +532,27 @@ grouplist_server_groups_removed_cb (gpointer server,
 	}
 
 	debug_exit ("grouplist_server_groups_removed_cb");
-	return 0;
 }
 
-static gint
-grouplist_group_changed_cb (gpointer group_data, gpointer unused1, gpointer unused2)
+static void
+grouplist_groups_changed_cb (gpointer gp_groups, gpointer gp_qty, gpointer unused2)
 {
-	const Group * group = GROUP(group_data);
-	if (group->server == grouplist_get_visible_server())
-		grouplist_update_groups (&group, 1);
-	return 0;
+	gint qty = GPOINTER_TO_INT(gp_qty);
+	const Group ** groups = (const Group**) gp_groups;
+
+	g_return_if_fail (qty>0);
+	g_return_if_fail (groups!=NULL);
+
+	if (groups[0]->server == grouplist_get_visible_server())
+		grouplist_update_groups (groups, qty);
 }
 
-static gint
+static void
 grouplist_server_activated_cb (gpointer server_data, gpointer unused1, gpointer unused2)
 {
-	grouplist_set_server (SERVER(server_data));
-	return 0;
+	pan_lock ();
+	grouplist_set_server_nolock (SERVER(server_data));
+	pan_unlock ();
 }
 
 /*****
@@ -953,8 +565,8 @@ grouplist_shutdown_module (void)
 	/* stop listening... */
 	pan_callback_remove (serverlist_get_server_activated_callback(),
 	                     grouplist_server_activated_cb, NULL);
-	pan_callback_remove (group_get_group_changed_callback(),
-	                     grouplist_group_changed_cb, NULL);
+	pan_callback_remove (group_get_groups_changed_callback(),
+	                     grouplist_groups_changed_cb, NULL);
 	pan_callback_remove (grouplist_group_selection_changed,
 	                     grouplist_selection_changed_cb, NULL);
 	pan_callback_remove (server_get_groups_added_callback(),
@@ -962,9 +574,6 @@ grouplist_shutdown_module (void)
 	pan_callback_remove (server_get_groups_removed_callback(),
 	                     grouplist_server_groups_removed_cb, NULL);
 
-	/* save state for next time */
-	gnome_config_set_int ("/Pan/State/group_mode", group_mode);
-
 	/* remember server so that we can default to it next time */
 	if (my_server!=NULL && is_nonempty_string(my_server->name))
 		gnome_config_set_string ("/Pan/State/Server", my_server->name);
@@ -996,8 +605,8 @@ grouplist_create (void)
 	/* register callbacks */
 	pan_callback_add (serverlist_get_server_activated_callback(),
 	                  grouplist_server_activated_cb, NULL);
-	pan_callback_add (group_get_group_changed_callback(),
-	                  grouplist_group_changed_cb, NULL);
+	pan_callback_add (group_get_groups_changed_callback(),
+	                  grouplist_groups_changed_cb, NULL);
 	pan_callback_add (grouplist_group_selection_changed,
 	                  grouplist_selection_changed_cb, NULL);
 	pan_callback_add (server_get_groups_added_callback(),
@@ -1011,13 +620,6 @@ grouplist_create (void)
 	gtk_container_set_border_width (GTK_CONTAINER(vbox), GNOME_PAD_SMALL); 
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 
-	group_mode = gnome_config_get_int ("/Pan/State/group_mode=1");
-	if ( group_mode!=GROUP_ALL
-	     && group_mode!=GROUP_SUBSCRIBED
-	     && group_mode!=GROUP_NEW
-	     && group_mode!=GROUP_FOLDERS)
-		group_mode = GROUP_ALL;
-	
 	Pan.group_clist = gtk_clist_new_with_titles (5, titles);
 	clist = GTK_CLIST(Pan.group_clist);
 	clist->button_actions[1] = clist->button_actions[0];
@@ -1036,11 +638,12 @@ grouplist_create (void)
 	w = gtk_label_new (_("Find:"));
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
-	groupname_filter_entry = w = gtk_entry_new ();
-	gtk_signal_connect (GTK_OBJECT (w), "activate",
-			    GTK_SIGNAL_FUNC(grouplist_filter_changed_cb), w);
-	gtk_signal_connect (GTK_OBJECT (w), "focus-out-event",
-			    GTK_SIGNAL_FUNC(grouplist_filter_changed_cb), w);
+	groupname_filter_entry = w = gnome_entry_new ("Grouplist Toolbar Filter");
+	gnome_entry_set_max_saved (GNOME_ENTRY(w), 10);
+	gtk_signal_connect (GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(w))), "activate",
+			    GTK_SIGNAL_FUNC(grouplist_filter_changed_cb_nolock), w);
+	gtk_signal_connect (GTK_OBJECT(GTK_COMBO(w)->popwin), "button_press_event",
+			    GTK_SIGNAL_FUNC(grouplist_filter_changed_cb_nolock), w);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
 		_("Type in a group search string and press ENTER.  "
 		  "Wildcards are allowed."), "");
@@ -1082,38 +685,37 @@ grouplist_create (void)
 	gtk_clist_set_column_width (clist, 3, 33);
 	gtk_clist_set_column_width (clist, 4, 350);
 
-/* JEL: this works well in paned mode, but not so well in notebook mode, we
-   need to find a solution that will cover both ... maybe splitting the justification
-   parts into the _construct funcs for both ...
-	gtk_clist_set_column_justification (clist, 0, GTK_JUSTIFY_LEFT );
-	gtk_clist_set_column_justification (clist, 1, GTK_JUSTIFY_RIGHT );
-	gtk_clist_set_column_justification (clist, 2, GTK_JUSTIFY_RIGHT );
-*/
 	gtk_clist_set_selection_mode (clist, GTK_SELECTION_EXTENDED);
 
 	/* Create the Popup Menu while we are here. */
 	group_clist_menu = gnome_popup_menu_new (group_menu);	
-	folder_clist_menu = gnome_popup_menu_new (folder_menu);	
 
 	/* Lets have the Pixmap ready too */
 	folder_pixmap = (GnomePixmap*) gnome_pixmap_new_from_xpm_d (folder_xpm);
 	subscribed_pixmap = GNOME_PIXMAP(gnome_stock_pixmap_widget_new (Pan.window, GNOME_STOCK_MENU_BOOK_RED));
 
 	/* bootstrap */
-	if (1) {
-		my_server = serverlist_get_active_server ();
-		grouplist_update_ui_thread (NULL);
-	}
+	grouplist_set_server_nolock (serverlist_get_active_server());
 
 	gtk_widget_show_all (vbox);
 	debug_exit ("grouplist_create");
 	return vbox;
 }
 
+static gchar * _last_filter_str = NULL;
+
 static void
-grouplist_filter_changed_cb (void)
+grouplist_filter_changed_cb_nolock (void)
 {
-	grouplist_refresh_filter ();
+	GtkWidget * e = gnome_entry_gtk_entry(GNOME_ENTRY(groupname_filter_entry));
+	gchar * pch = gtk_editable_get_chars (GTK_EDITABLE(e), 0, -1);
+
+	if (!pan_strcmp (pch, _last_filter_str))
+		g_free (pch);
+	else {
+		replace_gstr (&_last_filter_str, pch);
+		grouplist_refresh_filter_nolock ();
+	}
 }
 
 /*****
@@ -1127,20 +729,20 @@ grouplist_add_group_nolock (const Group * group)
 	GnomePixmap * pixmap;
 	gchar total_buf[16] = { '\0' };
 	gchar unread_buf[16] = { '\0' };
+	gchar name_buf[256] = { '\0' };
 	const gint total = group->article_qty;
 	const gint unread = MAX(0, total - group->article_read_qty);
 	gchar * text[5];
-	gchar * name;
 
 	/* commatize the totals */
 	commatize_ulong (total, total_buf);
 	commatize_ulong (unread, unread_buf);
 
 	/* look for special qualities of the group */
-	name = grouplist_get_group_readable_name (group);
+	grouplist_get_group_readable_name (group, name_buf, sizeof(name_buf));
        
 	text[0] = "";
-	text[1] = name;
+	text[1] = name_buf;
 	text[2] = unread_buf;
 	text[3] = total_buf;
 	text[4] = (gchar*)(group->description?group->description:"");
@@ -1159,27 +761,42 @@ grouplist_add_group_nolock (const Group * group)
 		gtk_clist_set_pixmap (GTK_CLIST(Pan.group_clist), row, 0,
 		                      pixmap->pixmap,
 		                      pixmap->mask);
-
-	/* cleanup */
-	g_free (name);
-}
-
-static gchar*
-grouplist_update_describe (const StatusItem *item)
-{
-	return g_strdup (_("Updating Group List"));
 }
 
 static void
-grouplist_update_ui_thread (void * unused)
+grouplist_rebuild_nolock (GPtrArray * groups, GPtrArray * sel)
+{
+	gint i;
+	GtkCList * clist = GTK_CLIST (Pan.group_clist);
+
+	gtk_clist_freeze (clist);
+	gtk_clist_clear (clist);
+
+	for (i=groups->len-1; i>=0; --i)
+	{
+		Group * group = GROUP(g_ptr_array_index (groups, i));
+
+		if (is_nonempty_string(groupname_filter_string) && fnmatch(groupname_filter_string,group->name,PAN_CASEFOLD))
+			continue;
+
+		grouplist_add_group_nolock (group);
+
+	}
+
+	if (sel!=NULL && sel->len!=0)
+		grouplist_set_selected_groups_nolock (sel);
+
+	gtk_clist_thaw (clist);
+}
+
+static gint
+grouplist_update_ui_mainthread (gpointer unused)
 {
 	gchar * title;
 	GtkCList * clist = GTK_CLIST(Pan.group_clist);
 	GPtrArray * groups;
 	GPtrArray * sel;
-	StatusItem *item = NULL;
-	int i;
-	debug_enter ("grouplist_update_ui_thread");
+	debug_enter ("grouplist_update_ui_mainthread");
 
 	/**
 	***  Get the list of groups that we're going to add.
@@ -1233,11 +850,8 @@ grouplist_update_ui_thread (void * unused)
 	pan_callback_add (server_get_groups_added_callback(),
 	                  grouplist_server_groups_added_cb, NULL);
 
-        /* create a new status item to get sort/thread messages */
-	item = STATUS_ITEM(status_item_new(grouplist_update_describe));
-	status_item_set_active (item, TRUE);
-	status_item_emit_init_steps (item, groups->len);
-	status_item_emit_status (item, _("Updating Group List"));
+	/* get the list of selected groups */
+       	sel = grouplist_get_selected_groups ();
 
 	/* set the groups column title */
 	switch (group_mode) {
@@ -1248,65 +862,120 @@ grouplist_update_ui_thread (void * unused)
 		default:               title = _("Bug in Code");        break;
 	}
 
-       	sel = grouplist_get_selected_groups ();
-
-	/* update the list */
+	/* update UI */
+	pan_lock_unconditional ();
 	g_static_mutex_lock (&grouplist_ui_mutex);
-	pan_lock ();
-	gtk_clist_freeze (clist);
-	gtk_clist_clear (clist);
 	gtk_clist_set_column_title (clist, 1, title);
-
-	/* add the groups/folders */
-	for (i=groups->len-1; i>=0; --i)
-	{
-		Group * group = GROUP(g_ptr_array_index (groups, i));
-
-		if (!(i%512)) {
-			pan_unlock ();
-			status_item_emit_set_step (item, groups->len-i);
-			pan_lock ();
-		}
-
-		if (is_nonempty_string(groupname_filter_string) && fnmatch(groupname_filter_string,group->name,PAN_CASEFOLD))
-			continue;
-
-		grouplist_add_group_nolock (group);
-
-	}
-	gtk_clist_thaw (clist);
-	pan_unlock();
+	grouplist_rebuild_nolock (groups, sel);
 	g_static_mutex_unlock (&grouplist_ui_mutex);
-
-	/* update the selection */
-	grouplist_set_selected_groups (sel);
-
-	/* clean out the status item */
-	status_item_set_active (item, FALSE);
-	pan_object_unref(PAN_OBJECT(item));
+	pan_unlock_unconditional ();
 
 	/* cleanup */
 	g_ptr_array_free (sel, TRUE);
 	g_ptr_array_free (groups, TRUE);
-	debug_exit ("grouplist_update_ui_thread");
+	debug_exit ("grouplist_update_ui_mainthread");
+	return 0;
 }
 
 void
-grouplist_update_ui (Server * server)
+grouplist_refresh_nolock (void)
 {
-	debug_enter ("grouplist_update_ui");
+	Server * server = grouplist_get_visible_server ();
 
 	if (server!=NULL && server==grouplist_get_visible_server())
-	{
-		pthread_t thread;
+		run_in_main_thread_nolock (grouplist_update_ui_mainthread, NULL);
+}
 
-		pthread_create (&thread, NULL, (void*)grouplist_update_ui_thread, NULL);
-		pthread_detach (thread);
+/* firing this callback is delegated to here because you should never fire
+ * a callback inside a pan_lock. */
+static void*
+fire_server_changed_callback (void * unused)
+{
+	pan_callback_call (grouplist_server_set_callback, my_server, NULL);
+	return NULL;
+}
+
+static void
+grouplist_set_server_nolock (Server * server)
+{
+	Server * old_server;
+	debug_enter ("grouplist_set_server_nolock");
+
+	/* find out what server we have, and what kind of groups it has */
+	old_server = my_server;
+	my_server = server;
+	run_in_worker_thread (fire_server_changed_callback, NULL);
+
+	if (my_server==NULL)
+	{
+		if (grouplist_get_visible_server()==old_server)
+			gtk_clist_clear (GTK_CLIST(Pan.group_clist));
+	}
+	else
+	{
+		gint view_mode;
+		GPtrArray * groups;
+		const ServerGroupsType have_type = file_grouplist_exists (my_server);
+
+		/* if we have subbed, always load those, otherwise load all */
+		if (have_type & SERVER_GROUPS_SUBSCRIBED)
+		{
+			view_mode = GROUP_SUBSCRIBED;
+
+			groups = my_server==NULL
+				? g_ptr_array_new ()
+				: server_get_groups (my_server, SERVER_GROUPS_SUBSCRIBED);
+		}
+		else if (have_type)
+		{
+			view_mode = GROUP_ALL;
+
+			groups = my_server==NULL
+				? g_ptr_array_new ()
+				: server_get_groups (my_server, SERVER_GROUPS_ALL);
+		}
+		else
+		{
+			gchar * s;
+			GtkWidget * w;
+
+			view_mode = GROUP_ALL;
+			groups = NULL;
+
+			if (grouplist_get_visible_server()==old_server)
+				gtk_clist_clear (GTK_CLIST(Pan.group_clist));
+
+			s = g_strdup_printf (
+				_("We don't have a group list for server \"%s\".\nShall we get one?"),
+				server->name);
+			w = gnome_message_box_new (s, GNOME_MESSAGE_BOX_QUESTION,
+						      GNOME_STOCK_BUTTON_YES,
+						      GNOME_STOCK_BUTTON_NO, NULL);
+			gnome_dialog_set_default (GNOME_DIALOG(w), 0);
+			gnome_dialog_set_parent (GNOME_DIALOG(w),
+						 GTK_WINDOW(Pan.window));
+			gtk_signal_connect (GTK_OBJECT(w), "clicked",
+					    GTK_SIGNAL_FUNC(group_dialog_clicked_cb),
+					    server);
+			gtk_widget_show_all (w);
+
+			g_free (s);
+		}
+
+		/* update the UI */
+		grouplist_set_view_mode_nolock_impl (view_mode);
+		if (groups != NULL)
+			grouplist_rebuild_nolock (groups, NULL);
+
+		/* cleanup */
+		if (groups != NULL)
+			g_ptr_array_free (groups, TRUE);
 	}
 
-	debug_exit ("grouplist_update_ui");
+	debug_exit ("grouplist_set_server_nolock");
 }
 
+
 /***
 ****
 ****
@@ -1331,8 +1000,7 @@ grouplist_update_group_nolock (const Group * group)
        	row = gtk_clist_find_row_from_data (list,(gpointer)group);
 	if (row != -1)
 	{
-		gchar buf[16];
-		gchar * name;
+		gchar buf[256] = { '\0' };
 		const gint total = group->article_qty;
 		const gint unread = MAX(0, total-group->article_read_qty);
 
@@ -1346,9 +1014,8 @@ grouplist_update_group_nolock (const Group * group)
 					    row, 0, "");
 
 		/* name column */
-		name = grouplist_get_group_readable_name (group);
-		gtk_clist_set_text (list, row, 1, name);
-		g_free (name);
+		grouplist_get_group_readable_name (group, buf, sizeof(buf));
+		gtk_clist_set_text (list, row, 1, buf);
 
 		/* total column */
 		commatize_ulong (total, buf);
@@ -1361,12 +1028,12 @@ grouplist_update_group_nolock (const Group * group)
 }
 
 static gint
-grouplist_update_groups_idle (gpointer data)
+grouplist_update_groups_mainthread (gpointer data)
 {
 	guint i;
 	GPtrArray * a = (GPtrArray*)data;
 	GtkCList * list = GTK_CLIST (Pan.group_clist);
-	debug_enter ("grouplist_update_groups_idle");
+	debug_enter ("grouplist_update_groups_mainthread");
 
 	g_static_mutex_lock (&grouplist_ui_mutex);
 	pan_lock_unconditional ();
@@ -1379,11 +1046,11 @@ grouplist_update_groups_idle (gpointer data)
 
 	/* cleanup */
 	g_ptr_array_free (a, TRUE);
-	debug_exit ("grouplist_update_groups_idle");
+	debug_exit ("grouplist_update_groups_mainthread");
 	return 0;
 }
 
-void
+static void
 grouplist_update_groups (const Group ** groups, int group_qty)
 {
 	gint i;
@@ -1404,7 +1071,7 @@ grouplist_update_groups (const Group ** groups, int group_qty)
 	/* if we have any that we should update, queue it */
 	if (a->len != 0) {
 		pan_lock ();
-		gtk_idle_add (grouplist_update_groups_idle, a);
+		run_in_main_thread_nolock (grouplist_update_groups_mainthread, a);
 		pan_unlock ();
 	} else {
 		g_ptr_array_free (a, TRUE);
@@ -1474,7 +1141,7 @@ grouplist_next_unread_group (gboolean activate)
 		}
 
 		/* walk through */
-		for ( i=0; i<row_qty; ++i, row=++row%row_qty)
+		for ( i=0; i<row_qty; ++i, row=(row+1)%row_qty)
 			if (grouplist_select_row_if_unread (row, activate, 0))
 				break;
 	}
@@ -1534,67 +1201,11 @@ grouplist_activate_next_group (void)
 static void
 group_menu_popup (GdkEventButton *event)
 {
-	gboolean selected = grouplist_get_selected_group() != NULL;
-	GList * l = NULL;
 	debug_enter ("group_menu_popup");
 
-	if (group_mode == GROUP_FOLDERS) /* We are dealing with folders */
+	/* We must be dealing with real groups */
 	{
 		pan_lock ();
-		l=GTK_MENU_SHELL(folder_clist_menu)->children;
-		/* l is new folder */
-		l=l->next; /* delete folder */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-
-		/* show the menu */
-		gtk_menu_popup (GTK_MENU(folder_clist_menu),
-			NULL, NULL, NULL, NULL,
-			event->button, event->time);
-		pan_unlock ();
-	}
-	else /* We must be dealing with real groups */
-	{
-		pan_lock ();
-
-		l=GTK_MENU_SHELL(group_clist_menu)->children;  /* download */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* l is separator */
-		l=l->next; /* l is get new headers in subscribed */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), TRUE);
-		l=l->next; /* l is get new headers in selected */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* l is get all headers in selected */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* l is sample for selected */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* separator */
-		l=l->next; /* l is get new headers & bodies in subscribed */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), TRUE);
-		l=l->next; /* l is get new headers & bodies in selected */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* l is get all headers & bodies in selected */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* l is sample new headers & bodies in selected */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* separator */
-		l=l->next; /* refresh article count */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* catch up */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-
-		l=l->next; /* separator */
-		l=l->next; /* subscribe */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* unsubscribe */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-
-		l=l->next; /* separator */
-		l=l->next; /* destroy group(s) */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* empty group */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* properties */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
 
 		/* show the menu */
 		gtk_menu_popup (GTK_MENU(group_clist_menu),
@@ -1607,14 +1218,12 @@ group_menu_popup (GdkEventButton *event)
 }
 
 static void
-grouplist_refresh_filter (void)
+grouplist_refresh_filter_nolock (void)
 {
 	gchar * new_filter;
         gchar * pch;
 
-	pan_lock ();
-	pch  = gtk_editable_get_chars (GTK_EDITABLE(groupname_filter_entry), 0, -1);
-	pan_unlock ();
+	pch  = gtk_editable_get_chars (GTK_EDITABLE(gnome_entry_gtk_entry(GNOME_ENTRY(groupname_filter_entry))), 0, -1);
 
 	/* build the new filter string */
 	new_filter = NULL;
@@ -1628,7 +1237,7 @@ grouplist_refresh_filter (void)
 	/* if it differs from the old filter string, refresh */
 	if (pan_strcmp (new_filter, groupname_filter_string)) {
 		replace_gstr (&groupname_filter_string, g_strdup(new_filter));
-		grouplist_update_ui (grouplist_get_visible_server());
+		grouplist_refresh_nolock ();
 	}
 
 	/* cleanup */
