@@ -240,6 +240,14 @@ gui_headers_set_nolock (GtkWidget         * table,
 		add_header_nolock (table, row++, _("Followup-To: "), pch, ',', max_header_width);
 	}
 
+	/* if newsreader header found, display it */
+	if ((header_fields & UI_HEADER_NEWSREADER)
+		&& is_nonempty_string (pch = article_get_newsreader(article)))
+	{
+		add_header_nolock (table, row++, _("Newsreader:"), pch, ' ', max_header_width);
+		g_free (pch);
+	}
+
 	if (header_fields & UI_HEADER_SUBJECT)
 	{
 		add_header_nolock (table, row++, _("Subject: "), article_get_subject(article), ' ', max_header_width);
