@@ -20,9 +20,10 @@
 
 #include <config.h>
 
-#include <gnome.h>
+#include <libgnomeui/libgnomeui.h>
 
 #include <pan/base/debug.h>
+#include <pan/base/pan-i18n.h>
 
 #include <pan/articlelist.h>
 #include <pan/globals.h>
@@ -35,33 +36,6 @@
 
 GtkWidget * notebook = NULL;
 
-
-/*---[ gui_notebook_select_all ]--------------------------------------
- * Called from gui_select_all(), this determines the page you are in
- * and performs the appropriate select_all function
- *--------------------------------------------------------------------*/
-void
-gui_notebook_select_all (void)
-{
-	switch (gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))
-	{
-	case 0: /* groups tab */
-		grouplist_select_all ();
-		break;
-
-	case 1: /* articles tab */
-		articlelist_select_all ();
-		break;
-
-	case 2: /* message tab */
-		text_select_all ();
-		break;
-
-	default:
-		break;
-	}
-}
-
 /*---[ gui_notebook_construct ]---------------------------------------
  * Build and pack the notebook GUI layout into Pan's main vbox
  *--------------------------------------------------------------------*/
@@ -82,13 +56,16 @@ gui_notebook_construct (void)
 
 	/* ref so that we don't lose the three panes */
 	gtk_widget_ref (groups_vbox);
-	gtk_container_remove (GTK_CONTAINER(groups_vbox->parent), groups_vbox);
+	if (groups_vbox->parent != NULL)
+		gtk_container_remove (GTK_CONTAINER(groups_vbox->parent), groups_vbox);
 	gtk_widget_unrealize (groups_vbox);
 	gtk_widget_ref (articlelist_ctree);
-	gtk_container_remove (GTK_CONTAINER(articlelist_ctree->parent), articlelist_ctree);
+	if (articlelist_ctree->parent != NULL)
+		gtk_container_remove (GTK_CONTAINER(articlelist_ctree->parent), articlelist_ctree);
 	gtk_widget_unrealize (articlelist_ctree);
 	gtk_widget_ref (text_box);
-	gtk_container_remove (GTK_CONTAINER(text_box->parent), text_box);
+	if (text_box->parent != NULL)
+		gtk_container_remove (GTK_CONTAINER(text_box->parent), text_box);
 	gtk_widget_unrealize (text_box);
 
 	/* build a new UI */
@@ -116,17 +93,18 @@ gui_notebook_construct (void)
 	debug_exit ("gui_notebook_construct");
 }
 
-/**
-***
-**/
-
-void gui_notebook_page_set (int page, GtkWidget * focus_item)
+void
+gui_notebook_page_set (int page, GtkWidget * focus_item)
 {
-	if (Pan.viewmode == GUI_NOTEBOOK)
-	{
-		pan_lock();
-		gtk_notebook_set_page (GTK_NOTEBOOK (notebook), page);
-		gtk_widget_grab_focus (focus_item);
-		pan_unlock();
-	}
+	pan_lock();
+	gtk_notebook_set_page (GTK_NOTEBOOK (notebook), page);
+	gtk_widget_grab_focus (focus_item);
+	pan_unlock();
 }
+
+int
+gui_notebook_get_current_pane (void)
+{
+	return gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+}
+
