@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,17 +21,8 @@
 #ifndef __GUI_NOTEBOOK_H__
 #define __GUI_NOTEBOOK_H__
 
-/***
- *** global variables
- ***/
-extern GtkWidget *notebook;
-
-/***
- *** function prototypes
- ***/
 void gui_notebook_construct  (void);
-void gui_notebook_select_all (void);
+int  gui_notebook_get_current_pane (void);
 void gui_notebook_page_set (int page, GtkWidget * focus_item);
 
-
 #endif /* __GUI_NOTEBOOK_H__ */
