@@ -21,11 +21,8 @@
 #ifndef __GUI_PANED_H__
 #define __GUI_PANED_H__
 
-
-int  gui_paned_get_current_pane (void);
-void gui_paned_construct	(void);
-void gui_paned_enter_clicked	(void);
-void gui_paned_select_all (void);
-
+int  gui_paned_get_current_pane         (void);
+void gui_paned_construct	        (void);
+void gui_paned_page_set                 (int page, GtkWidget * focus_item);
 
 #endif /* __GUI_PANED_H__ */
