@@ -53,7 +53,7 @@ void gui_set_layout(int);
 
 void gui_construct (const gchar *geometry);
 
-void gui_page_set (int page, GtkWidget *focus_item);
+void gui_page_set (int page);
 
 void gui_restore_column_widths    (GtkWidget * clist, const gchar * key);
 
@@ -65,7 +65,7 @@ void gui_save_window_size         (GtkWidget * window, const gchar * key);
 
 void gui_set_connecting_flag (gboolean is_connecting);
 void gui_inc_connection_size (int inc);
-void gui_set_queue_size (int running, int total);
+void gui_set_queue_size (guint running, guint total);
 
 extern GnomeUIInfo group_menu [];
 extern GnomeUIInfo folder_menu [];
