@@ -329,6 +329,7 @@ task_send_describe (const StatusItem * item)
 	const gboolean posting = data->do_post;
 	const gboolean mailing = data->do_mail;
 	const gchar * fmt;
+
 	if (posting && mailing)
 		fmt = _("Posting and Mailing Article ``%s''");
 	else if (posting)
@@ -337,6 +338,7 @@ task_send_describe (const StatusItem * item)
 		fmt = _("Mailing Article ``%s''");
 	else 
 		fmt = _("I have no idea what I'm doing with Article ``%s''");
+
 	return g_strdup_printf (fmt, article_get_subject(article));
 }
 
@@ -350,6 +352,8 @@ task_send_dtor (gpointer data)
        	task = (TaskFuncPtr*) data;
        	send = (SendStruct*) task->user_data;
 	group_unref_articles (send->original_group, NULL);
+	group_unref_articles (serverlist_get_named_folder(PAN_SENT), NULL);
+	memset (send, 0, sizeof(SendStruct));
 	g_free (send);
 
 	debug_exit ("task_send_dtor");
@@ -394,11 +398,15 @@ task_send_run (TaskFuncPtr* task, gpointer user_data)
 	else /* successful; dispose of the article object. */
 	{
 		Group * sent;
+		Article * copy;
 
-		/* save to "Sent".  Note that the call to group_add_article
-		   will also remove the article from pan.sendlater */
+		/* save to "sent".*/
 		sent = serverlist_get_named_folder (PAN_SENT);
-		group_add_article (sent, article);
+		copy = article_dup (sent, article);
+		group_add_article (sent, copy);
+
+		/* remove from "sendlater" */
+		group_remove_article (article->group, article);
 	}
 
 	/* if we failed, we make it hopeless because we've saved
@@ -535,6 +543,7 @@ queue_article_for_posting (Server     * server,
 		   be valid when the task runs.  The accompanying unref is
 		   in task_send_dtor... */
 		group_ref_articles (article->group, NULL);
+		group_ref_articles (serverlist_get_named_folder(PAN_SENT), NULL);
 
 		data->article = article;
 		data->original_group = article->group;
