@@ -34,6 +34,7 @@
 #include <pan/base/log.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
+#include <pan/base/util-mime.h>
 
 #include <pan/nntp.h>
 #include <pan/prefs.h>
@@ -442,9 +443,14 @@ init_header_if_match (Article       * a,
 	len = strlen (header_key);
 	if (!strncmp (header, header_key, len))
 	{
-		gchar * b = g_mime_utils_8bit_header_decode (header+len);
-		article_init_header (a, article_key, b, action);
-		g_free (b);
+		const gchar * val = header + len;
+		if (!pan_header_is_8bit_encoded (val))
+			article_init_header (a, article_key, val, action);
+		else {
+			gchar * b = g_mime_utils_8bit_header_decode ((const guchar*)val);
+			article_init_header (a, article_key, b, action);
+			g_free (b);
+		}
 		match = TRUE;
 	}
 
@@ -529,7 +535,7 @@ nntp_article_download (StatusItem      * status,
 
 		/* rfc 977: 2.4.1 */
 		if (line->len>=2 && line->str[0]=='.' && line->str[1]=='.')
-			g_string_erase (line, 0, 2);
+			g_string_erase (line, 0, 1);
 
 		/* save the line */
 		g_string_append (buf, line->str);
@@ -649,6 +655,7 @@ nntp_download_headers (StatusItem       * status,
 	gboolean read_status = 0;
 	gint result;
 	gint val;
+	GString * buf;
 	debug_enter ("nntp_download_headers");
 
 	/* sanity checks */
@@ -699,6 +706,7 @@ nntp_download_headers (StatusItem       * status,
 	***  Walk through all the articles headers that the server spits back
 	**/
 
+	buf = g_string_new (NULL);
 	pan_g_ptr_array_reserve (setme, setme->len + (high-low));
 	while (!*abort && !((read_status = pan_socket_getline (sock, &buffer))))
 	{
@@ -707,8 +715,6 @@ nntp_download_headers (StatusItem       * status,
 		const gchar * pch;
 		const gchar * s;
 		Article * a;
-		gchar * buf = NULL;
-		gchar * dec = NULL;
 		const gchar * march = NULL;
 		const gchar * tok = NULL;
 		int tok_len = 0;
@@ -719,32 +725,35 @@ nntp_download_headers (StatusItem       * status,
 
 		/* create the article data */
 		a = article_new (group);
-		group_set_articles_dirty (group);
 
 		/* setup for parsing */
-		buf = g_strdup (buffer);
-		march = buf;
+		g_string_assign (buf, buffer);
+		march = buf->str;
 
 		/* get article number */
 		a->number = get_next_token_ulong (march, '\t', &march);
 
 		/* get subject */
-		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((gchar*)tok)[tok_len] = '\0';
-			dec = g_mime_utils_8bit_header_decode (tok);
-			article_init_header (a, HEADER_SUBJECT, dec, DO_CHUNK_SHARE);
-			replace_gstr (&dec, NULL);
+			if (!pan_header_is_8bit_encoded (tok))
+				article_init_header (a, HEADER_SUBJECT, tok, DO_CHUNK_SHARE);
+			else {
+				gchar * dec = g_mime_utils_8bit_header_decode ((const guchar*)tok);
+				article_init_header (a, HEADER_SUBJECT, dec, DO_CHUNK_SHARE);
+				g_free (dec);
+			}
 		}
 
 		/* get author */
-		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((gchar*)tok)[tok_len] = '\0';
 			if (is_nonempty_string(tok))
 				article_init_author_from_header (a, tok);
 		}
 
 		/* get date */
-		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((gchar*)tok)[tok_len] = '\0';
 			a->date = is_nonempty_string(tok)
 				? g_mime_utils_header_decode_date (tok, NULL)
@@ -752,14 +761,14 @@ nntp_download_headers (StatusItem       * status,
 		}
 
 		/* get message id */
-		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((gchar*)tok)[tok_len] = '\0';
 			if (is_nonempty_string(tok) && *tok=='<')
 				article_init_header (a, HEADER_MESSAGE_ID, tok, DO_CHUNK);
 		}
 
 		/* get references */
-		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			if (tok_len>0 ) {
 				((gchar*)tok)[tok_len] = '\0';
 				if (is_nonempty_string(tok) && *tok=='<')
@@ -774,7 +783,7 @@ nntp_download_headers (StatusItem       * status,
 		a->linecount = get_next_token_int (march, '\t', &march);
 
 		/* get crossref */
-		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((gchar*)tok)[tok_len] = '\0';
 			if (tok_len!=0 || strncmp(tok,"Xref: ", 6))
 				a->crosspost_qty = 1;
@@ -809,11 +818,11 @@ nntp_download_headers (StatusItem       * status,
 		{
 			/* find the ']' of [n/N] */
 			--pch;
-			if ((pch[1]!=')' && pch[1]!=']') || !isdigit((int)*pch))
+			if ((pch[1]!=')' && pch[1]!=']') || !isdigit((guchar)*pch))
 				continue;
 
 			/* find the '/' of [n/N] */
-			while (s!=pch && isdigit((int)*pch))
+			while (s!=pch && isdigit((guchar)*pch))
 				--pch;
 			if (s==pch || (*pch!='/' && *pch!='|'))
 				continue;
@@ -823,7 +832,7 @@ nntp_download_headers (StatusItem       * status,
 			--pch;
 
 			/* find the '[' of [n/N] */
-			while (s!=pch && isdigit((int)*pch))
+			while (s!=pch && isdigit((guchar)*pch))
 				--pch;
 			if (s==pch || (*pch!='(' && *pch!='[')) {
 				parts = 0;
@@ -869,10 +878,8 @@ nntp_download_headers (StatusItem       * status,
 		status_item_emit_next_step (status);
 		if (!(setme->len % 57))
 			status_item_emit_status_va (status, progress_fmt, setme->len, (high-low));
-
-		/* cleanup */
-		g_free (buf);
 	}
+	g_string_free (buf, TRUE);
 
 	if (read_status!=0)
 		status_item_emit_error (status, sockread_err_msg);
@@ -997,7 +1004,7 @@ nntp_cancel (StatusItem     * status,
 	/* build the message to post */
 	msg = g_string_new (NULL);
 	author = article_get_author_str(article);
-        replace_gstr (&author, g_mime_utils_8bit_header_encode(author));
+        replace_gstr (&author, g_mime_utils_8bit_header_encode((const guchar*)author));
 	g_string_sprintfa (msg, "From: %s\r\n", author);
 	g_string_sprintfa (msg, "Newsgroups: %s\r\n", newsgroups);
 	g_string_sprintfa (msg, "Subject: cancel %s\r\n", msg_id);
