@@ -53,8 +53,6 @@ static gint save_session (GnomeClient        * client,
 static void session_die  (GnomeClient        * client,
                           gpointer             client_data);
 
-const gchar* PAN_SENT;
-const gchar* PAN_SENDLATER;
 const gchar* sockread_err_msg = NULL;
 const gchar* sockwrite_err_msg = NULL;
 
