@@ -27,9 +27,9 @@
 
 typedef enum
 {
-	ONLINE_PROMPT,
-	ONLINE_YES,
-	ONLINE_NO
+	ONLINE_PROMPT	= 0,
+	ONLINE_YES	= 1,
+	ONLINE_NO	= 2
 }
 GoingOnlinePreference;
 
@@ -45,6 +45,14 @@ extern ListClickAction         grouplist_mb1_action;
 extern ListClickAction         grouplist_mb2_action;
 extern ListClickAction         articlelist_mb1_action;
 extern ListClickAction         articlelist_mb2_action;
+extern gboolean                header_pane_is_threaded;
+extern gboolean                show_group_pane;
+extern gboolean                show_header_pane;
+extern gboolean                show_article_pane;
+extern gboolean                download_bodies_too;
+extern gboolean                navigate_cached_only;
+extern gboolean                navigate_read_on_select;
+extern gboolean                text_use_fixed_font;
 extern gboolean                text_window_smooth_scrolling;
 extern gint                    text_window_smooth_scrolling_speed;
 extern gboolean                fetch_new_on_group_load;
@@ -59,9 +67,8 @@ extern gchar                 * grouplist_font;
 extern gchar                 * thread_normal_font;
 extern gchar                 * thread_new_replies_font;
 extern gchar                 * message_body_font;
-extern gchar                 * attribution_line;
+extern gchar                 * message_body_font_fixed;
 extern gchar                 * external_web_browser;
-extern gchar                 * external_mailer;
 extern gchar                 * external_editor;
 extern gint                    watched_thread_span_days;
 extern gchar                 * extra_posting_headers;
@@ -70,6 +77,9 @@ extern gchar                 * layout_str;
 extern gint                    mail_server_port;
 extern gint                    wrap_column;
 extern gboolean                wrap_body_on_reply;
+extern gboolean                collapse_group_names;
+extern gboolean                use_system_fg;
+extern gboolean                use_system_bg;
 
 extern gchar *                 newsrc_filename;
 extern gboolean                newsrc_do_port;
