@@ -32,6 +32,7 @@
 
 #include <libgnomeui/libgnomeui.h>
 
+#include <pan/base/argset.h>
 #include <pan/base/debug.h>
 #include <pan/base/base-prefs.h>
 #include <pan/base/pan-i18n.h>
@@ -351,7 +352,7 @@ apply_now_describe (const StatusItem * item)
 	return g_strdup (_("Applying Rule to Selected Groups"));
 }
 
-static void
+static void*
 apply_now_thread (void * data)
 {
 	StatusItem * status;
@@ -376,7 +377,7 @@ apply_now_thread (void * data)
 		status_item_emit_status_va (status, _("Loading group `%s'"),
 			group_get_readable_name(g));
 		group_ref_articles (g, NULL);
-		group_thread_if_needed (g, NULL);
+		group_thread_if_needed (g);
 
 		status_item_emit_status_va (status, _("Applying rule `%s' to Group `%s'"),
 			rule->name, group_get_readable_name(g));
@@ -406,6 +407,7 @@ apply_now_thread (void * data)
 	/* cleanup memory */
 	g_ptr_array_free (groups, TRUE);
 	g_free (data);
+	return NULL;
 }
 
 
@@ -421,14 +423,12 @@ apply_rules_impl (RuleListDialog * d, GPtrArray * groups)
 	if (rule != NULL)
 	{
 		ApplyNow * now;
-		pthread_t thread;
 
 		now = g_new (ApplyNow, 1);
 		now->groups = pan_g_ptr_array_dup (groups);
 		now->rule = rule;
 
-		pthread_create (&thread, NULL, (void*)apply_now_thread, now);
-		pthread_detach (thread);
+		run_in_worker_thread (apply_now_thread, now);
 	}
 }
 
